<?php

/**
 * This class is the basetype of all extensions within the BlueSpice framework.
 *
 * @copyright Copyright (c) 2007-2009, HalloWelt! Medienwerkstatt GmbH, All rights reserved.
 * @author Sebastian Ulbricht, Robert Vogel
 * @version 0.1.0 beta
 *
 * $LastChangedDate: 2012-11-09 12:50:13 +0100 (Fr, 09. Nov 2012) $
 * $LastChangedBy: smuggli $
 * $Rev: 7228 $
 * $Id: Extension.class.php 7228 2012-11-09 11:50:13Z smuggli $
 */

/**
 * This class is the basetype of all extensions within the BlueSpice framework.
 */
// Last Review: MRG20100813
// TODO MRG20100813: Die Einrückungen stimmen nicht so richtig.
abstract class BsExtension {

	protected $mExtensionPath   = NULL;
	protected $mExtensionType   = NULL;

	protected $mInfo            = NULL;
	protected $mExtensionKey    = NULL;
	/**
	 *
	 * @var BsAdapter
	 */
	public    $mAdapter         = NULL;

	protected $aStandardContext = array('*', '*', '*');

	public abstract function __construct();

	/**
	 * Initializes the extension.
	 */
	public abstract function setup();

	// TODO MRG20100813: Wozu ist diese Funktion da?
	// @todo SU 20100927; Diese Methode dient dem Initialisieren der Extensions.
	// Wir können sie aber nicht abstract machen, weil sie dann in jeder Extension definiert werden müsste,
	// auch wenn wir sie garnicht brauchen.
	protected function initExt() {}

	/**
	 * Save a reference to current adapter instance.
	 * @param BsAdapter $adapter
	 */
	public function setAdapter( $adapter ) {
		$this->mAdapter = $adapter;
	}

	/**
	 *
	 * @param string $sExtensionDirectory
	 * @param string $sScriptFileName
	 * @param boolean $bLoadOnDemand
	 * @param boolean $bNeedExtJs
	 * @param boolean $bAvoidLanguageFile
	 */
	public function registerScriptFiles( $sExtensionDirectory, $sScriptFileName, $bLoadOnDemand = true, $bNeedExtJs = false, $bAvoidLanguageFile = false, $sContext = false ) {
		wfProfileIn( 'BS::'.__METHOD__ );
		if ( !$sContext ) {
			$sContext = $this->mExtensionKey;
			BsExtensionManager::setContext($this->mExtensionKey);
		}
		if ( $bNeedExtJs ) {
			BsScriptManager::addNeedsExtJs( $sContext );
			//BsCore::loadExtJs();
			//BsExtensionManager::addScriptFileToContext( $sContext, $this->mInfo[EXTINFO::NAME], 'Ext', BsFileManager::ORDER_IF_LOADED );
		}
		$options = ( $bLoadOnDemand ) ? BsFileManager::LOAD_ON_DEMAND : 0;

		BsExtensionManager::addScriptFileToContext( $sContext, $this->mInfo[EXTINFO::NAME], $sExtensionDirectory, $sScriptFileName, $options, !$bAvoidLanguageFile );
		wfProfileOut( 'BS::'.__METHOD__ );
	}

	/**
	 *
	 * @param string $sStyleSheetPath
	 * @param boolean $bLoadOnDemand
	 * @param string $sContext
	 */
	public function registerStyleSheet($sStyleSheetPath, $bLoadOnDemand = true, $sContext = false) {
		wfProfileIn( 'BS::'.__METHOD__ );
		if ( !$sContext ) {
			$sContext = $this->mExtensionKey;
			BsExtensionManager::setContext($this->mExtensionKey);
		}
		$options = ($bLoadOnDemand) ? BsFileManager::LOAD_ON_DEMAND : 0;

		BsExtensionManager::addStyleSheetToContext($sContext, $this->mInfo[EXTINFO::NAME], $sStyleSheetPath, $options);
		wfProfileOut( 'BS::'.__METHOD__ );
	}

	/**
	 * returns the extension informations as an array
	 * @return array
	 */
	public function getInfo() {
		return $this->mInfo;
	}

	// TODO MRG (01.09.10 01:57): Kommentar
	public function getName() {
		return $this->mInfo[EXTINFO::NAME];
	}

	// TODO MRG (01.09.10 01:57): Kommentar
	public function getExtensionKey() {
		return $this->mExtensionKey;
	}

	// TODO MRG (19.12.10 19:28): Was macht denn das?
	public function runPreferencePlugin( $sAdapterName, $oVariable ) {
		switch($sAdapterName) {
			case 'MW':
			case 'MW115':
			case 'CORE':
				return array();
		}
	}

	// TODO MRG (01.09.10 23:05): Kommentar
	public function registerView( $viewName ) {
		wfProfileIn( 'BS::'.__METHOD__ );
		$innerViewFile = str_replace( "View", "", $viewName );
		BsCore::registerClass( $viewName, dirname( $this->mExtensionFile ).DS.'views', 'view.'.$innerViewFile.'.php' );
		wfProfileOut( 'BS::'.__METHOD__ );
	}

}