<?php
/**
 * Processes search requests for ExtendedSearch
 *
 * Part of BlueSpice for MediaWiki
 *
 * @author     Mathias Scheer <scheer@hallowelt.biz>
 * @version    $Id: SearchRequest.class.php 7685 2012-12-11 19:05:29Z smuggli $
 * @package    BlueSpice_Core
 * @subpackage ExtendedSearch
 * @copyright  Copyright (C) 2010 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 * @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
 * @filesource
 */
/* Changelog
 * v0.1
 * FIRST CHANGES
 */
/**
 * Processes search request for ExtendedSearch
 * @package BlueSpice_Core
 * @subpackage ExtendedSearch
 */
class BsSearchRequest {

	/**
	 * Constructor of BsSearchRequest class
	 * @param string $sOrigin Key of origin of search request, e.g. 'ajax'
	 */
	public function __construct( $sOrigin ) {
		wfProfileIn( 'BS::'.__METHOD__ );
		$this->sRequestOrigin = $sOrigin;
		$this->setDefaults();
		$this->processSettings();
		$this->processInputs();
		wfProfileOut( 'BS::'.__METHOD__ );
	}

	/*
	 * Todo: Any member should be initialized. No detail can be overseen this way. To be implemented.
	 */
	/**
	 * Sets defaults for some internal variables
	 */
	protected function setDefaults() {
		$this->sFormat = 'html';
	}

	/**
	 * Fetches values for properties from settings
	 */
	protected function processSettings() {
		wfProfileIn( 'BS::'.__METHOD__ );
		$this->sHighlightSnippets = BsConfig::get( 'Core::Search::HighlightSnippets' );
		$this->bLogUsers          = BsConfig::get( 'Core::Search::LogUsers' );
		$this->bLogging           = BsConfig::get( 'Core::Search::Logging' );
		$this->iMaxDocSize        = ( BsConfig::get( 'Core::Search::MaxDocSizeMB', 10 ) * 1024 * 1024 );
		wfProfileOut( 'BS::'.__METHOD__ );
	}

	/**
	 * Fetches values for properties from URL params
	 */
	protected function processInputs() {
		wfProfileIn( 'BS::'.__METHOD__ );
		$this->sSubmit        = BsCore::getParam( 'search_submit' );
		$this->sGo            = BsCore::getParam( 'search_go' );
		$this->sInput         = BsCore::getParam( 'search_input', false, BsPARAM::REQUEST|BsPARAMTYPE::RAW );
		$this->sHidden        = BsCore::getParam( 'search_hidden' );
		$this->sRequestOrigin = BsCore::getParam( 'search_origin' );
		$this->sEditor        = BsCore::getParam( 'ed', array(), BsPARAMTYPE::ARRAY_STRING | BsPARAM::REQUEST );
		$this->sFormat        = BsCore::getParam( 'search_format', $this->sFormat );
		wfProfileOut( 'BS::'.__METHOD__ );
	}

}
