<?php

/**
 * Xml extension for BlueSpice
 *
 * Functions that faciliate working with xml
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * This file is part of BlueSpice for MediaWiki
 * For further information visit http://www.blue-spice.org
 *
 * @author     Mathias Scheer <scheer@hallowelt.biz>
 * @version    0.0.1 stable
 * @version    $Id: Xml.class.php 4129 2012-02-20 08:36:08Z swidmann $
 * @copyright  Copyright (C) 2011 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 */

/* Changelog
  * v0.1
  * - initial release provides static function simpleTag
  */

class BsXml {

	/**
	 * A simple tag is <type attr1="val1" attr2="val2" ... attrN="valN">body</type>
	 * This method creates one and automatically htmlspecialchars the values of the attributes
	 * @param string $type e.g. 'a', 'br', 'div', 'span' ... anything possible
	 * @param array $attributes syntax array( 'attr1' => 'val1', ..., 'attrN' => 'valN'). Leave the values unencoded EXCEPT URLENCODING (for e.g. href)
	 * @param string $body to put between the tags
	 * @param bool if body is empty allow rendering in only one tag (<div /> instead of <div></div>)
	 * @return string
	 */
	public static function simpleTag( $type, $attributes = array(), $body = '', $allowShortTag = true ) {
		wfProfileIn( 'BS::'.__METHOD__ );
		$attributeString = '';
		foreach ( $attributes as $name => $value ) {
			$valueEncoded = htmlspecialchars( $value, ENT_QUOTES, 'UTF-8', false ); // intentionally no ENT_COMPAT. no double encoding.
			$valueEncoded = str_replace(
					array( "\n", "\r", "\t" ),
					array( '&#10;', '&#13;', '&#9;' ),
					$valueEncoded
			);
			$attributeString .= " {$name}=\"{$valueEncoded}\"";
		}
		if ( empty( $body ) ) {
			if ( $allowShortTag ) return "<{$type}{$attributeString} />";
			$body = '';
		}
		wfProfileOut( 'BS::'.__METHOD__ );
		return "<{$type}{$attributeString}>{$body}</{$type}>";
	}

//	public static function indentedSimpleTag( $type, $body = '', $attributes = array( ), $allowShortTag = true ) {
//		if (!empty($body))
//			$body = "\n\t".str_replace("\n", "\n\t", $body)."\n";
//		return self::simpleTag( $type, $body, $attributes, $allowShortTag );
//	}

}
