<?php

abstract class BsExtensionMW extends BsExtension {

	protected static $aExtensionInstances = array();

	public static function getInstanceFor( $sExtensionKey ) {
		return isset( BsExtensionMW::$aExtensionInstances[$sExtensionKey] ) ? BsExtensionMW::$aExtensionInstances[$sExtensionKey] : false;
		// TODO MRG (29.01.11 23:58): false oder null? als Programmierrichtlinien festlegen
		// TODO RBV (19.05.11 08:49): Ich bin für null. Weil wir ja eigendlich ein Objekt erwarten. Und null ist einem Objekt zumindest ähnlicher als false.
	}

	/**
	 * Initializes the extension.
	 */
	public function setup() {
		global $wgExtensionCredits;
		// Extension credits that will show up on Special:Version
		$wgExtensionCredits[$this->mExtensionType][] = array(
			'path'        => $this->mExtensionPath,
			'name'        => $this->mInfo[EXTINFO::NAME],
			'version'     => str_replace( "$", "", $this->mInfo[EXTINFO::VERSION] ) . ' (' . $this->mInfo[EXTINFO::STATUS] . ')',
			'author'      => $this->mInfo[EXTINFO::AUTHOR],
			'url'         => $this->mInfo[EXTINFO::URL],
			'description' => $this->mInfo[EXTINFO::DESCRIPTION]
		);
		$this->initExt();
		BsExtensionMW::$aExtensionInstances[$this->mExtensionKey] = &$this;
	}

	/**
	 * register hooks
	 * @example $this->setHook('ParserFirstCallInit'); // register the method onParserFirstCallInit() to  hook ParserFirstCallInit
	 * @example $this->setHook('ParserFirstCallInit', 'initParser'); // register the method initParser() to  hook ParserFirstCallInit
	 * @global array $wgHooks
	 * @param string $hook name of the hook to register to
	 * @param string $method (optional) name of method register to
	 * @param bool $bExecuteFirst set this method to be first to be executed when hook triggered
	 */
	public function setHook( $hook, $method = false, $bExecuteFirst = false ) {
		global $wgHooks;
		// handle $method === 'on'.$hook as if $method == false
		$register = ( $method && ( $method !== 'on' . $hook ) ) ? array( &$this, $method ) : $this;
		// do not set same hook twice
		if ( isset( $wgHooks ) && isset( $wgHooks[$hook] ) 
			&& is_array( $wgHooks[$hook] ) && in_array( $register, $wgHooks[$hook] ) )
			return;
		if ( $bExecuteFirst && isset( $wgHooks[$hook] ) ) {
			array_unshift( $wgHooks[$hook], $register );
		} else {
			$wgHooks[$hook][] = & $register;
		}
	}

	/**
	 * returns standard image path for extensions
	 * @return string
	 */
	public function getImagePath() {
		// TODO: ScriptPath should be more abstract.
		// TODO: Bluespice-mw should be abstraced
		// CR MRG (30.06.11 10:23): Lokal cachen
		return BsConfig::get( 'MW::ScriptPath' ) . '/bluespice-mw/ext/' . $this->mInfo[EXTINFO::NAME] . '/images/';
	}

	/**
	 * If your BlueSpice extension for MediaWiki is in need of a certain
	 * database table just register it's schema file here. Next time update.php
	 * is run from command line the table will automagically be created!
	 * See http://www.mediawiki.org/wiki/Manual:Hooks/LoadExtensionSchemaUpdates for details.
	 * @param string $sSchemaFileName Full path + filename to (.sql) file with create statement
	 */
	protected $aExtensionSchemes = array();
	protected $bExtensionSchemaHookRegistered = false;

	public function registerExtensionSchemaUpdate( $sDatabaseTableName, $sSchemaFileName ) { 
		// TODO RBV (01.03.11 11:26): Change method to support different update types (i.e. modification)
		$aBaseSettings = BsConfig::get( 'Core::Database' );
		if ( isset( $aBaseSettings['core']['type'] ) && $aBaseSettings['core']['type'] == 'PostGreSQL' ) {
			$sSchemaFileName = substr( $sSchemaFileName, 0, -4 ).'.pg.sql';
		}
		else if ( isset( $aBaseSettings['core']['type'] ) && $aBaseSettings['core']['type'] == 'OracleDB' ) {
			$sSchemaFileName = substr( $sSchemaFileName, 0, -4 ).'.oci.sql';
		}
		$this->aExtensionSchemes[$sDatabaseTableName] = $sSchemaFileName;
		// register mediawiki hook only once
		if ( $this->bExtensionSchemaHookRegistered )
			return;
		global $wgHooks;
		$wgHooks['LoadExtensionSchemaUpdates'][] = array( $this, 'onLoadExtensionSchemaUpdates' );
		$this->bExtensionSchemaHookRegistered = true;
	}

	/**
	 * This method gets called by the MediaWiki Framework
	 * @param DatabaseUpdater $updater Provided by MediaWikis update.php
	 * @return boolean Always true to keep the hook running
	 */
	public function onLoadExtensionSchemaUpdates( $updater ) {
		$aNewTables = & BsCore::getInstance( 'MW' )->getAdapter()->ExtNewTables;
		foreach ( $this->aExtensionSchemes as $sDatabaseTableName => $sSchemaFileName ) {
			$aNewTables[] = array( $sDatabaseTableName, $sSchemaFileName );
		}
		return true;
	}

}
