<?php
/**
 * ArticleInfo extension for BlueSpice
 *
 * Provides information about an article for status bar.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * This file is part of BlueSpice for MediaWiki
 * For further information visit http://www.blue-spice.org
 *
 * @author     Markus Glaser <glaser@hallowelt.biz>
 * @version    1.20.0
 * @version    $Id: ArticleInfo.class.php 9690 2013-06-12 10:13:02Z pwirth $
 * @package    BlueSpice_Extensions
 * @subpackage ArticleInfo
 * @copyright  Copyright (C) 2011 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 * @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
 * @filesource
 */

/* Changelog
 * v1.20.0
 *
 * v1.1.0
 * - Added subpages info (new FD)
 * v1.0.0
 * - Stable state
 * - Added events
 * v0.3.0b
 * - Renewed StatusBar architecture
 * - Modified views
 * - Date is now clickable and links to the articles historypage
 * v0.2.0b
 * - Added views
 * v0.1.0b
 * FIRST CHANGES
*/

/**
 * Base class for ArticleInfo extension
 * @package BlueSpice_Extensions
 * @subpackage ArticleInfo
 */

class ArticleInfo extends BsExtensionMW {

	/**
	 * Contructor of the ArticleInfo class
	 */
	public function __construct() {
		wfProfileIn( 'BS::'.__METHOD__ );
		global $wgExtensionMessagesFiles;
		$wgExtensionMessagesFiles['ArticleInfo'] = dirname( __FILE__ ) . '/ArticleInfo.i18n.php';

		$this->mExtensionFile = __FILE__;
		$this->mExtensionType = EXTTYPE::OTHER; //SPECIALPAGE/OTHER/VARIABLE/PARSERHOOK
		$this->mInfo = array(
			EXTINFO::NAME        => 'ArticleInfo',
			EXTINFO::DESCRIPTION => 'Provides information about an article for status bar.',
			EXTINFO::AUTHOR      => 'Markus Glaser',
			EXTINFO::VERSION     => '1.21.0 ($Rev: 9690 $)',
			EXTINFO::STATUS      => 'stable',
			EXTINFO::URL         => 'http://www.hallowelt.biz',
			EXTINFO::DEPS        => array(
										'bluespice' => '1.21.0',
										'MW::StateBar' => '0.1.0b'
										)
		);
		$this->mExtensionKey = 'MW::ArticleInfo';
		wfProfileOut( 'BS::'.__METHOD__ );
	}

	/**
	 * Initialization of ArticleInfo extension
	 */
	public function  initExt() {
		wfProfileIn( 'BS::'.__METHOD__ );
		BsConfig::registerVar( 'MW::ArticleInfo::ImageLastEdited', 'bs-infobar-last-edited.png', BsConfig::LEVEL_PRIVATE|BsConfig::TYPE_STRING, 'bs-articleinfo-ImageLastEdited' );
		BsConfig::registerVar( 'MW::ArticleInfo::ImageLastEditor', 'bs-infobar-author.png',      BsConfig::LEVEL_PRIVATE|BsConfig::TYPE_STRING, 'bs-articleinfo-ImageLastEditor' );
		BsConfig::registerVar( 'MW::ArticleInfo::ImageCategories', 'bs-infobar-category.png',    BsConfig::LEVEL_PRIVATE|BsConfig::TYPE_STRING, 'bs-articleinfo-ImageCategories' );
		BsConfig::registerVar( 'MW::ArticleInfo::ImageSubpages',   'bs-infobar-subpages.png',    BsConfig::LEVEL_PRIVATE|BsConfig::TYPE_STRING, 'bs-articleinfo-ImageSubpages' );

		$this->mAdapter->registerBehaviorSwitch( 'NOARTICLEINFO', array( $this, 'noArticleInfoCallback' ) );

		// TODO RBV (05.10.10 11:32): Find better hook. MW Database Abstaction has to be loaded.
		$this->setHook( 'BSBlueSpiceSkinBeforeArticleHeadline', 'onBlueSpiceSkin_BeforeArticleHeadline' ); // STM (26.04.12 15.32) better?

		$this->setHook( 'BSStateBarAddSortTopVars', 'onStatebarAddSortTopVars' );
		$this->setHook( 'BSStateBarAddSortBodyVars', 'onStatebarAddSortBodyVars' );
		
		$this->setHook( 'ArticleSaveComplete' );
		$this->setHook( 'ArticleDeleteComplete' );

		$this->setHook( 'BSBlueSpiceSkinBeforePrintFooterLinks' );

		$this->registerView( 'ViewStateBarTopElementCategoryShortList' );

		$this->registerScriptFiles( BsConfig::get( 'MW::ScriptPath' ).'/bluespice-mw/ext/ArticleInfo/js', 'ArticleInfo', false, false, false, 'MW::StateBarShow' );

		wfProfileOut( 'BS::'.__METHOD__ );
	}

	public function noArticleInfoCallback() {
		BsExtensionManager::setContext( 'MW::ArticleInfo::Hide' );
	}

	/**
	 * Hook-Handler for Hook 'BSStatebarAddSortTopVars'
	 * @param array $aSortTopVars
	 * @return boolean Always true to keep hook running
	 */
	public function onStatebarAddSortTopVars( &$aSortTopVars ) {
		$aSortTopVars['statebartoplastedited'] = wfMsg( 'bs-articleinfo-statebartoplastedited' );
		$aSortTopVars['statebartoplasteditor'] = wfMsg( 'bs-articleinfo-statebartoplasteditor' );
		$aSortTopVars['statebartopcategories'] = wfMsg( 'bs-articleinfo-statebartopcategories' );
		$aSortTopVars['statebartopsubpages']   = wfMsg( 'bs-articleinfo-statebartopsubpages' );
		return true;
	}

	/**
	 * Hook-Handler for Hook 'BSStatebarAddSortBodyVars'
	 * @param array $aSortBodyVars
	 * @return boolean Always true to keep hook running
	 */
	public function onStatebarAddSortBodyVars( &$aSortBodyVars ) {
		$aSortBodyVars['statebarbodycategories']  = wfMsg( 'bs-articleinfo-statebarbodycategories' );
		$aSortBodyVars['statebarbodyeditsummary'] = wfMsg( 'bs-articleinfo-statebarbodyeditsummary' );
		$aSortBodyVars['statebarbodysubpages']    = wfMsg( 'bs-articleinfo-statebarbodysubpages' );
		return true;
	}

	/**
	 * Hook-Handler for 'BlueSpiceSkin:NavigationTop'. Adds various StateBarElements to the StateBar.
	 * @param array $aViews
	 * @param QuickTemplate $oQuickTemplate
	 * @return bool Always true to keep hook running.
	 */
	public function onBlueSpiceSkin_BeforeArticleHeadline( &$aViews, $wgUser, $oQuickTemplate ) {
		if ( BsAdapterMW::isSpecial() || BsCore::getParam( 'action', 'view', BsPARAM::REQUEST | BsPARAMTYPE::STRING | BsPARAMOPTION::DEFAULT_ON_ERROR ) != 'view' ) return true;
		if ( BsExtensionManager::isContextActive( 'MW::ArticleInfo::Hide' ) ) return true;
		if ( !BsExtensionManager::isContextActive( 'MW::StateBarShow') ) return true;

		$oTitle   = $this->mAdapter->get( 'Title' );
		$oRequest = $this->mAdapter->get( 'Request' );
		if( $oTitle->isRedirect() && $oRequest->getVal('redirect') != 'no' ) {
			//TODO: Use DB Query? Or WikiPage::getRedirectTarget() in later versions.
			$oArticle = new Article( $oTitle, 0 ); //New: current revision
			$sContent = $oArticle->fetchContent( 0 ); //Old: current revision
			$oTitle = Title::newFromRedirectRecurse( $sContent );
		}
		if ( $oTitle == null || $oTitle->exists() == false ) return true;
		wfProfileIn( 'BS::'.__METHOD__ );

		$this->makeStateBarTopLastEdited( $oTitle );
		$this->makeStateBarTopLastEditor( $oTitle );
		$this->makeStateBarTopCategories( $oTitle );
		$this->makeStateBarBodyEditSummary( $oTitle );
		$this->makeStateBarSubPages( $oTitle );

		wfProfileOut( 'BS::'.__METHOD__ );
		return true;
	}

	/**
	 * Hook-Handler for BlueSpice hook BSBlueSpiceSkinBeforePrintFooterLinks - Remove footer links
	 * @param Array $aFooterLinks
	 * @param Object $sSkinTemplate - Current skin template (default: BlueSpiceTemplate)
	 * @param String $sOption
	 * @return boolean - always true
	 */
	public function onBSBlueSpiceSkinBeforePrintFooterLinks( &$aFooterLinks, $sSkinTemplate, $sOption ) {
		$aRMLinks = array(
			'info' => array(
				'lastmod',
			),
		);

		foreach($aRMLinks as $sKey => $aLinks) {
			if( !isset($aFooterLinks[$sKey]) ) continue;
			foreach( $aFooterLinks[$sKey] as $sLnkKey => $sLink ) {
				if( !in_array($sLink, $aLinks) ) continue;
				unset($aFooterLinks[$sKey][$sLnkKey]);
			}
		}

		return true;
	}

	/**
	 * Called from ArticleInfo::onNavigationTop(). Adds "last edited" Icon to the StateBar.
	 * @return bool true
	 */
	/**
	 *
	 * @global Article $wgArticle
	 * @param Title $oTitle Current Title object. Redirects already resolved
	 * @return boolean 
	 */
	private function makeStateBarTopLastEdited( $oTitle ) {
		$oLastEditView = new ViewStateBarTopElement();
		$oArticle = Article::newFromID( $oTitle->getArticleID() );
		$iOldId = BsCore::getParam( 'oldid', 0, BsPARAM::REQUEST|BsPARAMTYPE::INT|BsPARAMOPTION::DEFAULT_ON_ERROR );
		if( $oArticle instanceof Article == false ) {
			return true;
		}
		wfProfileIn( 'BS::'.__METHOD__ );
		if ( $iOldId != 0 ) {
			$sTimestamp = Revision::getTimestampFromId( $oArticle->getTitle(), $iOldId );
		} else {
			$sTimestamp = $oArticle->getTimestamp();
		}

		$sFormattedTimestamp = BsFormatConverter::mwTimestampToAgeString( $sTimestamp, true );
		$sArticleHistoryPageLink = $oArticle->getTitle()->getLinkURL( 
			array( 
				'diff'   => 0, 
				'action' => 'historysubmit'
			)
		);

		$oLastEditView->setKey( 'LastEdited' );
		$oLastEditView->setIconSrc( $this->getImagePath().BsConfig::get( 'MW::ArticleInfo::ImageLastEdited' ) );
		$oLastEditView->setIconAlt( wfMsg( 'bs-articleinfo-last-edited' ) );
		$oLastEditView->setText( $sFormattedTimestamp );
		$oLastEditView->setTextLink( $sArticleHistoryPageLink );
		$oLastEditView->setTextLinkTitle( wfMsg( 'bs-articleinfo-last-edited-tooltip' ) );
		$oLastEditView->setDataAttribute( 'timestamp', wfTimestamp( TS_UNIX, $sTimestamp ) );

		$iPosition = 'always-10';

		wfRunHooks( 'BSArticleInfoBeforeAddLastEditView', array( $this, &$oLastEditView, &$iPosition ) );

		StateBar::addTopElement( $oLastEditView, 'statebartoplastedited' );

		wfProfileOut( 'BS::'.__METHOD__ );
		return true;
	}

	/**
	 * Called from ArticleInfo::onNavigationTop(). Adds "last editor" Icon to the StateBar.
	 * @param Title $oTitle Current Title object. Redirects already resolved
	 * @return bool true
	 */
	private function makeStateBarTopLastEditor( $oTitle ) {
		$oCurrentArticle = new Article($oTitle);
		if( !is_object( $oCurrentArticle ) || $oCurrentArticle->getUserText() == '' ) return true; //If a not existing article is viewed

		$oLastEditorView = new ViewStateBarTopElement();

		$oLastEditor = User::newFromName( $oCurrentArticle->getUserText() );
		if( is_object( $oLastEditor ) === false || $oLastEditor === null ) return true;

		wfProfileIn( 'BS::'.__METHOD__ );
		$sLastEditorName = $this->mAdapter->getUserDisplayName( $oLastEditor );
		$sLastEditorUserPageUrl = $oLastEditor->getUserPage()->getFullURL();

		$oLastEditorView->setKey( 'LastEditor' );
		$oLastEditorView->setIconSrc( $this->getImagePath().BsConfig::get('MW::ArticleInfo::ImageLastEditor') );
		$oLastEditorView->setIconAlt( wfMsg('bs-articleinfo-last-editor') );
		$oLastEditorView->setText( $sLastEditorName );
		$oLastEditorView->setTextLinkTitle( $sLastEditorName );
		$oLastEditorView->setTextLink( $sLastEditorUserPageUrl );

		$iPosition = 'always-20';

		wfRunHooks( 'BSArticleInfoBeforeAddTopElement', array( $this, &$oLastEditorView, &$iPosition ) );

		StateBar::addTopElement( $oLastEditorView, 'statebartoplasteditor' );

		wfProfileOut( 'BS::'.__METHOD__ );
		return true;
	}

	/**
	 * Called from ArticleInfo::onNavigationTop(). Adds "categories" Icon to the StateBar.
	 * @param Title $oTitle Current Title object. Redirects already resolved
	 * @return bool true
	 */
	private function makeStateBarTopCategories( $oTitle ) {
		$aCurrenPagesCategories = $oTitle->getParentCategories();
		if( empty( $aCurrenPagesCategories ) ) return false;

		wfProfileIn( 'BS::'.__METHOD__ );
		$oCategoriesLinks = new ViewStateBarTopElementCategoryShortList();
		$bIsProcessed = false;

		wfRunHooks( 'BSArticleInfoBeforeAddLastEditorView', array( $this, &$aCurrenPagesCategories , &$bIsProcessed ) );

		if( $bIsProcessed === false ){
			ksort( $aCurrenPagesCategories );
		}
		$aAllCategoriesWithUrls = array();
		$iLoopCount = 0;
		foreach( $aCurrenPagesCategories as $sPrefixedCategoryTitle => $sCurrentPageTitle ) {

			$aCategoryTitleParts = explode( ':', $sPrefixedCategoryTitle );
			$oTitle = Title::makeTitle( NS_CATEGORY, $aCategoryTitleParts[1] );

			if( $iLoopCount < 3 ) $oCategoriesLinks->addCategory( $oTitle ); //Only three for the topelement
			$aAllCategoriesWithUrls[] = $oTitle; //But all for the body element
			$iLoopCount++;
		}
		if ( count( $aAllCategoriesWithUrls ) > 3 ) {
			$oCategoriesLinks->setMoreCategoriesAvailable( true );
		}

		$aBodyCats = array();
		foreach( $aAllCategoriesWithUrls as $oCategory ){
			$sDisplayName = $oCategory->getText();
			$aBodyCats[] = BsLinkProvider::makeLinkerLink( $oCategory, $sDisplayName );
		}

		$oCategoriesLinkBodyElement = new ViewStateBarBodyElement();
		$oCategoriesLinkBodyElement->setKey( 'AllCategories' );
		$oCategoriesLinkBodyElement->setHeading( wfMsg( 'bs-articleinfo-all-categories-heading' ) );
		$oCategoriesLinkBodyElement->setBodyText( implode( '<br />', $aBodyCats ) );

		$iPosition = 'always-31';

		wfRunHooks( 'BSArticleInfoBeforeAddCategoryBodyView', array( $this, &$oCategoriesLinkBodyElement, &$iPosition ) );

		StateBar::addBodyElement( $oCategoriesLinkBodyElement, 'statebarbodycategories' );

		$oCategoriesLinks->setKey( 'Categories' );
		$oCategoriesLinks->setIconSrc( $this->getImagePath().BsConfig::get('MW::ArticleInfo::ImageCategories') );
		$oCategoriesLinks->setIconAlt( wfMsg( 'bs-articleinfo-categories' ) );

		$iPosition = 'always-30';

		wfRunHooks('BSArticleInfoBeforeAddCategoryView', array( $this, &$oCategoriesLinks, &$iPosition ));

		StateBar::addTopElement( $oCategoriesLinks, 'statebartopcategories' );

		wfProfileOut( 'BS::'.__METHOD__ );
		return true;
	}

	/**
	 * Called from ArticleInfo::onNavigationTop(). Adds "edit summary" text to the StateBar.
	 * @param Title $oTitle Current Title object. Redirects already resolved
	 * @return bool true
	 */
	private function makeStateBarBodyEditSummary( $oTitle ) {
		wfProfileIn( 'BS::'.__METHOD__ );
		$oLastEditSummaryView = new ViewStateBarBodyElement();

		$sSummary = wfMsg( 'bs-articleinfo-no-summary' );
		$oArticle = new Article( $oTitle );
		if ( $oArticle == null )
			$sSummary = wfMsg( 'bs-articleinfo-not-logged-in' );
		else if ( $oArticle->getComment() )
			$sSummary = $oArticle->getComment(); // TODO RBV (30.11.10 13:14): Would be cool to parse WikiText but there stands the problem with image links. Some images are pretty huge and won't get scaled.

		$oLastEditSummaryView->setKey( 'EditSummary' );
		$oLastEditSummaryView->setHeading( wfMsg( 'bs-articleinfo-edit-summary' ) );
		$oLastEditSummaryView->setBodyText( $sSummary );

		$iPosition = 'always-30';

		wfRunHooks( 'BSArticleInfoBeforeAddEditSummaryView', array( $this, &$oLastEditSummaryView, &$iPosition ) );

		StateBar::addBodyElement( $oLastEditSummaryView, 'statebarbodyeditsummary' );

		wfProfileOut( 'BS::'.__METHOD__ );
		return true;
	}

	/**
	 * Called from ArticleInfo::onNavigationTop(). Adds "subpages" list to the StateBar.
	 * @param Title $oTitle Current Title object. Redirects already resolved
	 * @return bool true
	 */
	private function makeStateBarSubPages( $oTitle ) {
		global $wgUser;
		if( $oTitle->hasSubpages() == false ) return true;

		wfProfileIn( 'BS::'.__METHOD__ );
		$oSubpageIcons = new ViewStateBarTopElement();
		$oSubpageIcons->setKey( 'Subpages' );
		$oSubpageIcons->setIconSrc( $this->getImagePath().BsConfig::get('MW::ArticleInfo::ImageSubpages') );
		$oSubpageIcons->setIconAlt( wfMsg( 'bs-articleinfo-subpages-available' ) );
		$oSubpageIcons->setIconTogglesBody( true );
		$oSubpageIcons->setText( wfMsg( 'bs-articleinfo-subpages' ) );
		$oSubpageIcons->setTextLinkTitle( wfMsg( 'bs-articleinfo-subpages' ) );
		$oSubpageIcons->setTextLink( '#' );

		$iPosition = 'always-40';
		wfRunHooks('BSArticleInfoBeforeSubpagesTopView', array( $this, &$oSubpageIcons, &$iPosition ));

		StateBar::addTopElement( $oSubpageIcons, 'statebartopsubpages' );

		$oSubpageListView = new ViewStateBarBodyElement();
		$oSubpageListView->setKey( 'Subpages' );
		$oSubpageListView->setHeading( wfMsg( 'bs-articleinfo-subpages' ) );

		$aSubpages = $oTitle->getSubpages( -1 );

		if( count($aSubpages) > 100 ) {
			$oSubpageListView->setBodyText( wfMsg( 'bs-articleinfo-subpages-too-much' ) );
		}
		else {
			// TODO RBV (22.02.12 10:22): Less inline CSS, more use of classes
			$oList = new ViewBaseElement();
			$oList->setAutoWrap( '<ul style="margin:0">###CONTENT###</ul>' );
			$oList->setTemplate( '<li style="{STYLE}">&rarr; {LINK}</li>' );

			foreach( $aSubpages as $oTitle ) {
				$sLink = $wgUser->getSkin()->link( $oTitle, $oTitle->getSubpageText() );
				$sStyle = 'margin-left:'.( count( explode( '/', $oTitle->getText() ) ) - 1 ).'em';
				$oList->addData( array( 'LINK' => $sLink, 'STYLE' => $sStyle ) );
			}

			$oSubpageListView->setBodyText( $oList->execute() );
		}

		$iPosition = 'always-70';
		wfRunHooks('BSArticleInfoBeforeSubpagesBodyView', array( $this, &$oSubpageListView, &$iPosition ));

		StateBar::addBodyElement( $oSubpageListView, 'statebarbodysubpages' );

		wfProfileOut( 'BS::'.__METHOD__ );
		return true;
	}
	
	/**
	 * Hook-Handler for Mediawiki hook ArticleDeleteComplete
	 * @param Article $article
	 * @param User $user
	 * @param string $text
	 * @param string $summary
	 * @param integer $minoredit
	 * @param type $watchthis
	 * @param type $sectionanchor
	 * @param integer $flags
	 * @param Revision $revision
	 * @param Status $status
	 * @param integer $baseRevId
	 * @return boolean - always true
	 */
	public function onArticleSaveComplete( &$article, &$user, $text, $summary, $minoredit, $watchthis, $sectionanchor, &$flags, $revision, &$status, $baseRevId ) {
		if( $status->value['new'] === false ) return true; 

		$oTitle = $article->getTitle();
		if( !$oTitle->isSubpage() ) return true;

		$sParentTitle = substr( $oTitle->getPrefixedText(), 0, strrpos($oTitle->getPrefixedText(), '/') );
		$oParentTitle = Title::newFromText( $sParentTitle );

		if( is_null($oParentTitle) || !$oParentTitle->exists() ) return true;

		$oParentTitle->invalidateCache();
		return true;
	}
	
	/**
	 * Hook-Handler for Mediawiki hook ArticleDeleteComplete
	 * @param Article $article
	 * @param User $user
	 * @param string $reason
	 * @param integer $id
	 * @return boolean
	 */
	public function onArticleDeleteComplete( &$article, &$user, $reason, $id ) {
		$oTitle = $article->getTitle();
		if( !$oTitle->isSubpage() ) return true;

		$sParentTitle = substr( $oTitle->getPrefixedText(), 0, strrpos($oTitle->getPrefixedText(), '/') );
		$oParentTitle = Title::newFromText( $sParentTitle );

		if( is_null($oParentTitle) || !$oParentTitle->exists() ) return true;

		$oParentTitle->invalidateCache();
		return true;
	}
}