/**
 * Js for ArticleInfo extension
 *
 * @author     Patric Wirth <wirth@hallowelt.biz>
 * @version    $Id: ArticleInfo.js 7441 2012-11-22 08:40:54Z pwirth $
 * @package    Bluespice_Extensions
 * @subpackage ArticleInfo
 * @copyright  Copyright (C) 2011 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 * @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
 * @filesource
 */

/**
 * Base class for all ArticleInfo related methods and properties
 */
BsArticleInfo = {
	i18n: {
		unitAgo: ' $1 ago',
		unitsAgo: ' $1 and $2 ago',

		second: '$1 second',
		seconds: '$1 seconds',
		minute: '$1 minute',
		minutes: '$1 minutes',
		hour: '$1 hour',
		hours: '$1 hours',
		day: '$1 day',
		days: '$1 days',
		week: '$1 week',
		weeks: '$1 weeks',
		month: '$1 month',
		months: '$1 months',
		year: '$1 year',
		years: '$1 years',
		
		now: 'Now'
	},
	lastEditInterval: 10000,
	lastEditTimestamp: undefined,
	lastEditedID: undefined,

	init: function () {
		if(BsArticleInfo.lastEditTimestamp !== undefined && BsArticleInfo.lastEditedID !== undefined) {
			BsArticleInfo.updateLastEdited();
		}
	},
	updateLastEdited: function() {
		//This is a js version of "adapter/Utility/FormatConverter.class.php" -> timestampToAgeString
		//TODO: -This should be part of BlueSpice framework (adapter)
		//		-Use ResourceLoader for i18n
		var start = (new Date(BsArticleInfo.lastEditTimestamp));
		var now = (new Date());
		var diff = now - start;
		
		var sDateTimeOut = '';
		var sYears = '';
		var sMonths = '';
		var sWeeks = '';
		var sDays = '';
		var sHrs = '';
		var sMins = '';
		var sSecs = '';

		var sTsPast =  BsArticleInfo.lastEditTimestamp;
		var sTsNow = Math.round((new Date()).getTime() / 1000);
		var iDuration = sTsNow - sTsPast;

		var iYears=Math.floor(iDuration/(60*60*24*365)); iDuration%=60*60*24*365;
		var iMonths=Math.floor(iDuration/(60*60*24*30.5)); iDuration%=60*60*24*30.5;
		var iWeeks=Math.floor(iDuration/(60*60*24*7)); iDuration%=60*60*24*7;
		var iDays=Math.floor(iDuration/(60*60*24)); iDuration%=60*60*24;
		var iHrs=Math.floor(iDuration/(60*60)); iDuration%=60*60;
		var iMins=Math.floor(iDuration/60);
		var iSecs=iDuration%60;


		if (iYears == 1) { sYears = BsArticleInfo.i18n.year.replace('$1',iYears); }
		if (iYears > 1) { sYears = BsArticleInfo.i18n.years.replace('$1',iYears); }
		
		if (iMonths == 1) { sMonths = BsArticleInfo.i18n.month.replace('$1',iMonths); }
		if (iMonths > 1) { sMonths = BsArticleInfo.i18n.months.replace('$1',iMonths); }

		if (iWeeks == 1) { sWeeks = BsArticleInfo.i18n.week.replace('$1',iWeeks); }
		if (iWeeks > 1) { sWeeks = BsArticleInfo.i18n.weeks.replace('$1',iWeeks); }

		if (iDays == 1) { sDays = BsArticleInfo.i18n.day.replace('$1',iDays); }
		if (iDays > 1) { sDays = BsArticleInfo.i18n.days.replace('$1',iDays); }

		if (iHrs == 1) { sHrs = BsArticleInfo.i18n.hour.replace('$1',iHrs); }
		if (iHrs > 1) { sHrs = BsArticleInfo.i18n.hours.replace('$1',iHrs); }

		if (iMins == 1) { sMins = BsArticleInfo.i18n.minute.replace('$1',iMins); }
		if (iMins > 1) { sMins = BsArticleInfo.i18n.minutes.replace('$1',iMins); }

		if (iSecs == 1) { sSecs = BsArticleInfo.i18n.second.replace('$1',iSecs); }
		if (iSecs > 1) { sSecs = BsArticleInfo.i18n.seconds.replace('$1',iSecs); }

		if (iYears > 0) sDateTimeOut = sMonths ? BsArticleInfo.i18n.unitsAgo.replace('$1',sYears).replace('$2',sMonths) : BsArticleInfo.i18n.unitAgo.replace('$1',sYears);
		else if (iMonths > 0) sDateTimeOut = sWeeks ? BsArticleInfo.i18n.unitsAgo.replace('$1',sMonths).replace('$2',sWeeks) : BsArticleInfo.i18n.unitAgo.replace('$1',sMonths);
		else if (iWeeks > 0) sDateTimeOut = sDays ? BsArticleInfo.i18n.unitsAgo.replace('$1',sWeeks).replace('$2',sDays) : BsArticleInfo.i18n.unitAgo.replace('$1',sWeeks)
		else if (iDays > 0) sDateTimeOut = sHrs ? BsArticleInfo.i18n.unitsAgo.replace('$1',sDays).replace('$2',sHrs) : BsArticleInfo.i18n.unitAgo.replace('$1',sDays);
		else if (iHrs > 0) sDateTimeOut = sMins ?BsArticleInfo.i18n.unitsAgo.replace('$1',sHrs).replace('$2',sMins) : BsArticleInfo.i18n.unitAgo.replace('$1',sHrs);
		else if (iMins > 0) sDateTimeOut = sSecs ? BsArticleInfo.i18n.unitsAgo.replace('$1',sMins).replace('$2',sSecs) : BsArticleInfo.i18n.unitAgo.replace('$1',sMins);
		else if (iSecs > 0) sDateTimeOut = BsArticleInfo.i18n.unitAgo.replace('$1',sSecs);
		else if (iSecs == 0) sDateTimeOut = BsArticleInfo.i18n.now;

		$('#sb-LastEdited-link').text(sDateTimeOut);
		BsArticleInfo.timeout = setTimeout("BsArticleInfo.updateLastEdited()", BsArticleInfo.lastEditInterval);
	}
}

$(document).ready(function() {
	BsArticleInfo.lastEditedID = $('#sb-LastEdited');
	BsArticleInfo.lastEditTimestamp = $('#sb-LastEdited').attr('data-timestamp');

	BsArticleInfo.init()
});