<?php
/**
 * Renders the CategoryShortList from the ArticleInfo extension.
 *
 * Part of BlueSpice for MediaWiki
 *
 * @author     Robert Vogel <vogel@hallowelt.biz>
 * @version    $Id: view.StateBarTopElementCategoryShortList.php 7583 2012-12-04 10:53:52Z smuggli $
 * @package    BlueSpice_Extensions
 * @subpackage ArticleInfo
 * @copyright  Copyright (C) 2011 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 * @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
 * @filesource
 */

/**
 * This view renders the CategoryShortList from the ArticleInfo extension.
 * @package    BlueSpice_Extensions
 * @subpackage ArticleInfo
 */
class ViewStateBarTopElementCategoryShortList extends ViewStateBarTopElement {

	/**
	 * Holds all categories as array( 'url' => '...', 'name' => 'CategoryName' ).
	 * @var array
	 */
	protected $aCategories = array();
	/**
	 * Wether to show an ellipisis or not.
	 * @var bool
	 */
	protected $bMoreCategoriesAvailable = false;

	/**
	 * Adder for a category array like array( 'url' => '...', 'name' => 'CategoryName' ).
	 * @param array $aCategory
	 */
	public function addCategory( $aCategory ) {
		$this->aCategories[] = $aCategory;
	}

	/**
	 * Setter for the internal field $bMoreCategoriesAvailable.
	 * @param bool $bMoreCategoriesAvailable The value.
	 */
	public function setMoreCategoriesAvailable( $bMoreCategoriesAvailable ) {
		$this->bMoreCategoriesAvailable = $bMoreCategoriesAvailable;
	}

	/**
	 * This method actually generates the output
	 * @param mixed $params Comes from base class definition. Not used in this implementation.
	 * @return string HTML output
	 */
	public function execute( $params = false ) {
		$aOut = array();

		$sMaxDisplayNameLength = 45 / count( $this->aCategories );

		foreach ( $this->aCategories as $oCategory ) {
			$sDisplayName = $oCategory->getText();
			if( strlen( $sDisplayName ) + 2 > $sMaxDisplayNameLength ) {
				$sDisplayName = BsStringHelper::shorten( $sDisplayName, array( 'max-length' => $sMaxDisplayNameLength, 'position' => 'middle' ) );
			}
			$aOut[] = BsLinkProvider::makeLinkerLink( $oCategory, $sDisplayName );
		}
		if( $this->bMoreCategoriesAvailable ) {
			$aOut[] = '<a onclick="$(\'#bs-statebar-view\').slideToggle( \'fast\' ); return false;" href="#" title="'.wfMsg( 'bs-articleinfo-more-categories' ).'">[...]</a>';
		}

		$this->sText = implode( ', ', $aOut );

		if ( !empty ( $this->sKey ) ) {
			return parent::execute();
		} else {
			return NULL;
		}
	}

}
