<?php
/**
 * Extended Search extension for BlueSpice
 *
 * Search plugin on Apache Solr basis
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * This file is part of BlueSpice for MediaWiki
 * For further information visit http://www.blue-spice.org
 *
 * @author     Mathias Scheer <scheer@hallowelt.biz>
 * @author     Markus Glaser <glaser@hallowelt.biz>
 * @author     Stephan Muggli <muggli@hallowelt.biz>
 * @version    1.20.1 stable
 * @version    $Id: ExtendedSearch.class.php 9690 2013-06-12 10:13:02Z pwirth $
 * @package    BlueSpice_Extensions
 * @subpackage ExtendedSearch
 * @copyright  Copyright (C) 2011 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 * @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
 * @filesource
 */
/* Changelog
 * v1.20.1
 * - Fixed indexing of zipped doctypes
 * - Layout Improvments
 * - Serveral bug fixes
 * v1.20.0
 *
 * v1.0.0
 * - raised to stable
 * v0.1
 * - initial release
 */

/**
 * Base class for ExtendedSearch extension
 * @package BlueSpice_Extensions
 * @subpackage ExtendedSearch
 */
class ExtendedSearch extends BsExtensionMW {

	/**
	 * Object of BsExtendedSearchBaseMW
	 * @var Object
	 */
	protected $oExtendedSearchBaseMW = null;

	/**
	 * Constructor of ExtendedSearch class
	 */
	public function __construct() {
		wfProfileIn( 'BS::'.__METHOD__ );
		global $wgExtensionMessagesFiles;
		$wgExtensionMessagesFiles['ExtendedSearch'] = dirname( __FILE__ ) . '/ExtendedSearch.i18n.php';

		// Base settings
		$this->mExtensionFile = __FILE__;
		$this->mExtensionType = EXTTYPE::VARIABLE; //SPECIALPAGE/OTHER/VARIABLE/PARSERHOOK
		$this->mInfo = array(
			EXTINFO::NAME => 'ExtendedSearch',
			EXTINFO::DESCRIPTION => 'Apache Solr based search plugin for BlueSpice',
			EXTINFO::AUTHOR => 'Stephan Muggli, Mathias Scheer, Markus Glaser',
			EXTINFO::VERSION => '1.21.0 ($Rev: 9690 $)',
			EXTINFO::STATUS => 'stable',
			EXTINFO::URL => 'http://www.hallowelt.biz',
			EXTINFO::DEPS => array( 'bluespice' => '1.21.0' )
		);
		$this->mExtensionKey = 'MW::ExtendedSearch';
		$this->registerExtensionSchemaUpdate( 'bs_searchstats', dirname( __FILE__ ).DS.'ExtendedSearch.sql' );

		// called with action=remote&mod=ExtendedSearch&rf=getRequestJson
		BsCore::getInstance( 'MW' )->getAdapter()->addRemoteHandler( 'ExtendedSearch', $this, 'getRequestJson', 'read' );

		WikiAdmin::registerModuleClass( 'ExtendedSearchAdmin', array(
			'image' => '/bluespice-mw/ext/WikiAdmin/images/bs-btn_suche_v1.png',
			'level' => 'editadmin'
		) );
		wfProfileOut( 'BS::'.__METHOD__ );
	}

	/**
	 * Initialization of ExtendedSearch extension
	 */
	protected function initExt() {
		wfProfileIn( 'BS::'.__METHOD__ );
		global $wgAPIModules;

		BsExtensionManager::setContext( 'MW::ExtendedSearch::Active' );

		$sDir = __DIR__;

		BsCore::registerClass( 'ExtendedSearchAdmin',            $sDir );
		BsCore::registerClass( 'ExtendedSearchBaseMW',           $sDir );
		BsCore::registerClass( 'SearchIndexMW',                  $sDir.'/lib/SearchIndex', 'SearchIndex.class.php' );
		BsCore::registerClass( 'SearchOptions',                  $sDir.'/lib/SearchIndex' );
		BsCore::registerClass( 'SearchRequestMW',                $sDir.'/lib/SearchIndex', 'SearchRequest.class.php');
		BsCore::registerClass( 'SearchUriBuilderMW',             $sDir.'/lib/SearchIndex', 'SearchUriBuilder.class.php' );
		BsCore::registerClass( 'BuildIndexMainControlMw',        $sDir.'/lib/BuildIndex' );
		BsCore::registerClass( 'BuildIndexMwArticles',           $sDir.'/lib/BuildIndex' );
		BsCore::registerClass( 'BuildIndexMwRepository',         $sDir.'/lib/BuildIndex' );
		BsCore::registerClass( 'BuildIndexMwExternalRepository', $sDir.'/lib/BuildIndex' );
		BsCore::registerClass( 'AbstractBuildIndexMwLinked',     $sDir.'/lib/BuildIndex' );
		BsCore::registerClass( 'BuildIndexMwSingleFile',         $sDir.'/lib/BuildIndex' );
		BsCore::registerClass( 'BuildIndexMwSpecialLinked',      $sDir.'/lib/BuildIndex' );
		BsCore::registerClass( 'BuildIndexMwLinked',             $sDir.'/lib/BuildIndex' );

		//Setting up the Feed API
		$wgAPIModules['feedextendedsearch'] = 'ApiFeedExtendedSearch';
		BsCore::registerClass( 'ApiFeedExtendedSearch', $sDir.'/api', 'ApiFeedExtendedSearch.php' );

		$this->registerView( 'ViewSearchExtendedOptionsForm' );
		$this->registerView( 'ViewSearchResultMW' );
		$this->registerView( 'ViewNoOfResultsFound' );
		$this->registerView( 'ViewExtendedSearchFormPage' );
		$this->registerView( 'ViewSearchMultivalueField' );
		$this->registerView( 'ViewExtendedSearchFacetBox' );
		$this->registerView( 'ViewSearchSuggest' );
		$this->registerView( 'ViewSpell' );
		$this->registerView( 'ViewExtendedSearchResultEntry' );

		$this->setHook( 'FormDefaults' );
		// ---------- Hooks for indexing ----------
		$this->setHook( 'ArticleSaveComplete' );
		$this->setHook( 'ArticleDeleteComplete' );
		$this->setHook( 'ArticleUndelete' );
		$this->setHook( 'TitleMoveComplete' );
		$this->setHook( 'FileUpload' );
		$this->setHook( 'FileDeleteComplete' );
		$this->setHook( 'FileUndeleteComplete' );
		$this->setHook( 'SpecialPage_initList' );

		// create object ExtendedSearchBaseMW and register it to the search base in the core.
		// This will register all necessary variables automatically
		BsExtendedSearchBase::registerInstance( new ExtendedSearchBaseMW() );

		if ( $this->oExtendedSearchBaseMW === null ) {
			// BsExtendedSearchBase::getInstance( 'MW' ) returns instance of BsExtendedSearchBaseMW
			$this->oExtendedSearchBaseMW = BsExtendedSearchBase::getInstance( 'MW' );
		}

		// registering SpecialPage inside MediaWiki, pointing to subfolder specialpages
		$this->mAdapter->registerSpecialPage( 'SpecialExtendedSearch', dirname( __FILE__ ).'/specialpages', 'ExtendedSearchAlias' );

		if ( BsConfig::get( 'MW::ExtendedSearch::ShowAutocomplete' ) === true ) {
			$this->registerStyleSheet( BsConfig::get( 'MW::ScriptPath' ).'/bluespice-mw/ext/ExtendedSearch/css/ExtendedSearch.css', false, 'MW::ExtendedSearch::Active' );
			BsScriptManager::add( 'BsSearchAutoComplete', BsConfig::get( 'MW::ScriptPath' ) . '/bluespice-mw/ext/ExtendedSearch/js/ExtendedSearchAutocompleteWidget.js' );
		}

		wfProfileOut( 'BS::'.__METHOD__ );
	}

	public function onSpecialPage_initList( &$aList ) {
		$aList['ExtendedSearch'] = 'SpecialExtendedSearch';
		return true;
	}

	/**
	 * Sets parameters for more complex options in preferences
	 * @param string $sAdapterName Name of the adapter, e.g. MW
	 * @param BsConfig $oVariable Instance of variable
	 * @return array Preferences options
	 */
	public function runPreferencePlugin( $sAdapterName, $oVariable ) {
		$aPrefs = array();
		switch ( $oVariable->getName() ) {
			case 'DefScopeUser' :
				$aPrefs = array(
					'options' => array(
						wfMessage( 'bs-extendedsearch-pref-scope-text' )->plain()  => 'text',
						wfMessage( 'bs-extendedsearch-pref-scope-title' )->plain() => 'title'
					)
				);
				break;
		}

		return $aPrefs;
	}

	/**
	 * Returns rendered inner part of search results page. Used for faceting and paging. AJAX function.
	 * @return string JSON encoded HTML of search results page.
	 */
	public function getRequestJson() {
		$sParamMode    = BsCore::getParam( 'mode', false, BsPARAM::GET | BsPARAMTYPE::STRING );
		$sSearchString = BsCore::getParam( 'searchstring', false, BsPARAM::GET | BsPARAMTYPE::STRING );

		if ( $sParamMode === 'autocomplete' ) {
			$aResults = $this->oExtendedSearchBaseMW->searchAutocomplete( $sSearchString );
			exit( $aResults );
		}

		$viewContentsSpecialPage = $this->oExtendedSearchBaseMW->renderSpecialpage();

		$oDummy = new StdClass();
		$oDummy->data = array( 'bodytext' => $viewContentsSpecialPage->execute() );
		wfRunHooks( 'ExtendedSearchBeforeAjaxResponse', array( null, &$oDummy ) );
		// CR MRG (23.06.11 23:52): Warum kein return?
		exit( json_encode( array( 'contents' => $oDummy->data['bodytext'] ) ) );
	}

	/**
	 * Determines default values for search form.
	 * @param BsEvent $oEvent Contains additional information, e.g. calling instance.
	 * @param array $aSearchBoxKeyValues List of parameters sent by calling event.
	 * @return array Modified parameters list.
	 */
	public function onFormDefaults( $oCallingInstance, &$aSearchBoxKeyValues ) {
		$aLocalUrl = SpecialPage::getTitleFor( 'SpecialExtendedSearch' )->getLocalUrl();
		$aLocalUrl = explode( '?', $aLocalUrl );

		$aSearchBoxKeyValues['SearchDestination'] = $aLocalUrl[0];
		if ( isset( $aLocalUrl[1] ) && strpos( $aLocalUrl[1], '=' ) !== false ) {
			$aTitle = explode( '=', $aLocalUrl[1] );
			$aSearchBoxKeyValues['HiddenFields']['title'] = urldecode( $aTitle[1] );
		}
		if ( BsConfig::get( 'MW::ExtendedSearch::SearchFiles' ) ) {
			$aSearchBoxKeyValues['HiddenFields']['search_files'] = '1';
		}
		$aSearchBoxKeyValues['TitleKeyValuePair'] = array( 'search_scope', 'title' );
		$aSearchBoxKeyValues['FulltextKeyValuePair'] = array( 'search_scope', 'text' );
		$aSearchBoxKeyValues['SearchTextFieldName'] = 'search_input';
		// Default scope 
		$aSearchBoxKeyValues['DefaultKeyValuePair'] = ( BsConfig::get( 'MW::ExtendedSearch::DefScopeUser' ) == 'title' )
			? $aSearchBoxKeyValues['TitleKeyValuePair']
			: $aSearchBoxKeyValues['FulltextKeyValuePair'];
		if ( $oCallingInstance instanceof ViewExtendedSearchFormPage )
			$aSearchBoxKeyValues['HiddenFields']['search_origin'] = 'search_form_body';
		elseif ( $oCallingInstance instanceof BlueSpiceTemplate )
			$aSearchBoxKeyValues['HiddenFields']['search_origin'] = 'titlebar';
		$aSearchBoxKeyValues['HiddenFields']['search_go'] = 'true';
		$aSearchBoxKeyValues['method'] = ( 0 == strcasecmp( BsConfig::get( 'Core::Search::FormMethod' ), 'get' ) ) ? 'get' : 'post';
		if ( false && BsConfig::get( 'Core::Search::ShowSuggestions' ) && $oCallingInstance !== null )
				$aSearchBoxKeyValues['IdBsSearchLeft'] = ' <img src="'.$oCallingInstance->data['stylepath'].'/bluespice/bs-icon-search-dropdown.png" id="bs-search-dropdown" alt="Search drop down" />';
		if ( !empty( SearchOptions::$searchStringRaw ) ) $aSearchBoxKeyValues['SearchTextFieldText'] = SearchOptions::$searchStringRaw;

		return true;
	}

	/**
	 * Delete search index entry on article deletion
	 * @param Article $oArticle The article that is being deleted.
	 * @param User $oUser The user that deletes.
	 * @param string $sReason A reason for article deletion
	 * @param int $iID Id of article that was deleted.
	 * @return bool allow other hooked methods to be executed. Always true.
	 */
	public function onArticleDeleteComplete( &$oArticle, &$oUser, $sReason, $iID ) {
		try {
			$this->oExtendedSearchBaseMW->deleteFromIndexWiki( $iID );
		}
		catch ( BsException $e ) {
			wfDebugLog( 'ExtendedSearch', 'onArticleDeleteComplete: '.$e->getMessage() );
		}

		return true;
	}

	/**
	 * Update index on article change.
	 * @param Article $oArticle The article that is created.
	 * @param User $oUser User that saved the article.
	 * @param string $sText New text.
	 * @return bool allow other hooked methods to be executed. Always true.
	 */
	public function onArticleSaveComplete( &$oArticle, &$oUser ) {
		try {
			$this->oExtendedSearchBaseMW->updateIndexWiki( $oArticle );
		}
		catch ( BsException $e ) {
			wfDebugLog( 'ExtendedSearch', 'onArticleSaveComplete: '.$e->getMessage() );
		}

		return true;
	}

	/**
	 * Update index on article undelete
	 * @param Title $oTitle MediaWiki title object of recreated article
	 * @param bool $bCreate Whether or not the restoration caused the page to be created (i.e. it didn't exist before)
	 * @return bool allow other hooked methods to be executed. Always true.
	 */
	public function onArticleUndelete( $oTitle, $bCreate ) {
		try {
			$this->oExtendedSearchBaseMW->updateIndexWikiByTitleObject( $oTitle );
		}
		catch ( BsException $e ) {
			wfDebugLog( 'ExtendedSearch', 'onArticleUndelete: '.$e->getMessage() );
		}

		return true;
	}

	/**
	 * Update search index when an article is moved.
	 * @param Title $oTitle Old title of the moved article.
	 * @param Title $oNewtitle New title of the moved article.
	 * @param User $oUser User that moved the article.
	 * @param int $iOldID ID of the page that has been moved.
	 * @param int $iNewID ID of the newly created redirect.
	 * @return bool allow other hooked methods to be executed. Always true.
	 */
	public function onTitleMoveComplete( &$oTitle, &$oNewtitle, &$oUser, $iOldID, $iNewID ) {
		try {
			// Moving article
			$this->oExtendedSearchBaseMW->updateIndexWikiByTitleObject( $oNewtitle );
			// Check if redirect is created; 0 is no redirect
			if ( $iNewID != 0 ) {
				$this->oExtendedSearchBaseMW->updateIndexWikiByTitleObject( $oTitle );
			}
			// Moving file if namespace of title is the file namespace
			if ( $oTitle->getNamespace() == NS_FILE ) {
				global $wgVersion;
				$oOldFile = LocalFile::newFromTitle( $oTitle, RepoGroup::singleton()->getLocalRepo() );
				$oNewFile = RepoGroup::singleton()->findFile( $oNewtitle );
				if ( $wgVersion < '1.18.0' ) {
					$sFilePath = $oOldFile->getPath();
				}
				else {
					$oFileRepoLocalRef = $oOldFile->getRepo()->getLocalReference( $oOldFile->getPath() );
					if ( !is_null( $oFileRepoLocalRef ) ) {
						$sFilePath = $oFileRepoLocalRef->getPath();
					}
				}
				$this->oExtendedSearchBaseMW->deleteIndexFile( -1,$sFilePath );
				$this->oExtendedSearchBaseMW->updateIndexFile( $oNewFile );
			}
		}
		catch ( BsException $e ) {
			wfDebugLog( 'ExtendedSearch', 'onTitleMoveComplete: '.$e->getMessage() );
		}

		return true;
	}

	/**
	 * Update index on file upload
	 * @param File $oFile MediaWiki file object of uploaded file
	 * @param bool $bReupload indicates if file was uploaded before
	 * @param bool $bHasDescription indicates if a description page existed before
	 * @return bool allow other hooked methods to be executed. Always true.
	 */
	public function onFileUpload( $oFile, $bReupload = false, $bHasDescription = false ) {
		try {
			$this->oExtendedSearchBaseMW->updateIndexFile( $oFile );
		}
		catch ( BsException $e ) {
			wfDebugLog( 'ExtendedSearch', 'onFileUpload: '.$e->getMessage() );
		}

		return true;
	}

	/**
	 * Delete file from index when file is deleted
	 * @param File $oFile MediaWiki file object of deleted file
	 * @param unknown $uOldimage the name of the old file
	 * @param Article $oArticle reference to the article if all revisions are deleted
	 * @param User $oUser user who performed the deletion
	 * @param string $sReason reason
	 * @return bool allow other hooked methods to be executed. Always true.
	 */
	public function onFileDeleteComplete( $oFile, $uOldimage, $oArticle, $oUser, $sReason ) {
		try {
			global $wgVersion;
			if ( $wgVersion < '1.18.0' ) {
				$sFilePath = $oFile->getPath();
			}
			else {
				$oFileRepoLocalRef = $oFile->getRepo()->getLocalReference( $oFile->getPath() );
				if ( !is_null( $oFileRepoLocalRef ) ) {
					$sFilePath = $oFileRepoLocalRef->getPath();
				}
			}
			$this->oExtendedSearchBaseMW->deleteIndexFile( -1, $sFilePath );
		}
		catch ( BsException $e ) {
			wfDebugLog( 'ExtendedSearch', 'onFileDeleteComplete: '.$e->getMessage() );
		}

		return true;
	}

	/**
	 * Update index when file is undeleted
	 * @param Title $oTitle MediaWiki title object of undeleted file
	 * @param array $aFileVersions array of undeleted versions
	 * @param User $oUser user who performed the undeletion
	 * @param string $sReason reason
	 * @return bool allow other hooked methods to be executed. Always true.
	 */
	public function onFileUndeleteComplete( $oTitle, $aFileVersions, $oUser, $sReason ) {
		try {
			$oFile = wfFindFile( $oTitle );
			$this->oExtendedSearchBaseMW->updateIndexFile( $oFile );
		}
		catch ( BsException $e ) {
			wfDebugLog( 'ExtendedSearch', 'onFileUndeleteComplete: '.$e->getMessage() );
		}

		return true;
	}

}