<?php
/**
 * Base class for ExtendedSearch for MediaWiki
 *
 * Part of BlueSpice for MediaWiki
 *
 * @author     Mathias Scheer <scheer@hallowelt.biz>
 * @author     Markus Glaser <glaser@hallowelt.biz>
 * @author     Stephan Muggli <muggli@hallowelt.biz>
 * @version    $Id: ExtendedSearchBaseMW.class.php 9548 2013-05-31 11:12:31Z smuggli $
 * @package    BlueSpice_Extensions
 * @subpackage ExtendedSearch
 * @copyright  Copyright (C) 2010 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 * @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
 * @filesource
 */
/* Changelog
 * v0.1
 * FIRST CHANGES
 */
/**
 * Base class for ExtendedSearch for MediaWiki
 * @package BlueSpice_Extensions
 * @subpackage ExtendedSearch
 */
class ExtendedSearchBaseMW extends BsExtendedSearchBase {

	/**
	 * Unique wiki id
	 */
	private $sWikiID = '';

	/**
	 * Constructor of ExtendedSearchBaseMW class
	 */
	public function __construct() {
		wfProfileIn( 'BS::'.__METHOD__ );
		global $wgDBname, $wgDBprefix;
		if ( $wgDBprefix ) {
			$this->sWikiID = "$wgDBname-$wgDBprefix";
		} else {
			$this->sWikiID = $wgDBname;
		}
		$this->sFrameworkName = 'MW';
		$this->sExtensionName = 'ExtendedSearch';
		$this->registerVariables();
		wfProfileOut( 'BS::'.__METHOD__ );
	}

	/**
	 * Magic getter method
	 * @param string $sName Name of variable to get.
	 * @return mixed Value of the requested member variable.
	 */
	public function __get( $sName ) {
		return ( isset( $this->$sName ) ) ? $this->$sName : null;
	}

	/**
	 * Register adapter specific settings for Extended Search
	 */
	protected function registerVariables() {
		// max 32 chars with userlevel! 123 456789012345678 90123456789012 '::' counts as one char :-)
		BsConfig::registerVar( 'MW::ExtendedSearch::DefFuzziness',     '0.5',  BsConfig::TYPE_STRING, 'bs-extendedsearch-pref-defduzziness' );
		BsConfig::registerVar( 'MW::ExtendedSearch::ShowPercent',      true,   BsConfig::TYPE_BOOL, 'bs-extendedsearch-pref-showpercent', 'toggle' );
		BsConfig::registerVar( 'MW::ExtendedSearch::LimitResultDef',   25,     BsConfig::TYPE_INT|BsConfig::LEVEL_PUBLIC,  'bs-extendedsearch-pref-limitresultdef', 'int' );
		BsConfig::registerVar( 'MW::ExtendedSearch::SearchFiles',      false,  BsConfig::TYPE_BOOL|BsConfig::LEVEL_USER, 'bs-extendedsearch-pref-searchfiles', 'toggle' );
		BsConfig::registerVar( 'MW::ExtendedSearch::HideWikiIcon',     false,  BsConfig::TYPE_BOOL|BsConfig::LEVEL_PUBLIC, 'bs-extendedsearch-pref-hidewikiicon', 'toggle' );
		BsConfig::registerVar( 'MW::ExtendedSearch::JumpToTitle',      false,  BsConfig::TYPE_BOOL|BsConfig::LEVEL_PUBLIC, 'bs-extendedsearch-pref-jumptotitle', 'toggle' );
		//BsConfig::registerVar( 'MW::ExtendedSearch::ShowCatFilter',    true,   BsConfig::TYPE_BOOL|BsConfig::LEVEL_PUBLIC, 'bs-extendedsearch-pref-showcatfilter', 'toggle' );
		BsConfig::registerVar( 'MW::ExtendedSearch::ShowCreateSugg',   false,  BsConfig::TYPE_BOOL|BsConfig::LEVEL_PUBLIC, 'bs-extendedsearch-pref-showcreatesugg', 'toggle' );
		BsConfig::registerVar( 'MW::ExtendedSearch::ShowSpell',        true,   BsConfig::TYPE_BOOL|BsConfig::LEVEL_PUBLIC, 'bs-extendedsearch-pref-showspell', 'toggle' );
		BsConfig::registerVar( 'MW::ExtendedSearch::ShowFacets',       true,   BsConfig::TYPE_BOOL|BsConfig::LEVEL_PUBLIC, 'bs-extendedsearch-pref-showfacets', 'toggle' );
		BsConfig::registerVar( 'MW::ExtendedSearch::ShowAutocomplete', true,   BsConfig::TYPE_BOOL|BsConfig::LEVEL_PUBLIC, 'bs-extendedsearch-pref-showautocomplete', 'toggle' );
		BsConfig::registerVar( 'MW::ExtendedSearch::ShowCreSugInAc',   true,   BsConfig::TYPE_BOOL|BsConfig::LEVEL_USER, 'bs-extendedsearch-pref-showcresuginac', 'toggle' );
		BsConfig::registerVar( 'MW::ExtendedSearch::AcEntries',        10,     BsConfig::TYPE_INT|BsConfig::LEVEL_PUBLIC,  'bs-extendedsearch-pref-acentries', 'int' );
		BsConfig::registerVar( 'MW::ExtendedSearch::ShowTime',         true,   BsConfig::TYPE_BOOL|BsConfig::LEVEL_PUBLIC, 'bs-extendedsearch-pref-showtime', 'toggle' );
		BsConfig::registerVar( 'MW::ExtendedSearch::IndexTyLinked',    true,   BsConfig::TYPE_BOOL|BsConfig::LEVEL_PUBLIC, 'bs-extendedsearch-pref-indextylinked', 'toggle' );
		BsConfig::registerVar( 'MW::ExtendedSearch::IndexTypesRepo',   true,   BsConfig::TYPE_BOOL|BsConfig::LEVEL_PUBLIC, 'bs-extendedsearch-pref-indextypesrepo', 'toggle' );
		BsConfig::registerVar( 'MW::ExtendedSearch::IndexTypesWiki',   true,   BsConfig::TYPE_BOOL|BsConfig::LEVEL_PUBLIC, 'bs-extendedsearch-pref-indextypeswiki', 'toggle' );
		BsConfig::registerVar( 'MW::ExtendedSearch::ExternalRepo',     '',     BsConfig::LEVEL_PUBLIC|BsConfig::TYPE_STRING, 'bs-extendedsearch-pref-externalrepo' );
		BsConfig::registerVar( 'MW::ExtendedSearch::DefScopeUser',     'text', BsConfig::LEVEL_USER|BsConfig::TYPE_STRING|BsConfig::USE_PLUGIN_FOR_PREFS, 'bs-extendedsearch-pref-defscopeuser', 'select' );
		BsConfig::registerVar( 'Core::Search::CustomerID', md5( $this->sWikiID ), BsConfig::TYPE_STRING|BsConfig::LEVEL_PRIVATE, 'bs-extendedsearch-pref-customerid' ); // replaces WikiID
	}

	/**
	 * Renders the inner content of search result page.
	 * @return ViewBaseElement View with inner content of search result page.
	 */
	public function renderSpecialpage() {
		// form und results als views mit addItem an ein ViewBaseElement übergeben,
		$oView = new ViewBaseElement();
		$aMonitor = array();

		$aResultAllSearches = BsExtendedSearchBase::searchAllModules( $aMonitor );

		$vNoOfResultsFound = new ViewNoOfResultsFound();
		$vNoOfResultsFound->setOptions( $aMonitor );
		$oView->addItem( $vNoOfResultsFound );

		foreach ( $aResultAllSearches as $name => $view ) {
			$oView->addItem( $view );
		}
		return $oView;
	}

	/**
	 * Triggers a search index update for a specified article.
	 * @param Article $oArticle MediaWiki article object of article to be indexed.
	 * @param string $sText Text to be indexed (optional, fetched from article if not present)
	 */
	public function updateIndexWiki( &$oArticle ) {
		if ( $oArticle === null ) return;
		$oSearchService           = self::getSolrSearchService(); // may throw BsException 'server-not-available'
		$oBuildIndexMainControl   = self::getBuildIndexMainControl();
		$oBuildIndexMainControlMw = new BuildIndexMainControlMw( $this, $oBuildIndexMainControl, '', $oBuildIndexMainControl->setFileTypes() );
		$oBuildIndexMwArticles    = new BuildIndexMwArticles( $oBuildIndexMainControlMw );

		$oTitle    = $oArticle->getTitle();
		$oRevision = Revision::newFromTitle( $oTitle );

		wfRunHooks( 'BS::ExtendedSearch::UpdateIndexWiki', array( &$oTitle, &$oRevision ) );
		if ( $oTitle === null ) return;

		$iPageID         = $oTitle->getArticleID();
		$iPageNamespace  = $oTitle->getNamespace();
		$sPageTitle      = $oTitle->getText();
		$iPageTimestamp  = $oTitle->getTouched();
		$aPageCategories = $this->getCategoriesFromDbForCertainPageId( $iPageID );
		$aPageEditors    = $this->getEditorsFromDbForCertainPageId( $iPageID );
		$sPageContent    = $oRevision->getText();

		// http://www.mediawiki.org/wiki/Manual:WfTimestamp
		// wfTimestamp( TS_MW ) returns actual UTC in format YmdHis which results in gmdate( 'YmdHis', time() );
		// do not use date( 'YmdHis' ); it does not return GMT but timestamp with timezone-offset
		if ( strpos( $iPageTimestamp, '1970' ) === 0 ) $iPageTimestamp = wfTimestamp( TS_MW );

		$oSolrDocument = $oBuildIndexMwArticles->makeSingleDocument( $sPageTitle, $sPageContent, $iPageID, $iPageNamespace, $iPageTimestamp, $aPageCategories, $aPageEditors );
		try {
			$oSearchService->addDocument( $oSolrDocument );
		}
		catch ( Exception $ex ) {}

		try {
			// Indexing file links 
			$oBuildIndexMainControlMw->buildIndexLinked( '', $iPageID );
		}
		catch ( Exception $ex ) {}
		// commit reopens the IndexReaders in Solr-Server. Document might not be findable otherwise.
		$oSearchService->commit( true, true, true, 60 );
	}

	/**
	 * Triggers a search index update for a file.
	 * @param File $oFile file object.
	 */
	public function updateIndexFile( $oFile ) {
		$oBuildIndexMainControl   = self::getBuildIndexMainControl();
		$oBuildIndexMainControlMw = new BuildIndexMainControlMw( $this, $oBuildIndexMainControl );
		$oIndexFile               = new BuildIndexMwSingleFile( $oBuildIndexMainControlMw, $oFile );
		$oIndexFile->indexCrawledDocuments();
		return true;
	}

	/**
	 * Triggers deletion of a specified file from search index.
	 * @param int $id Article id of page to be deleted.
	 * @param string $path path to the file.
	 */
	public function deleteIndexFile( $iID, $sPath ) {
		$oSearchService         = self::getSolrSearchService();
		$oBuildIndexMainControl = self::getBuildIndexMainControl();
		$sUniqueID              = $oBuildIndexMainControl->getUniqueId( $iID, $sPath );
		$oSearchService->deleteByQuery( 'uid:'.$sUniqueID );
		$oSearchService->commit( true, true, true, 60 );
	}

	/**
	 * Triggers deletion of a specified item from search index.
	 * @param int $iID Article id of page to be deleted.
	 */
	public function deleteFromIndexWiki( $iID ) {
		$oSearchService         = self::getSolrSearchService(); // may throw BsException 'server-not-available'
		$oBuildIndexMainControl = self::getBuildIndexMainControl();
		$sUniqueID = $oBuildIndexMainControl->getUniqueId( $iID );
		$oSearchService->deleteById( $sUniqueID );
		$oSearchService->commit( true, true, true, 60 ); // todo: not sure if necessary
	}

	/**
	 * Triggers search index update for a given title.
	 * @param Title $title MediaWiki title object of article to be updated.
	 */
	public function updateIndexWikiByTitleObject( $oTitle ) {
		// $this->oSearchService is available.
		// If it was not, new BsException had been thrown beforehand
		$oArticle = new Article( $oTitle );
		$this->updateIndexWiki( $oArticle );
	}

	/**
	 * Reads out table %dbPrefix%categorylinks for certain page_id
	 * @param int $pageId ID of article that category links should be read for.
	 * @return array Categorynames as values
	 */
	public function getCategoriesFromDbForCertainPageId( $iPageID ) {
		$oDbr = wfGetDB( DB_SLAVE );

		// returns false on failure
		$oDbResTableCategories = $oDbr->select(
				'categorylinks',
				'DISTINCT cl_to',
				array( 'cl_from' => $iPageID )
		);

		$aCategories = array();
		if ( $oDbResTableCategories && $oDbr->numRows( $oDbResTableCategories ) > 0 ) {
			while ( $rowTableCategories = $oDbr->fetchObject( $oDbResTableCategories ) ) {
				$aCategories[] = $rowTableCategories->cl_to;
			}
		}
		$oDbr->freeResult( $oDbResTableCategories );
		return $aCategories;
	}

	/**
	 * Reads out table %dbPrefix%revision for certain page_id
	 * @param int $pageId ID of article that revisions should be read for.
	 * @return array editors as values
	 */
	public function getEditorsFromDbForCertainPageId( $iPageID ) {
		$oDbr = wfGetDB( DB_SLAVE );

		// returns false on failure
		$oDbResTableRevision = $oDbr->select(
				'revision',
				'DISTINCT rev_user_text',
				array( 'rev_page' => $iPageID )
		);

		$aEditors = array();
		if ( $oDbr->numRows( $oDbResTableRevision ) > 0 ) {
			$oUser = null;
			$sEditor = '';
			while ( $rowTableRevision = $oDbr->fetchObject( $oDbResTableRevision ) ) {
				$sEditor = $rowTableRevision->rev_user_text;
				$oUser = User::newFromName( $sEditor );
				if ( !is_object( $oUser ) ) $sEditor = 'unknown';
				$aEditors[] = $sEditor;
			}
		}
		$oDbr->freeResult( $oDbResTableRevision );
		return $aEditors;
	}

	/**
	 * Starts a search for a given search request.
	 * @param BsSearchRequest $oSearchRequestCore Search request as compiled from URL parameters or default values.
	 * @param array $aMonitor Set of options.
	 * @return ViewBaseElement View for search results.
	 */
	public function &search( BsSearchRequest $oSearchRequestCore, &$aMonitor ) {
		try {
			$oSearchService = self::getSolrSearchService();
		}
		catch ( BsException $e ) {
			if ( $e->getMessage() == 'bs-extendedsearch-server-not-available' ) {
				wfDebugLog( 'BS::ExtendedSearch', 'ExtendedSearchBaseMW::search: '.$e->getMessage() );
				$oSearchService = null;
			}
			else {
				$vItem = new ViewBaseElement();
				$vItem->setTemplate( 'Error: {error}' );
				$vItem->addData( array( 'error' => $e->getMessage() ) );
				return $vItem;
			}
		}
		if ( $this->oSearchIndex === null ) {
			$this->oSearchIndex = new SearchIndexMW( $oSearchService );
		}
		try {
			$vItem = $this->oSearchIndex->search( $oSearchRequestCore, $aMonitor );
		}
		catch ( BsException $e ) {
			if ( $e->getMessage() == 'redirect' ) throw $e;
			$vItem = new ViewBaseElement();
			$vItem->setTemplate( 'Error: {error}' );
			$vItem->addData( array( 'error' => $e->getMessage() ) );
		}
		return $vItem;
	}

	/**
	 * Starts a search for Autocomplete
	 * @param String $sSearchString The string to be searched for.
	 * @return String JSON of search results.
	 */
	public function searchAutocomplete( $sSearchString ) {
		// Lookup for solr
		try {
			$oSearchService = ExtendedSearchBaseMW::getSolrSearchService();
		}
		catch ( BsException $e ) {
			return '';
		}

		// Check if curl is available
		if ( BsExtendedSearchBase::isCurlActivated() === false ) return '';

		// Create all instances
		$oSearchRequest   = new BsSearchRequest( 'autocomplete' );
		$oSearchRequestMW = new SearchRequestMW( $oSearchRequest );
		$oSearchOptions   = new SearchOptions( $oSearchRequestMW );

		// Prepare search input
		$sSearchString     = urldecode( $sSearchString );
		$sSolrSearchString = BsSearchService::preprocessSearchInput( $sSearchString );
		$sSearchString     = BsSearchService::sanitzeSearchString( $sSearchString );

		// Make solr query suitable for autocomplete
		$aSolrQuery = $oSearchOptions->getSolrQuery();

		$aSolrQuery['searchString']           = 'titleWord:('.$sSolrSearchString.') OR titleWord:('.$sSolrSearchString.'*)';
		$aSolrQuery['searchLimit']            = BsConfig::get( 'MW::ExtendedSearch::AcEntries' );
		$aSolrQuery['searchOptions']['hl']    = 'off';
		$aSolrQuery['searchOptions']['facet'] = 'off';

		// Results
		$oHits = $oSearchService->search( $aSolrQuery['searchString'], $aSolrQuery['offset'], $aSolrQuery['searchLimit'], $aSolrQuery['searchOptions'] );

		$aResults   = array();
		$iID        = 0;
		$oDocuments = $oHits->response->docs;

		// Check if result is not empty
		if ( !empty( $oDocuments ) ) {
			$oTitle                = null;
			$iPosition             = 0;
			$sPartOfTitle          = '';
			$sModifiedSearchString = '';
			$sLabelText            = '';
			$sEscapedPattern       = '';
			$aSearchStringParts    = array();

			foreach ( $oDocuments as $oDoc ) {
				if ( $oDoc->namespace == '999' ) {
					$oTitle = Title::makeTitle( NS_FILE, $oDoc->title );
				} else {
					$oTitle = Title::makeTitle( $oDoc->namespace, $oDoc->title );
				}

				if ( !$oTitle->userCan( 'read' ) ) continue;

				// Highlighting
				$sModifiedSearchString = str_replace( "/", " ", $sSearchString );
				$aSearchStringParts    = explode( " ", $sModifiedSearchString );
				$sLabelText            = BsStringHelper::shorten( $oTitle->getPrefixedText(), array( 'max-length' => '48', 'position' => 'middle', 'ellipsis-characters' => '...' ) );

				$iPosition = stripos( $sLabelText, $sSearchString );

				if ( $iPosition !== false ) {
					$sPartOfTitle    = mb_substr( $sLabelText, $iPosition, mb_strlen( $sSearchString ) );
					$sEscapedPattern = preg_quote( $sPartOfTitle, '#' );
					$sLabelText      = preg_replace( '#'.$sEscapedPattern.'#i', '<b>'.$sPartOfTitle.'</b>', $sLabelText , 1 );
				}
				else {
					$iOccurences  = 0;
					$aOccurrences = array();

					foreach ( $aSearchStringParts as $sPart ) {
						if ( empty( $sPart ) ) continue;
						$sModifiedPart = strtolower( $sPart );
						if ( in_array( $sModifiedPart, $aOccurrences ) ) {
							continue;
						}
						else {
							$iOccurences = substr_count( strtolower( $sSearchString ), $sModifiedPart );
						}
						$iPosition = mb_strripos( $sLabelText, $sPart );

						if ( $iPosition !== false ) {
							$sPartOfTitle    = mb_substr( $sLabelText, $iPosition, mb_strlen( $sPart ) );
							$sEscapedPattern = preg_quote( $sPartOfTitle, '#' );
							$sLabelText      = preg_replace( '#'.$sEscapedPattern.'#i', '['.$sPartOfTitle.']', $sLabelText , $iOccurences );

							$aOccurrences[] = $sModifiedPart;
						}
					}

					// Do some parsing and highlight the matches; "[" and "]" are not allowed in an article title
					$sLabelText = str_replace( array( '[', ']' ), array( '<b>', '</b>' ), $sLabelText );
				}

				// Make result object
				$oItem        = new stdClass();
				$oItem->id    = ++$iID;
				$oItem->value = $oTitle->getPrefixedText();
				$oItem->label = $sLabelText;
				$oItem->type  = $oDoc->type;
				$oItem->link  = $oTitle->getFullURL();
				$oItem->attr  = '';

				$aResults[] = $oItem;
			}
		}

		$bTitleExists = $oSearchOptions->titleExists( $sSearchString );
		// Hook to inject some "results"
		wfRunHooks( 'BSExtendedSearchAutocomplete', array( &$aResults, $sSearchString, &$iID, $bTitleExists ) );


		$iSearchfiles = ( BsConfig::get( 'MW::ExtendedSearch::SearchFiles' ) ) ? '1' : '0' ;
		// object for fulltext search link
		$oItem        = new stdClass();
		$oItem->id    = ++$iID;
		$oItem->value = $sSearchString;
		$oItem->label = wfMessage( 'bs-extendedsearch-searchfulltext' )->plain() . '<br /><b>' . BsStringHelper::shorten( $sSearchString, array( 'max-length' => '45', 'position' => 'middle', 'ellipsis-characters' => '...' ) ) . '</b>';
		$oItem->type  = '';
		$oItem->link  = SpecialPage::getTitleFor( 'SpecialExtendedSearch' )->getFullUrl( array( 'search_origin' => 'titlebar', 'search_go' => 'true', 'search_scope' => 'text', 'search_input' => $sSearchString, 'search_files' => $iSearchfiles ) );
		$oItem->attr  = 'bs-extendedsearch-ac-noresults';

		$aResults[] = $oItem;

		return json_encode( $aResults );
	}

	/**
	 * Completely (re)builds search index.
	 * @param BsBuildIndexMainControl $oBsBuildIndexMainControl Cental controller for rebuild process.
	 * @param string $sLimit Number and offset of articles to be indexed. Format {iLimitStart},{iLimitRange}
	 * @param array $aFiletypes List of file types to be indexed.
	 * @param array $aTypes List of document types to be indexed.
	 * @param int $iTimeLimit Maximum execution time per document in seconds.
	 * @return string Always empty
	 */
	// called via ExtendedSearchBase::buildIndexForAllModules(), $instance->buildIndex();
	public function buildIndex( $oBsBuildIndexMainControl, $sLimit, $aFiletypes, $aTypes, $iTimeLimit ) { // may throw BsException('server-not-available');
		$oBuildIndexMainControlMw = new BuildIndexMainControlMw( $this, $oBsBuildIndexMainControl, $sLimit, $aFiletypes, $aTypes, $iTimeLimit );
		$oBuildIndexMainControlMw->buildIndexWiki( wfMessage( 'bs-extendedsearch-indexing_wiki_articles' )->plain() );
		$oBuildIndexMainControlMw->buildIndexRepo( wfMessage( 'bs-extendedsearch-indexing_files_in_repo' )->plain() );
		$oBuildIndexMainControlMw->buildIndexLinked( wfMessage( 'bs-extendedsearch-indexing_linked_files' )->plain() );
		$oBuildIndexMainControlMw->buildIndexExternalRepo( wfMessage( 'bs-extendedsearch-indexing_external_files_in_repo' )->plain() );

		wfRunHooks( 'BSExtendedSearchBuildIndex', array( $oBuildIndexMainControlMw, $sLimit, $aFiletypes, $aTypes, $iTimeLimit ) );
		return true;
	}

}