/**
 * ExtendedSearch extension
 *
 * Part of BlueSpice for MediaWiki
 *
 * @author     Markus Glaser <glaser@hallowelt.biz>
 * @author     Stephan Muggli <muggli@hallowelt.biz>
 * @version    $Id: bs-extendedsearchspecialpage.js 8261 2013-01-22 14:13:15Z smuggli $
 * @package    Bluespice_Extensions
 * @subpackage ExtendedSearch
 * @copyright  Copyright (C) 2011 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 * @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
 * @filesource
 */

// Uses jQuery!

/**
 * Base class for all extended search related methods and properties
 */
BsExtendedSearchSpecialPage = {

	/**
	 * Initialization method for search result view.
	 */
	init: function() {
		$( '#bs-extendedsearch-inputfieldtext-specialpage' ).focus( function() {
			if( $( this ).val() == $( this ).attr( 'defaultvalue' ) ) $( this ).val( '' );
			if ( $( '#bs-search-button-hidden-specialpage' ).val() == 'title' ) {
				$( '#bs-extendedsearch-titlebuttonimage-specialpage' ).attr( 'shine', 'on' );
			}
			else {
				$( '#bs-extendedsearch-fulltextbuttonimage-specialpage' ).attr( 'shine', 'on' );
			}
		}).blur( function() {
			if( $( this ).val() == '' ) $( this ).val( $( this ).attr( 'defaultvalue' ) );
			$( '#bs-extendedsearch-titlebuttonimage-specialpage' ).attr( 'shine', '' );
			$( '#bs-extendedsearch-fulltextbuttonimage-specialpage' ).attr( 'shine', '' );
		});

		$( '#bs-search-button-specialpage' ).click( function( event ) {
			$(this).parent().find('*[name=search_scope]').val( 'title' );
			$(this).parent().submit();
		});

		$( '#bs-search-fulltext-specialpage' ).click( function() {
			$(this).parent().find('*[name=search_scope]').val( 'text' );
			$(this).parent().submit();
		});
	}

}

/**
 * Class for AJAX search result updates
 */
BsExtendedSearchAjaxManager = function() {};

/**
 * Class definition for AJAX search result updates
 */
BsExtendedSearchAjaxManager.prototype = {
	/**
	 * Script base that is prepended to any local url.
	 * @var string Base part of URL
	 */
	oUrlBase : null,
	/**
	 * URL parameters.
	 * @var array With key value pairs.
	 */
	oUrlParams : {},
	/**
	 * The AJAX-Request-Object is stored here. If new Request is done with oAjaxQuery != null oAjaxQuery is aborted first. On successful request it is reset to null
	 * @var object AXAY request object.
	 */
	oAjaxQuery : null,

	/**
	 * Builds URI from oUrlParams
	 * @return string Compiled URL.
	 */
	getUri : function() {
		var out = [];
		for( key in BsCore.ExtendedSearchAjaxManager.oUrlParams ) {
			if ( typeof( BsCore.ExtendedSearchAjaxManager.oUrlParams[key] ) == 'object' 
				&& ( BsCore.ExtendedSearchAjaxManager.oUrlParams[key] instanceof Array ) ) {

				for ( key2 in BsCore.ExtendedSearchAjaxManager.oUrlParams[key] ){
					if ( typeof( BsCore.ExtendedSearchAjaxManager.oUrlParams[key][key2] ) != 'string' ) continue;
					out.push( key + '[]=' + BsCore.ExtendedSearchAjaxManager.oUrlParams[key][key2] );
				}
			}
			else {
				out.push( key + '=' + BsCore.ExtendedSearchAjaxManager.oUrlParams[key] );
			}
		}
		var newUri = BsCore.ExtendedSearchAjaxManager.oUrlBase + '?' + out.join( '&' );
		return newUri;
	},

	/**
	 * Convert any URI in a format that talks to AJAX handler.
	 * @param string uriIn Original URI.
	 * @return string Modified URI.
	 */
	ajaxifyUri: function( uriIn ) {
		var uriParts = uriIn.split( '?' )
		var uriOut = BlueSpice.buildRemoteString(
			'ExtendedSearch',
			'getRequestJson',
			{
				"search_origin": "ajax"
			}
		);

		if ( 1 in uriParts ) {
			var uriParams = uriParts[1].split( '&' );
			var key;
			for ( i in uriParams ) {
				if ( typeof( uriParams[i] ) != 'string' ) continue;
				key = uriParams[i].split( '=' )[0].toLowerCase();
				if ( key == 'title' || key == 'search_origin' ) continue;
				uriOut += '&' + uriParams[i];
			}
		}
		return uriOut;
	},

	/* executed once on $(document).ready */
	/**
	 * Initializes AJAX functions.
	 */
	init: function() {
		this.renewUriForAjaxRequest();
		this.modifyLoadedElements();

		$( window ).resize(function() {
			BsCore.ExtendedSearchAjaxManager.spinnerResize();
		});
	},

	/**
	 * Updates URI parameters.
	 */
	renewUriForAjaxRequest: function() {
		if ( typeof( $( '#bs-extendedsearch-siteuri' ).attr( 'siteuri' ) ) == "undefined" ) return;
		BsCore.ExtendedSearchAjaxManager.oUrlParams = {};
		var uriParts = $( '#bs-extendedsearch-siteuri' ).attr( 'siteuri' ).split( '?' );
		/* the ajax-Request is different:
		 *  - no title=Special:SpecialExtendedSearch nor index.php/Special:Specialextendedsearch
		 *  - instead action=remote&mod=ExtendedSearch&rf=getRequestJson
		 *  - value of search_origin is overriden with 'ajax'
		 */
		//BsCore.ExtendedSearchAjaxManager.oUrlBase = uriParts[0].replace(/(^.*\/index\.php).*$/, '$1');
		BsCore.ExtendedSearchAjaxManager.oUrlBase = wgScriptPath + '/';
		if ( 1 in uriParts ) {
			var uriParams = uriParts[1].split( '&' );
			var aKeyValue;
			for ( i in uriParams ) {
				if ( typeof( uriParams[i] ) != 'string' ) continue;
				aKeyValue = uriParams[i].split( '=' );
				if ( aKeyValue[0].toLowerCase() == 'title' ) continue;
				BsCore.ExtendedSearchAjaxManager.addParamToUrl( aKeyValue[0], aKeyValue[1] );
			}
		}
		BsCore.ExtendedSearchAjaxManager.addParamToUrl( 'action', 'remote' );
		BsCore.ExtendedSearchAjaxManager.addParamToUrl( 'mod', 'ExtendedSearch' );
		BsCore.ExtendedSearchAjaxManager.addParamToUrl( 'rf', 'getRequestJson' );
		BsCore.ExtendedSearchAjaxManager.addParamToUrl( 'search_origin', 'ajax' );
	},

	/**
	 * Updates search result view.
	 */
	modifyLoadedElements: function() {
		$('.search-wrapper').mouseover(function(){
			$(this).css( 'background-color', '#EEEEEE' );
		});
		$('.search-wrapper').mouseleave(function(){
			$(this).css( 'background-color', '#FFFFFF' );
		});

		this.searchAsYouType();

		if ( $( '.bs-extendedsearch-paging A' ).length === 0 ) // paging bar looks like < 1 >
		{
			$( '.bs-extendedsearch-paging' ).css( 'visibility', 'hidden' );
		}

		// facets armed with attribute urldiff...
		$( '[urldiff]' ).removeAttr( 'onchange' ).change( function() {
			BsCore.ExtendedSearchAjaxManager.changeRequestFacets( $( this ).attr( 'urldiff' ), $( this ).attr( 'checked' ) );
		});

		$('#bs-extendedsearch-filters-results-paging A').click( function( event ) {
			// if middle button is pressed don't use ajax (allows opening in new tab)
			if( event.which == 2 ) {
				return true;
			}

			if ( $( this ).attr( 'href' ).indexOf( 'search_origin=uri_builder' ) == -1 ) {
				return true;
			}
			else {
				event.preventDefault();
				BsCore.ExtendedSearchAjaxManager.ajaxMeANewResultsPlz( $( this ).attr( 'href' ) );
				return false;
			}
		});
		this.spinnerResize();
	},

	/**
	 * Requests a new search result when facet was clicked.
	 * @param string urldiff Difference to URI tha was triggered by this facet.
	 * @param bool checked True if facet was checked.
	 */
	changeRequestFacets: function( urldiff, checked ) {
		var aAllParams = urldiff.split( '&' );
		var aKeyValue = '';
		var length = aAllParams.length;
		for ( var i=0; i<length; i++ ) {
			aKeyValue = aAllParams[i].split( '=' );
			if ( checked ) {
				BsCore.ExtendedSearchAjaxManager.addParamToUrl( aKeyValue[0], aKeyValue[1] );
			}
			else {
				BsCore.ExtendedSearchAjaxManager.stripParamFromUrl( aKeyValue[0], aKeyValue[1] );
			}
		}
		BsCore.ExtendedSearchAjaxManager.ajaxMeANewResultsPlz();
	},

	/**
	 * AJAX call that actually fetches a new search result.
	 * @param string paramUri URL that describes the search.
	 */
	ajaxMeANewResultsPlz: function( paramUri ) {
		if ( paramUri == undefined ) {
			paramUri = BsCore.ExtendedSearchAjaxManager.getUri();
		}
		else {
			paramUri = this.ajaxifyUri( paramUri );
		}
		if ( BsCore.ExtendedSearchAjaxManager.oAjaxQuery != null )
			BsCore.ExtendedSearchAjaxManager.oAjaxQuery.abort();
		$( '#bs-extendedsearch-spinner' ).show();
		BsCore.ExtendedSearchAjaxManager.oAjaxQuery = $.ajax({
			url: paramUri,
			dataType: 'json',
			success: function( response, textStatus ){
				$( '#bs-extendedsearch-spinner' ).hide();
				$( '#bs-extendedsearch-noofresultsandsearchterm' ).parent().after( response.contents ).remove();
				BsCore.ExtendedSearchAjaxManager.oAjaxQuery = null;
				BsCore.ExtendedSearchAjaxManager.renewUriForAjaxRequest();
				BsCore.ExtendedSearchAjaxManager.modifyLoadedElements();
			}
		});
	},

	/**
	 * This function places the loading spinner all over the search result div.
	 */
	spinnerResize: function() {
		var spinner = $( '#bs-extendedsearch-spinner' );
		spinner.height( $( '#bs-extendedsearch-results' ).height());
		var offset = $( '#bs-extendedsearch-results' ).offset();
		if ( !offset ) return;
		offset.top += $( window ).scrollTop();
		offset.left += $( window ).scrollLeft();
		spinner.offset( offset );
		spinner.width($( '#bs-extendedsearch-results' ).width());
	},

	/**
	 * Adds another parameter to oUrlParams
	 * @param string key Name of URI parameter
	 * @param string value Value of URI parameter
	 */
	addParamToUrl: function( key, value ) {
		if ( key.indexOf('[]') != -1 ) {
			key = key.replace( '[]', '' );
			if ( BsCore.ExtendedSearchAjaxManager.oUrlParams[key] == null ) BsCore.ExtendedSearchAjaxManager.oUrlParams[key] = [ value ];
			else {
				if ( $.inArray( value, BsCore.ExtendedSearchAjaxManager.oUrlParams[key] )== -1 )
					BsCore.ExtendedSearchAjaxManager.oUrlParams[key].push( value );
			}
		}
		else BsCore.ExtendedSearchAjaxManager.oUrlParams[key] = value;
	},

	/**
	 * Removes a parameter to oUrlParams
	 * @param string key Name of URI parameter
	 * @param string value Value of URI parameter
	 */
	stripParamFromUrl: function( key, value ) {
		if ( key.indexOf( '[]' ) != -1 ) 
			key = key.replace( '[]', '' );
		if ( BsCore.ExtendedSearchAjaxManager.oUrlParams[key] == null ) return;
		if ( typeof( BsCore.ExtendedSearchAjaxManager.oUrlParams[key] ) == 'object' 
			&& ( BsCore.ExtendedSearchAjaxManager.oUrlParams[key] instanceof Array ) ) {

			for ( i in BsCore.ExtendedSearchAjaxManager.oUrlParams[key] ) {
				if ( BsCore.ExtendedSearchAjaxManager.oUrlParams[key][i] == value ){
					BsCore.ExtendedSearchAjaxManager.oUrlParams[key].splice( i, 1 );
				}
			}
			if ( BsCore.ExtendedSearchAjaxManager.oUrlParams[key].length == 0 ) {
				delete BsCore.ExtendedSearchAjaxManager.oUrlParams[key];
			}
		}
		else {
			delete BsCore.ExtendedSearchAjaxManager.oUrlParams[key];
		}
	},

	/**
	 * Search as you type results
	 */
	searchAsYouType: function() {
		var inputField = $( '#bs-extendedsearch-inputfieldtext-specialpage' );
		var url;
		var thread;
		var keys = new Array( 13, 16, 17, 18, 20, 27, 32, 37, 38, 39, 40, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 123 );

		inputField.keydown( function() {
			clearTimeout( thread );
		} );

		inputField.keyup( function( event ) {
			clearTimeout( thread );
			if ( $.inArray( event.which, keys ) > 0 ) return false;
			if ( inputField.val() == '' ) return false;
			url = wgServer + wgScriptPath + '?search_scope=text&search_submit=1&search_input=' + encodeURIComponent( inputField.val() );
			thread = setTimeout( function() { BsCore.ExtendedSearchAjaxManager.ajaxMeANewResultsPlz( url ) }, 300 );
		} );
	}
}

BsCore.ExtendedSearchAjaxManager = null;

$(document).ready(function(){
	$(document).unbind( 'keydown' ).bind('keydown', function ( event ) {
		var doPrevent = false;
		if ( event.keyCode === 8 ) {
			var element = event.srcElement || event.target;

			if ( ( element.tagName.toLowerCase() === 'input' && element.type.toLowerCase() === 'text' ) ) {
				doPrevent = element.readOnly || element.disabled;
			}
			else {
				doPrevent = true;
			}
		}

		if ( doPrevent ) {
			event.preventDefault();
		}
	});
	BsExtendedSearchSpecialPage.init();
	BsCore.ExtendedSearchAjaxManager = new BsExtendedSearchAjaxManager();
	BsCore.ExtendedSearchAjaxManager.init();
	$("#bs-extendedsearch-checkbox-searchfiles").change(function(){
		if ($(this).is(':checked'))
			$('#bs-extendedsearch-input-searchfiles').val('1');
		else
			$('#bs-extendedsearch-input-searchfiles').val('0');
	});
});