<?php
/**
 * Processes search request for ExtendedSearch for MediaWiki
 *
 * Part of BlueSpice for MediaWiki
 *
 * @author     Mathias Scheer <scheer@hallowelt.biz>
 * @author     Markus Glaser <glaser@hallowelt.biz>
 * @author     Stephan Muggli <muggli@hallowelt.biz>
 * @version    $Id: SearchRequest.class.php 8814 2013-03-05 16:28:50Z smuggli $
 * @package    BlueSpice_Extensions
 * @subpackage ExtendedSearch
 * @copyright  Copyright (C) 2010 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 * @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
 * @filesource
 */
/* Changelog
 * v0.1
 * FIRST CHANGES
 */
/**
 * Processes search request for ExtendedSearch for MediaWiki
 * @package BlueSpice_Extensions
 * @subpackage ExtendedSearch
 */
class SearchRequestMW extends BsDecorator {

	/**
	 * Cosntructor for SearchRequestMW class
	 * @param BsSearchRequest $instanceToDecorate Object to extend with additional functionality
	 */
	public function __construct( BsSearchRequest $oInstanceToDecorate ) {
		parent::__construct( $oInstanceToDecorate );
		$this->setDefaults();
		$this->processSettings();
		$this->processInputs();
	}

	/*
	 * Todo: Any member should be initialized. No detail can be overseen this way. To be implemented.
	 */
	/**
	 * Sets the defaults for a search request.
	 */
	protected function setDefaults() {
		$this->sOrder = 'score';
		$this->sAsc = 'desc';
		$this->iOffset = 0;
	}

	/**
	 * Get values from settings
	 */
	protected function processSettings() {
		$this->sDefaultFuzziness = BsConfig::get( 'MW::ExtendedSearch::DefFuzziness' );
	}

	/**
	 * Get values from url parameters
	 */
	protected function processInputs() {
		$this->sScope        = BsCore::getParam( 'search_scope' );
		$this->sOrigin       = BsCore::getParam( 'search_origin' );
		$this->sMoreLikeThis = BsCore::getParam( 'search_mlt' );
		$this->sOperator     = BsCore::getParam( 'op' );
		$this->sAsc          = BsCore::getParam( 'search_asc',      $this->sAsc );
		$this->iOffset       = BsCore::getParam( 'search_offset',   $this->iOffset ); // todo: type is int??
		$this->sOrder        = BsCore::getParam( 'search_order',    $this->sOrder );
		$this->sFormat       = BsCore::getParam( 'search_format',   $this->sFormat );
		$this->sCategories   = BsCore::getParam( 'ca',              array(), BsPARAMTYPE::ARRAY_STRING | BsPARAM::REQUEST );
		$this->aNamespaces   = BsCore::getParam( 'na',              array(), BsPARAMTYPE::ARRAY_STRING | BsPARAM::REQUEST );
		$this->sType         = BsCore::getParam( 'ty',              false,   BsPARAMTYPE::ARRAY_STRING | BsPARAM::REQUEST );
		$this->sId           = BsCore::getParam( 'search_id',       false,   BsPARAMTYPE::INT | BsPARAM::REQUEST );
		$this->bExtendedForm = BsCore::getParam( 'search_extended', false,   BsPARAMTYPE::BOOL | BsPARAM::REQUEST );

		if ( BsCore::getParam( 'search_files' ) !== false ) {
			if ( $this->sOrigin != 'ajax' ) {
				if ( BsCore::getParam( 'search_files' ) == '1' ) {
					$this->bSearchFiles = true;
				}
				else {
					$this->bSearchFiles = false;
				}
			}
		}

		if ( !$this->sScope ) {
			$this->sScope = BsConfig::get( 'MW::ExtendedSearch::DefScopeUser' );
		}

		wfRunHooks( 'BSExtendedSearchRequestProcessInputs', array( &$this ) );
	}

	/**
	 * Can we actually commit a search?
	 * @return bool True if yes.
	 */
	public function isSearchable() {
		$submit = $this->sSubmit;
		$go     = $this->sGo;
		$input  = $this->sInput; // take care:  empty( $this->sInput ) does not work 'cause of getter magic method
		return (bool) ( ( $submit !== false || $go !== false ) && !empty( $input ) );
	}

}