<?php
/**
 * Special page for ExtendedSearch for MediaWiki
 *
 * Part of BlueSpice for MediaWiki
 *
 * @author     Mathias Scheer <scheer@hallowelt.biz>
 * @author     Markus Glaser <glaser@hallowelt.biz>
 * @author     Stephan Muggli <muggli@hallowelt.biz>
 * @version    $Id: SpecialExtendedSearch.class.php 9548 2013-05-31 11:12:31Z smuggli $
 * @package    BlueSpice_Extensions
 * @subpackage ExtendedSearch
 * @copyright  Copyright (C) 2010 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 * @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
 * @filesource
 */
/* Changelog
 * v0.1
 * FIRST CHANGES
 */
/**
 * Special page class for ExtendedSearch for MediaWiki
 * @package BlueSpice_Extensions
 * @subpackage ExtendedSearch
 */
class SpecialExtendedSearch extends BsSpecialPage {

	/**
	 * Reference to instance of SearchOptions class.
	 * @var Object SearchOptions class.
	 */
	protected $oSearchOptions = null;
	/**
	 * Reference to instance of base class.
	 * @var Object ExtendedSearchBaseMW class.
	 */
	protected $oExtendedsearchBaseMW = null;

	/**
	 * Constructor of SpecialExtendedSearch class
	 */
	function __construct() {
		parent::__construct( 'SpecialExtendedSearch' );
		$this->oExtendedsearchBaseMW = BsExtendedSearchBase::getInstance( 'MW' );
	}

	/**
	 * Renders extended search options form.
	 * @param array $aMonitor List that contains form view.
	 * @return ViewBaseElement View that describes search options.
	 */
	public function renderExtendedForm( &$aMonitor ) {
		$vForm = $aMonitor;

		$aHiddenFieldsInForm                  = array();
		$aHiddenFieldsInForm['search_asc']    = $this->oSearchOptions->getOption( 'asc' );
		$aHiddenFieldsInForm['search_order']  = $this->oSearchOptions->getOption( 'order' );// score|titleSort|type|ts
		$aHiddenFieldsInForm['search_submit'] = '1';

		$vForm->setOptions( array (
			'hiddenFields'                    => $aHiddenFieldsInForm,
			'files'                           => $this->oSearchOptions->getOption( 'files' ),
			'method'                          => ( ( strcasecmp( BsConfig::get( 'Core::Search::FormMethod' ), 'post' ) == 0 ) ? 'post' : 'get' ),
			'scope'                           => $this->oSearchOptions->getOption( 'scope' ),
			'user_is_allowed_to_search_files' => BsCore::getInstance( 'MW' )->getAdapter()->User->isAllowed( 'searchfiles' )
			)
		);

		$vOptionsFormWiki    = & $vForm->getOptionsForm( 'wiki', wfMessage( 'bs-extendedsearch-search-wiki' )->plain() );
		$vNamespaceBox       = & $vOptionsFormWiki->getBox( 'NAMESPACE-FIELD', 'bs-extendedsearch-search-namespace', 'na[]' );
		$oMwContLang         = BsCore::getInstance( 'MW' )->getAdapter()->ContLang;
		$aMwNamespaces       = $oMwContLang->getNamespaces();
		$aSelectedNamespaces = $this->oSearchOptions->getOption( 'namespaces' );

		if ( BsConfig::get( 'MW::SortAlph' ) ) asort( $aMwNamespaces );

		foreach ( $aMwNamespaces as $namespace ) {
			$iNsIndex = $oMwContLang->getNsIndex( $namespace );
			if ( $iNsIndex < 0 ) continue;
			if ( $iNsIndex == 0 ) $namespace = wfMessage( 'bs-extendedsearch-articles' )->plain();
			$vNamespaceBox->addEntry( 
				$iNsIndex,
				array(
					'value'    => $iNsIndex,
					'text'     => $namespace,
					'selected' => in_array( (string) $iNsIndex, $aSelectedNamespaces )
				)
			);
		}

		$checkboxSearchFilesAttributes = array(
			'type'  => 'checkbox',
			'name'  => 'search_files',
			'id'    => 'bs-extendedsearch-checkbox-searchfiles',
			'value' => '1',
		);

		if ( BsConfig::get( 'MW::ExtendedSearch::SearchFiles' ) || $this->oSearchOptions->getOption( 'files' ) ) {
			$checkboxSearchFilesAttributes['checked'] = 'checked';
		}
		$checkboxSearchFiles = BsXml::simpleTag( 'input', $checkboxSearchFilesAttributes, wfMessage( 'bs-extendedsearch-files' )->plain() );

		$vNamespaceBox->dirtyAppend( '<br />'.$checkboxSearchFiles );

		$dbr = wfGetDB( DB_SLAVE ); // needed for categories and editors

		$catRes = $dbr->select(
				array( 'category' ),
				array( 'cat_id', 'cat_title' ),
				'',
				null,
				array( 'ORDER BY' => 'cat_title asc' )
		);
		if ( $dbr->numRows( $catRes ) != 0 ) {
			$vCategoryBox = & $vOptionsFormWiki->getBox( 'CATEGORY-FIELD', 'bs-extendedsearch-search-category', 'ca[]' );
			$aSelectedCategories = $this->oSearchOptions->getOption( 'cats' );
			while ( $catRow = $dbr->fetchObject( $catRes ) ) {
				$key = $catRow->cat_title;
				$vCategoryBox->addEntry(
					$key,
					array(
						'value'    => $catRow->cat_title,
						'text'     => $catRow->cat_title,
						'selected' => in_array( $key, $aSelectedCategories )
					)
				);
			}
		}

		$dbr->freeResult( $catRes );

		$vEditorsBox = & $vOptionsFormWiki->getBox( 'EDITORS-FIELD', 'bs-extendedsearch-search-editors', 'ed[]' );
		$edRes = $dbr->select(
			array( 'revision' ),
			array( 'DISTINCT rev_user_text' ),
			'',
			null,
			array( 'ORDER BY' => 'rev_user_text' )
		);
		$aSelectedEditors = $this->oSearchOptions->getOption( 'editor' );
		while ( $edRow = $dbr->fetchObject( $edRes ) ) {
			$oUser = User::newFromName( $edRow->rev_user_text );
			if ( !is_object( $oUser ) ) continue;

			$vEditorsBox->addEntry(
				$oUser->getName(),
				array(
					'value'    => $oUser->getName(),
					'text'     => $oUser->getName(),
					'selected' => in_array( $oUser->getName(), $aSelectedEditors )
				)
			);
		}

		$dbr->freeResult( $edRes );

		$vbe = new ViewBaseElement();
		$vbe->setAutoElement( false );
		return $vbe;
	}

	/**
	 * Actually render the page content.
	 * @param string $sParameter URL parameters to special page.
	 * @return string Rendered HTML output.
	 */
	function execute( $sParameter ) {
		parent::execute( $sParameter );

		BsStyleManager::add( 'BsSearch', BsConfig::get( 'Core::BlueSpiceScriptPath' ) . '/lib/Search/css/extendedsearch.css' );
		BsScriptManager::add( 'BsSearch', BsConfig::get( 'MW::ScriptPath' ) . '/bluespice-mw/ext/ExtendedSearch/js/bs-extendedsearchspecialpage.js' );

		$oSearchRequest       = new BsSearchRequest( 'search_form_body' );
		$oSearchRequestMW     = new SearchRequestMW( $oSearchRequest );
		$this->oSearchOptions = new SearchOptions( $oSearchRequestMW );
		$oSearchformView      = new ViewBaseElement();
		$oSearchform          = new ViewExtendedSearchFormPage();
		$this->oUriBuilder    = new SearchUriBuilderMW( $this->oSearchOptions );

		if ( $this->oSearchOptions->getOptionBool( 'bExtendedForm' ) ) {
			$aMonitor['linkToExtendedPageMessageKey'] = 'bs-extendedsearch-specialpage-form-return-to-simple';
			$aMonitor['linkToExtendedPageUri'] = $this->oUriBuilder->buildUri( SearchUriBuilderMW::ALL );

			$oSearchformView->addItem( $this->renderExtendedForm( $oSearchform ) );
			$bShowExtendedForm = true;
		}
		else {
			$aMonitor['linkToExtendedPageMessageKey'] = 'bs-extendedsearch-specialpage-form-expand-to-extended';
			$aMonitor['linkToExtendedPageUri'] = $this->oUriBuilder->buildUri( SearchUriBuilderMW::ALL | SearchUriBuilderMW::EXTENDED );
		}

		$oSearchform->setOptions( $aMonitor );
		$oSearchformView->addItem( $oSearchform );

		BsCore::getInstance( 'MW' )->getAdapter()->get( 'Out' )->addHTML( $oSearchformView->execute() );

		if ( isset( $bShowExtendedForm ) && $bShowExtendedForm == 'true' ) return;

		try {
			$oView = $this->oExtendedsearchBaseMW->renderSpecialpage();
		}
		catch ( BsException $e ) {
			if ( $e->getMessage() == 'redirect' ) return;
			throw $e;
		}

		//		- Initialisieren des Suchmanagers
		//			=> Abgabe des Ablaufs an ExtendedSearch
		//			- Zusammenbau des Requests
		//		- Abfragen: Suchformular oben oder unten
		//		- Ausgabe der views:
		//			- output Suchformular
		//			- output Suchergebnis

		BsCore::getInstance( 'MW' )->getAdapter()->get( 'Out' )->addHTML( $oView->execute() );
	}

}