<?php
/**
 * Renders a single ExtendedSearch facet.
 *
 * Part of BlueSpice for MediaWiki
 *
 * @author     Mathias Scheer <scheer@hallowelt.biz>
 * @author     Markus Glaser <glaser@hallowelt.biz>
 * @author     Stephan Muggli <muggli@hallowelt.biz>
 * @version    $Id: view.ExtendedSearchFacetBox.php 8147 2013-01-16 08:52:16Z smuggli $
 * @package    BlueSpice_Extensions
 * @subpackage ExtendedSearch
 * @copyright  Copyright (C) 2011 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 * @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
 * @filesource
 */

/**
 * This view renders a single ExtendedSearch facet.
 * @package    BlueSpice_Extensions
 * @subpackage ExtendedSearch 
 */
class ViewExtendedSearchFacetBox extends ViewBaseElement {

	/**
	 * Number of checked facets. Used to determine whether overall checkbox should be checked as well
	 * @var int Number of checked facets
	 */
	protected $iEntriesChecked = 0;
	/**
	 * List of checked facets.
	 * @var array List of checked facets.
	 */
	protected $aEntriesChecked = array();
	/**
	 * List of unchecked facets.
	 * @var array List of checked facets.
	 */
	protected $aEntriesUnChecked = array();

	/**
	 * Add facet either to checked or unchecked set.
	 * @param array $dataSet Set of facets.
	 */
	public function addData( array $dataSet ) {
		if ( isset( $dataSet['diff'] ) ) {
			$dataSet['diff'] = ' urldiff="'.htmlspecialchars( $dataSet['diff'], ENT_QUOTES, 'UTF-8' ).'"';
		}
		if ( isset( $dataSet['uri'] ) ) {
			$dataSet['uri'] = htmlspecialchars( $dataSet['uri'], ENT_QUOTES, 'UTF-8' );
		}
		$dataSet['name-and-count'] = ( $dataSet['count'] > 0 )
						? "&nbsp;{$dataSet['name']}&nbsp;({$dataSet['count']})"
						: "&nbsp;<span class=\"bs-es-empty-facet-item\">{$dataSet['name']}&nbsp;(0)</span>";
		if ( isset( $dataSet['checked'] ) && ( $dataSet['checked'] === true ) ) {
			$this->iEntriesChecked++;
			$dataSet['checked'] = ' checked="checked"';
			$this->aEntriesChecked[] = $dataSet;

		}
		else {
			$dataSet['checked'] = '';
			$this->aEntriesUnChecked[] = $dataSet;
		}
	}

	/**
	 * Template for single facet item.
	 */
	public function setTemplate( $template ) {
		$out = '<div class="facetBarEntry">';
		$out .= '<input type="checkbox"{checked} onchange="document.location=\'{uri}\';"{diff} class="searchcheckbox" />';
		$out .= '<label>{name-and-count}</label>';
		$out .= '</div>';
		parent::setTemplate( $out );
	}

	/**
	 * This method actually generates the output
	 * @return string HTML output
	 */
	public function execute( $params = false ) {
		$this->setTemplate('');
		$titleBar = '<div class="facet-title bs-extendedsearch-default-textspacing">';
		if ( $this->iEntriesChecked > 0 ) {
			$checked = ' checked="checked"';
			$uri = $this->getOption( 'uri-facet-delete' );
		} else {
			$checked = '';
			$uri = $this->getOption( 'uri-facet-all' );
		}
		$onchange = htmlspecialchars( $uri, ENT_QUOTES, 'UTF-8' );
		$titleBar .= '<input type="checkbox"' . $checked .
				" onchange=\"document.location='{$onchange}'\"" .
				' urldiff="'.$this->getOption( 'uri-facet-all-diff' ).'"></input>';
		//'<input type="checkbox"'.$checked.' onchange="document.location=\''.$uri.'\';" urldiff="'.$this->getOption('uri-facet-all-diff').'" />';
		$titleBar .= '&nbsp;<label>' . wfMessage( $this->getOption( 'i18n-key-facet-title' ) )->plain(). '</label>';
		$titleBar .= '</div>';
		$this->addCompleteDataset( $this->aEntriesChecked );
		$body = parent::execute();
		if ( !empty( $this->aEntriesChecked ) && !empty( $this->aEntriesUnChecked ) )
			$body .= BsXml::simpleTag( 'div', array( 'class' => 'bs-extendedsearch-facetbox-separator-checked-unchecked' ), '', false );
		$this->addCompleteDataset( $this->aEntriesUnChecked );
		$body .= parent::execute();
		$body .= '<div class="bs-extendedsearch-facetend"></div><div class="facetBarEntry bs-extendedsearch-facetbox-more"></div>';
		$body = BsXml::simpleTag( 'div', array( 'class' => 'bs-extendedsearch-facetbox-container' ), $body );
		return $titleBar.$body;
	}

}