<?php
/**
 * Renders the ExtendedSearch search form.
 *
 * Part of BlueSpice for MediaWiki
 *
 * @author     Mathias Scheer <scheer@hallowelt.biz>
 * @author     Markus Glaser <glaser@hallowelt.biz>
 * @author     Stephan Muggli <muggli@hallowelt.biz>
 * @version    $Id: view.ExtendedSearchFormPage.php 8003 2013-01-08 08:30:51Z smuggli $
 * @package    BlueSpice_Extensions
 * @subpackage ExtendedSearch
 * @copyright  Copyright (C) 2011 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 * @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
 * @filesource
 */

/**
 * This view renders the ExtendedSearch search form.
 * @package    BlueSpice_Extensions
 * @subpackage ExtendedSearch 
 */
class ViewExtendedSearchFormPage extends ViewBaseElement {

	/**
	 * Fetches a View for extended options.
	 * @param string $key Type key of data to be searched
	 * @param string $headlineI18Nned Title of the options section
	 * @return ViewSearchExtendedOptionsForm View of extended options for a certain key.
	 */
	public function &getOptionsForm( $key, $headlineI18Nned ) {
		$item = new ViewSearchExtendedOptionsForm();
		$item->setOption( 'headlineI18N', $headlineI18Nned );
		$this->addItem( $item, $key );
		return $item;
	}

	/**
	 * This method actually generates the output
	 * @return string HTML output
	 */
	public function execute( $params = false ) {
		$aSearchBoxKeyValues = array();
		wfRunHooks( 'FormDefaults', array( $this, &$aSearchBoxKeyValues ) );

		$sInputFields = '';
		if ( isset( $aSearchBoxKeyValues['HiddenFields'] ) && is_array( $aSearchBoxKeyValues['HiddenFields'] ) ) {
			foreach ( $aSearchBoxKeyValues['HiddenFields'] as $key => $value ) {
				$sInputFields .= BsXml::simpleTag( 'input', array(
						'type'  => 'hidden',
						'name'  => $key,
						'value' => $value ), '' );
			}
		}

		$sInputFields .= BsXml::simpleTag( 'input', array(
				'name'  => $aSearchBoxKeyValues['SearchTextFieldName'],
				'type'  => 'text',
				'title' => $aSearchBoxKeyValues['SearchTextFieldTitle'],
				'id'    => 'bs-extendedsearch-inputfieldtext-specialpage',
				'value' => ( isset( $aSearchBoxKeyValues['SearchTextFieldText'] ) )
					? BsSearchService::sanitzeSearchString( $aSearchBoxKeyValues['SearchTextFieldText'] )
					: $aSearchBoxKeyValues['SearchTextFieldDefaultText'],
				'defaultvalue' => $aSearchBoxKeyValues['SearchTextFieldDefaultText'],
				'accesskey'    => 'f' ), '' );

		$sInputFields .= BsXml::simpleTag( 'input', array(
				'type'  => 'hidden',
				'name'  => 'search_scope',
				'value' => $aSearchBoxKeyValues['DefaultKeyValuePair'][1],
				'id'    => 'bs-search-button-hidden-specialpage' ), '' );

		$sImageTitleButton = BsXml::simpleTag( 'div', array(
				'id'    => 'bs-extendedsearch-titlebuttonimage-specialpage',
				'class' => 'bs-extendedsearch-buttonimage-specialpage'), '', false );

		$sInputFields .= BsXml::simpleTag( 'button', array(
				'type'  => 'button',
				'title' => $aSearchBoxKeyValues['TitleKeyValuePair'][1],
				'id'    => 'bs-search-button-specialpage',
				'class' => 'bs-search-button-specialpage' ), $sImageTitleButton.$aSearchBoxKeyValues['SubmitButtonTitle'] );

		$sImageFulltextButton = BsXml::simpleTag( 'div', array(
				'id'    => 'bs-extendedsearch-fulltextbuttonimage-specialpage',
				'class' => 'bs-extendedsearch-buttonimage-specialpage' ), '', false );

		$sInputFields .= BsXml::simpleTag( 'button', array(
				'type'  => 'button',
				'title' => $aSearchBoxKeyValues['FulltextKeyValuePair'][1],
				'id'    => 'bs-search-fulltext-specialpage',
				'class' => 'bs-search-button-specialpage' ), $sImageFulltextButton.$aSearchBoxKeyValues['SubmitButtonFulltext'] );

		$sLinkToExtendedPage = BsXml::simpleTag( 'a', array(
				'href' => $this->getOption( 'linkToExtendedPageUri' ),
				'id'   => 'bs-extendedsearch-linktoextendedpage' ),
			wfMessage( $this->getOption( 'linkToExtendedPageMessageKey' ) )->plain() );

		if ( empty( $this->_mItems ) ) {
			$sDivSearchDomains = '';
		} else {
			$itemsOut = '';
			foreach ( $this->_mItems as $item ) {
				$itemsOut .= $item->execute();
			}
			$sDivSearchDomains = BsXml::simpleTag( 'div', array(
					'class' => 'bs-extendedsearch-domaindiv' ),
				$itemsOut );
		}

		$aFormAttributes = array(
				'class'  => 'bs-search-form',
				'id'     => 'bs-extendedsearch-form-specialpage',
				'action' => $aSearchBoxKeyValues['SearchDestination'],
				'method' => $aSearchBoxKeyValues['method'] );

		if ( !isset( $aSearchBoxKeyValues['SearchTextFieldText'] ) )
			$aFormAttributes['onsubmit'] = 'if (document.getElementById(\'bs-extendedsearch-inputfieldtext-specialpage\').value == \''.$aSearchBoxKeyValues['SearchTextFieldDefaultText'].'\') return false;';

		$sForm = BsXml::simpleTag( 'form', $aFormAttributes, $sInputFields.$sLinkToExtendedPage .$sDivSearchDomains );

		return $sForm;
	}

}