<?php
/**
 * Renders a single ExtendedSearch search result.
 *
 * Part of BlueSpice for MediaWiki
 *
 * @author     Mathias Scheer <scheer@hallowelt.biz>
 * @author     Markus Glaser <glaser@hallowelt.biz>
 * @author     Stephan Muggli <muggli@hallowelt.biz>
 * @version    $Id: view.ExtendedSearchResultEntry.php 8147 2013-01-16 08:52:16Z smuggli $
 * @package    BlueSpice_Extensions
 * @subpackage ExtendedSearch
 * @copyright  Copyright (C) 2011 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 * @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
 * @filesource
 */

/**
 * This view renders a single ExtendedSearch search result.
 * @package    BlueSpice_Extensions
 * @subpackage ExtendedSearch 
 */
class ViewExtendedSearchResultEntry extends ViewBaseElement {

	/**
	 * Preprocesses highlight snippets as they come from Solr.
	 * @param array $snippets List of snippets with search text occurrences.
	 * @return string Modified snipped.
	 */
	protected function processSnippets( array $snippets ) {
		$out = '';

		foreach ( $snippets as $frag ) {
			$frag = strip_tags( $frag, '<em>' );              // now that everything is clear (no tags)
			$out .= "{$frag}<br />";
		}
		return $out;
	}

	/**
	 * Add data for a single search result item.
	 * @param array $aDataSet Key value pairs of search result item properties.
	 * @return ViewBaseElement Returns an reference to itself to enable method chaining.
	 */
	public function addData( array $aDataSet ) {
		if ( isset( $aDataSet['highlightsnippets'] ) && is_array( $aDataSet['highlightsnippets'] ) ) {
			$aDataSet['highlightsnippets'] = $this->processSnippets( $aDataSet['highlightsnippets'] );
		}
		return parent::addData( $aDataSet );
	}

	/**
	 * Template for a single search entry.
	 */
	public function setTemplate( $template ) {
		$template = <<<HEREDOC
<div class="search-wrapper">
	<div class="search-hit-title">
		<div class="search-left">
<div style="position: absolute">
{searchicon}</div>
			<span class="bs-extendedsearch-result-entry-margin-left">
				{searchlink}
			</span>
HEREDOC;
		if ( $this->getOption( 'showpercent' ) )
				$template .= <<<HEREDOC
			<span class="hits-percent">
				({scorepercent}%)
			</span>
HEREDOC;
		if ( $this->getOption( 'showtime' ) )
				$template .= <<<HEREDOC
			<div class="search-titlebar-right-time bs-extendedsearch-result-entry-margin-left">
				&nbsp;({timestamp})
			</div>
HEREDOC;
		$template .= <<<HEREDOC
		</div>
		<div class="search-titlebar-right">
			{catstr}
		</div>
	</div>
HEREDOC;
		if ( $this->getOption( 'highlightsnippets' ) )
				$template .= <<<HEREDOC
			<span class="search-hit-text search-left">
				{highlightsnippets}
			</span>
HEREDOC;
		$template .= <<<HEREDOC
</div>
HEREDOC;
		parent::setTemplate( $template );
	}

	/**
	 * This method actually generates the output
	 * @return string HTML output
	 */
	public function execute( $param = false ) {
		$this->setTemplate('');
		return parent::execute();
	}

}