BsInsertMagicI18N = {
	dlg_title: 'Insert Tag or MagicWord',
	type_tags: 'Tags',
	type_variables: 'Variables',
	type_switches:'Behavior switches',
	btn_ok: 'Ok',
	btn_cancel: 'Cancel',
	btn_preview: 'Preview',
	label_first: '1. Choose Tag or MagicWord',
	label_second: '2. Modify code',
	label_third: '3. Check result',
	label_desc: 'Description'
}

BsInsertMagicDialog = Ext.extend(Ext.Window, {
	id: 'bs-InsertMagic-dlg-window',
	modal: true,
	closeAction: 'hide',
	width: 600,
	height: 450,
	layout: 'border',
	instance: null,
	caller: null,
	data: {
		selection: false,
		type:false,
		id: false,
		name: false
	},

	initComponent: function() {
		this.setTitle( BsInsertMagicI18N.dlg_title );

		this.btnSave = new Ext.Button({
			id: 'bs-InsertMagic-btn-ok',
			text: BsInsertMagicI18N.btn_ok,
			handler: this.onBtnSaveClicked,
			scope: this
		});

		this.btnCancel = new Ext.Button({
			id: 'bs-InsertMagic-btn-cancel',
			text: BsInsertMagicI18N.btn_cancel,
			handler: this.onBtnCancelClicked,
			scope: this
		});
		
		this.btnPreview = new Ext.Button({
			id: 'bs-InsertMagic-btn-preview',
			text: BsInsertMagicI18N.btn_preview,
			handler: this.onBtnPreviewClicked,
			scope: this
		});
		
		this.buttons = [
			//this.btnPreview,
			this.btnSave,
			this.btnCancel
		];
		
		var typesArray = [
			[ 'tag',      BsInsertMagicI18N.type_tags ],
			[ 'switch',   BsInsertMagicI18N.type_switches ],
			[ 'variable', BsInsertMagicI18N.type_variables ]
		];
		//TODO: Make hook?
		
		//HINT: http://stackoverflow.com/questions/4834285/extjs-combobox-acting-like-regular-select
		this.cmbType = new Ext.form.ComboBox({
			id: 'bs-InsertMagic-cmb-type',
			mode: 'local',
			triggerAction: 'all',
			editable: false,
			readonly: true,
			allowBlank: false,
			forceSelection: true,
			value:'tag', //default selection
			store: typesArray
		});
		this.cmbType.addListener( 'select', this.onTypeSelected, this );
		
		this.tagsStore = new Ext.data.JsonStore({
			url: wgScriptPath + '/index.php',
			baseParams: {
				action: 'ajax',
				rs: 'InsertMagic::ajaxGetData'
			},
			autoLoad: true,
			idProperty: 'id',
			root: 'result',
			fields: ['id', 'type', 'name', 'desc', 'code' ],
			sortInfo: {
				field: 'id',
				direction: 'ASC'
			},
			listeners: {
				'load': {
					fn: this.onStoreLoad,
					scope: this
				}
			}
		});
		
		this.tagsGrid = new Ext.grid.GridPanel({
			title: '',
			id: 'bs-InsertMagic-grid-tag',
			sm: new Ext.grid.RowSelectionModel({ singleSelect: true }),
			store: this.tagsStore,layout: 'fit',
			loadMask: true,
			colModel: new Ext.grid.ColumnModel({
				columns: [
					{
						id: 'name',
						sortable: true,
						dataIndex: 'name'
					}
				]
			}),
			viewConfig: { forceFit: true },
			/*view:new Ext.grid.GroupingView({
				forceFit:true,
				groupTextTpl: '{text}'
			}),*/
			border: true,
			columnLines: false,
			enableHdMenu: false,
			stripeRows: true,
			hideHeaders: true,
			flex: 1,
			style: 'padding-top: 5px'
		});
		this.tagsGrid.getSelectionModel().addListener( 'rowselect', this.onRowSelect, this );
		
		this.syntaxTextArea = new Ext.form.TextArea({
			id: 'bs-InsertMagic-textarea-syntax',
			hideLabel: true,
			name: 'syntaxTextArea',
			flex: 1,
			style: 'background-color: #EFF4FF;'
		});
		
		this.previewPanel = new Ext.Panel({
			id: 'bs-InsertMagic-panel-preview',
			border: true,
			flex: 1,
			bodyStyle: 'padding:5px;',
			autoScroll: true
		});
		
		this.descPanel = new Ext.Panel({
			id: 'bs-InsertMagic-panel-desc',
			border: true,
			flex: 1,
			bodyStyle: 'padding:5px; background-color: #F4F4F4;',
			autoScroll: true
		});
		
		this.pnlWest = new Ext.Panel({
			region: 'west',
			width: 250,
			border: false,
			padding: 5,
			layout: {
				//HINT: http://dev.sencha.com/deploy/ext-3.3.1/examples/form/vbox-form.js
				type: 'vbox',
				align: 'stretch' // Child items are stretched to full width
			},
			items: [
				new Ext.form.Label({ text: BsInsertMagicI18N.label_first }),
				this.cmbType,
				this.tagsGrid,
				new Ext.form.Label({ text: BsInsertMagicI18N.label_desc, style: 'padding-top: 10px' }),
				this.descPanel
			]
		});
		
		this.pnlCenter = new Ext.Panel({
			region: 'center',
			border: false,
			padding: 5,
			layout: {
				type: 'vbox',
				align: 'stretch'
			},
			items:[
				new Ext.form.Label({ text: BsInsertMagicI18N.label_second }),
				this.syntaxTextArea/*,
				new Ext.form.Label({ text: BsInsertMagicI18N.label_third, style: 'padding-top: 10px' }),
				this.previewPanel*/
			]
		});
		
		this.items = [
			this.pnlWest,
			this.pnlCenter
		]
		
		BsInsertMagicDialog.superclass.initComponent.call(this);
	},
	
	show: function( caller ) {
		this.caller = caller;
		if( typeof this.caller == 'object' ) { //VisualEditor
			this.initFromTinyMCE();
		}
		else { //WikiCode Editor
			this.initFromWikiTextArea();
		}
		BsInsertMagicDialog.superclass.show.call(this);
	},
	
	initFromTinyMCE: function() {
		var node = this.caller.selection.getNode();
		this.data.selection = this.caller.selection.getBookmark();
		this.data.id    = node.getAttribute('data-bs-id');
		this.data.type = node.getAttribute('data-bs-type');
		this.data.name = node.getAttribute('data-bs-name');
		var currentCode = '';
		//var idx = '';
		//TODO: dublicapte code!
		//TODO: Laufzeitproblem: onShow ist Store noch nicht unbedingt geladen und Grid nicht unbedingt gerendert. --> selection speichern und StoreOnLoad Fokus und Selection setzen!
		if( this.data.type == 'template' ) {
			currentCode = this.caller.plugins.hwcode._templates[this.data.id];
			this.cmbType.setValue( 'variable' );
			//idx = this.variablesStore.find(
			//	'id',
			//	'{{'+node.getAttribute( 'data-bs-template' )+'}}'
			//);
			//this.variablesGrid.getSelectionModel().selectRow(idx);
		}
		else if(this.data.type == 'tag') {
			currentCode = this.caller.plugins.hwcode._specialtags[this.data.id];
			this.cmbType.setValue( 'tag' );
			//idx = this.variablesStore.find(
			//	'id',
			//	node.getAttribute( 'data-bs-tag' ) //"bs:" prefix?!
			//);
			//this.tagsGrid.getSelectionModel().selectRow(idx);
			//TODO: Read-in attributes via DOM
			//var x= $(this.caller.plugins.hwcode._specialtags[this.data.id]);
			//$.each(x[0].attributes, function(i){
			//    console.log(x[0].attributes[i].name);
			//    console.log(x[0].attributes[i].value);
			//});
		}
		else if(this.data.type == 'switch') {
			currentCode = this.caller.plugins.hwcode._switches[this.data.id];
			this.cmbType.setValue( 'switch' );
			//idx = this.variablesStore.find(
			//	'id',
			//	'__'+node.getAttribute( 'data-bs-switch' )+'__'
			//);
			//this.switchesGrid.getSelectionModel().selectRow(idx);
			//this.variablesGrid.getView().focusRow(idx)
		}
		this.syntaxTextArea.setValue( currentCode );
	},
	
	initFromWikiTextArea: function() {
		BsCore.saveScrollPosition();
		var currentCode = BsCore.saveSelection();
		this.syntaxTextArea.setValue( currentCode );
	},
	
	onBtnSaveClicked: function( oSender, oEvent ) {
		if( typeof this.caller == 'object' ){ //VisualEditor
			this.saveToTinyMCE();
		}
		else { //WikiCode Editor
			this.saveToWikiTextArea();
		}
	},
	
	onStoreLoad: function( store, records, options ) {
		this.tagsStore.filter( 'type', 'tag'); //just initial
	},
	
	onTypeSelected: function( combo, record, index ){
		this.tagsStore.filter( 'type', record.data.field1 ); //field1 is because of ArrayStore. Could be optimized.
	},

	saveToTinyMCE: function() {
		this.caller.selection.moveToBookmark(this.data.selection);
		var code = this.syntaxTextArea.getValue();
		var html = '<span id="{0}" class="mceNonEditable {1}" data-bs-name="{2}" data-bs-type="{3}" data-bs-id="{4}">{5}</span>';

		this.setCurrentTypeFromText( code );
		if( this.data.type == 'switch' ) {
			if( this.data.id ) {
				this.caller.plugins.hwcode._switches[this.data.id] = code;
			}
			else {
				this.data.id = this.caller.plugins.hwcode._switches.length;
				this.caller.plugins.hwcode._switches.push(code);
			}
			code = html.format(
				'hw_switch:@@@SWT'+this.data.id+'@@@',
				'switch',
				this.data.name,
				'switch',
				this.data.id,
				'__ '+this.data.name+' __'
			);
		}
		else if( this.data.type == 'variable') {
			if( this.data.id ) {
				this.caller.plugins.hwcode._templates[this.data.id] = code;
			}
			else {
				this.data.id = this.caller.plugins.hwcode._templates.length;
				this.caller.plugins.hwcode._templates.push(code);
			}
			code = html.format(
				'hw_template:@@@TPL'+this.data.id+'@@@',
				'template',
				this.data.name,
				'template',
				this.data.id,
				'{{ '+this.data.name+' }}'
			);
		}
		else if( this.data.type == 'tag' ) {
			if( this.data.id ) {
				this.caller.plugins.hwcode._specialtags[this.data.id] = code;
			}
			else {
				this.data.id = this.caller.plugins.hwcode._specialtags.length;
				this.caller.plugins.hwcode._specialtags.push(code);
			}
			code = html.format(
				'hw_specialtag:@@@ST'+this.data.id+'@@@',
				'tag',
				this.data.name,
				'tag',
				this.data.id,
				'&lt; '+this.data.name+' &gt;'
			);
		}
		//this.caller.activeEditor.dom.setOuterHTML(this.data.link, ""); //This is important for IE
		//this.caller.dom.remove( this.caller.selection.getNode() );
		this.caller.dom.remove(this.caller.selection.getNode()); // remove old node to ensure that new one is not place within
		this.caller.execCommand('mceInsertRawHTML', false, code );
		this.hide();
	},
	
	saveToWikiTextArea: function() {
		BsCore.restoreSelection( this.syntaxTextArea.getValue() );
		BsCore.restoreScrollPosition();
		this.hide();
	},
	
	onBtnCancelClicked: function( oSender, oEvent ) {
		this.hide();
	},
	
	onBtnPreviewClicked: function( oSender, oEvent ) {
		//this.previewPanel;
	},
	
	onRowSelect: function( selectionModel, rowIndex, selectedRecord ) {
		var data = {
			desc : selectedRecord.get('desc'),
			type : selectedRecord.get('type')
		}
		this.data.type = data.type;
		this.data.name = selectedRecord.get('name');

		this.setCommonFields( selectedRecord.get('code'), data )
	},

	setCommonFields: function( text, data ) {
		this.descPanel.update(data.desc);
		this.syntaxTextArea.setValue( text );
		this.syntaxTextArea.focus();
		
		var start = text.indexOf('"') + 1;
		var end   = text.indexOf('"', start );
		if( data.type != 'tag' ) {
			start = start - 1;
			end = end + 1;
		}
		this.syntaxTextArea.selectText(start, end);
	},
	
	setCurrentTypeFromText: function( text ) {
		//What about tags containing variables? Or variables containing tags?
		if( text.match(/<(.*?)>/gmi) ) { //TODO: find better test
			this.data.type = 'tag';
			return;
		}
		else if( text.match(/\{\{(.*?)\}\}/gmi) ) {
			this.data.type = 'variable';
			return;
		}
		else if( text.match(/__(.*?)__/gmi) ){
			this.data.type = 'switch';
			return;
		}
		else {
			this.data.type = false;
		}
	}
});

BsInsertMagicDialog.singleton = function(){
	if( BsInsertMagicDialog.instance == null ) {
		BsInsertMagicDialog.instance = new BsInsertMagicDialog();
	}
	return BsInsertMagicDialog.instance;
}

$(document).bind('hwactions-init', function( event, plugin, buttons, commands ){
	buttons.push({
		buttonId: 'hwmagic',
		buttonConfig: {
			title : BsInsertMagicI18N.dlg_title,
			cmd : 'mceHwMagic',
			image : wgScriptPath+'/bluespice-mw/ext/InsertMagic/images/btn_insertmagic.png'
		}
	});
	commands.push({
		commandId: 'mceHwMagic',
		commandCallback: function() {
			BsInsertMagicDialog.singleton().show( this ); //this --> TinyMCE instance
		}
	});
});

$(document).bind('hwbehavior-_setDisabled', function( event, plugin, setValue, leaveEnabledIds, selectedNode ){
	leaveEnabledIds.push( plugin.editor.editorId+'_'+'hwmagic');
});