/**
 * InterWikiManager extension
 *
 * @author     Markus Glaser <glaser@hallowelt.biz>
 * @author     Sebastian Ulbrich
 * @version    0.1 beta
 * @version    $Id: InterWikiManager.js 2751 2011-07-04 08:41:54Z mglaser $
 * @package    Bluespice_Extensions
 * @subpackage InterWikiLinks
 * @copyright  Copyright (C) 2011 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 * @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
 * @filesource
 */
 
 /* Changelog
  * v0.1
  * - initial commit
  */
 
/**
 * Base class for all InterWikiLinks related methods and properties
 */
BsInterWikiManager = {
	/**
	 * Internationalisation
	 * @var object list of i18n key value pairs
	 */
	i18n: {
		headerPrefix: 'Prefix',
		headerUrl: 'Url',
		headerActions: 'Actions',
		tipEditInterWikiLink: 'Edit interwiki link',
		tipDeleteInterWikiLink: 'Delete interwiki link',
		btnOk: 'Ok',
		btnCancel: 'Cancel',
		titleError: 'Error',
		unknownError: 'An unknown error occurred.',
		titleAddInterWikiLink: 'Add interwiki link',
		titleEditInterWikiLink: 'Edit interwiki link',
		labelPrefix: 'Prefix',
		labelUrl: 'Url',
		titleDeleteInterWikiLink: 'Delete interwiki link',
		confirmDeleteInterWikiLink: 'Are you sure, you want delete this interwiki link?',
		showEntries: 'Displaying {0} - {1} of {2}'
	},
	/**
	 * Basic canvas to be rendered to
	 * @var Ext.grid.GridPanel
	 */
	panel: false,
	/**
	 * Store for interwiki data, displayed in grid.
	 * @var Ext.data.JsonStore
	 */
	store: new Ext.data.JsonStore({
		autoDestroy: true,
		url: BlueSpice.buildRemoteString('InterWikiLinks', 'getInterWikiLinks'),
		storeId: 'BsInterWikiManagerStore',
		root: 'iwlinks',
		fields: [
			'prefix',
			'url',
		],
		totalProperty: 'totalCount'
	}),
	/**
	 * Renders the interwiki link grid and fills it with data
	 */
	show: function() {
		if(!this.panel) {
			this.panel = new Ext.grid.GridPanel({
				renderTo: 'InterWikiManagerGrid',
				id: 'imGrid',
				plugins: (Ext.isIE ? [] : ['fittoparent']),
				loadMask: true,  
				store: this.store,
				tbar: new Ext.Toolbar({
					style: {
						backgroundColor: '#FFFFFF',
						backgroundImage: 'none'
					},
					items: [{
						tooltip: this.i18n.titleAddInterWikiLink,
						icon: stylepath+'/bluespice/bs-m_add.png',
						iconCls: 'btn44',
						id: 'btnInterWikiAdd',
						handler: function(btn, ev) {
							this.showEditInterWikiLink(false, false);
						},
						scope: this
					}, {
						xtype: 'tbseparator',
						height: 44,
						cls: 'sep44'
					}, {
						tooltip: this.i18n.titleEditDetails,
						icon: stylepath+'/bluespice/bs-um_config-0.png',
						iconCls: 'btn44',
						id: 'btnInterWikiEdit',
						disabled: true,
						handler: function(btn, ev) {
							this.showEditInterWikiLink(this.panel.getSelectionModel().getSelected(), true);
						},
						scope: this
					}, {
						xtype: 'tbseparator',
						height: 44,
						cls: 'sep44'
					}, {
						tooltip: this.i18n.titleDeleteInterWikiLink,
						icon: stylepath+'/bluespice/bs-m_delete-0.png',
						iconCls: 'btn44',
						id: 'btnInterWikiRemove',
						disabled: true,
						handler: function(btn, ev) {
							if (this.panel.getSelectionModel().getSelected() ) {
								this.showDeleteInterWikiLink(this.panel.getSelectionModel().getSelected());
							}
						},
						scope: this
					}
					]
				}),
				bbar: new Ext.PagingToolbar({
					displayMsg: this.i18n.showEntries,
					store: this.store,
					displayInfo: true,
					pageSize: 25,
					prependButtons: true
				}),
				colModel: new Ext.grid.ColumnModel({
					columns: [
					{
						id: 'prefix',
						header: this.i18n.headerPrefix,
						sortable: true,
						width: 60,
						dataIndex: 'prefix'
					}, {
						header: this.i18n.headerUrl,
						sortable: true,
						dataIndex: 'url'
					}, {
						header: this.i18n.headerActions,
						xtype: 'actioncolumn',
						width: 40,
						cls: 'hideAction',
						items: [
						{
							icon: stylepath+'/bluespice/bs-um_config_tn.png',
							tooltip: this.i18n.tipEditInterWikiLink,
							handler: function(grid, rowIndex, colIndex) {
								this.showEditInterWikiLink(this.store.getAt(rowIndex), true);
							},
							scope: this
						}, {
							icon: stylepath+'/bluespice/bs-m_delete_tn.png',
							tooltip: this.i18n.tipDeleteInterWikiLink,
							handler: function(grid, rowIndex, colIndex) {
								this.showDeleteInterWikiLink(this.store.getAt(rowIndex));
							},
							scope: this
						}
						],
						sortable: false
					}
					]
				}),
				viewConfig: {
					forceFit: true
				},
				sm: new Ext.grid.RowSelectionModel({
					singleSelect: true,
					listeners: {
						'selectionchange': function( sm ) {
							if (sm.hasSelection()) {
								Ext.getCmp('btnInterWikiEdit').setIcon(stylepath+'/bluespice/bs-um_config.png');
								Ext.getCmp('btnInterWikiEdit').enable();
								Ext.getCmp('btnInterWikiRemove').setIcon(stylepath+'/bluespice/bs-m_delete.png');
								Ext.getCmp('btnInterWikiRemove').enable();
							} else {
								Ext.getCmp('btnInterWikiEdit').setIcon(stylepath+'/bluespice/bs-um_config-0.png');
								Ext.getCmp('btnInterWikiEdit').disable();
								Ext.getCmp('btnInterWikiRemove').setIcon(stylepath+'/bluespice/bs-m_delete-0.png');
								Ext.getCmp('btnInterWikiRemove').disable();
							}
						}
					}

				}),
				border: false,
				columnLines: true,
				enableHdMenu: false,
				stripeRows: true,
				autoHeight: true
			});
			this.store.load();
		}
		this.panel.show();
	},
	/**
	 * Reference to the InterWiki Links Dialogue Window
	 * @var Window Ext.Window object
	 */
	window: false,
	/**
	 * Displays the interwiki link dialoge
	 * @param record Ext.data.Record an entry in the interwiki link store
	 * @param bool editmode Create new link (false) or edit existing link (true)
	 */
	showEditInterWikiLink: function(record, editmode) {
		this.window = new Ext.Window({
			title: editmode?this.i18n.titleEditInterWikiLink:this.i18n.titleAddInterWikiLink,
			modal: true,
			width: 350,
			oldprefix: record?record.get('prefix'):'',
			items: [
			{
				xtype: 'form',
				id: 'panelFormEditInterWikiLink',
				labelWidth: 50,
				padding: 10,
				border: false,
				items: [
				{
					xtype: 'field',
					inputType: 'text',
					fieldLabel: this.i18n.labelPrefix,
					name: 'iweditprefix',
					id: 'iweditprefix',
					allowBlank: false,
					width: 260,
					value: record?record.get('prefix'):''
				}, {
					xtype: 'field',
					inputType: 'text',
					fieldLabel: this.i18n.labelUrl,
					name: 'iwediturl',
					allowBlank: false,
					width: 260,
					value: record?record.get('url'):''
				}
				]
			}
			],
			buttons: [
			{
				text: this.i18n.btnOk,
				handler: function(){
					Ext.getCmp('panelFormEditInterWikiLink').getForm().doAction('submit', {
						clientValidation: false,
						submitEmptyText: false,
						method: 'post',
						url: BlueSpice.buildRemoteString('InterWikiLinks', 'doEditInterWikiLink', {"iweditmode":editmode,"iweditoldprefix":this.window.oldprefix}),
						success: function(form, action) {
							this.window.close();
							if(action.result.messages.length) {
								BsInterWikiManager.panel.store.reload();
								var tmp = '';
								for(i in action.result.messages) {
									if(typeof(action.result.messages[i]) != 'string') {
										continue;
									}
									tmp = tmp + action.result.messages[i] + '<br />';
								}
								Ext.Msg.alert('Status', tmp);
							}
						},
						failure: function(form, action) {
							if(action.result.messages.length) {
								var tmp = '';
								for(i in action.result.messages) {
									if(typeof(action.result.messages[i]) != 'string') {
										continue;
									}
									tmp = tmp + action.result.messages[i] + '<br />';
								}
								Ext.Msg.alert('Status', tmp);
							}
						},
						scope: this
					});
				},
				scope: this
			}, {
				text: this.i18n.btnCancel,
				handler: function(){
					this.ownerCt.ownerCt.close();
				}
			}
			]
		});
		this.window.show();
	},
	/**
	 * Displays the interwiki link deletion dialoge
	 * @param record Ext.data.Record an entry in the interwiki link store
	 */
	showDeleteInterWikiLink: function(record) {
		confirmMessage = this.i18n.confirmDeleteInterWikiLink 
				+ "<br>" + "<br>"
				+ "<b>" + this.i18n.labelPrefix + ":</b> " + record.get('prefix')
				+ "<br>"
				+ "<b>" + this.i18n.labelUrl + ":</b> " + record.get('url')
				+ "<br>";
		Ext.Msg.confirm(this.i18n.titleDeleteInterWikiLink, confirmMessage, function(btn) {
			if(btn == 'yes') {
				Ext.Ajax.request({
					method: 'post',
					params: {
						deleteprefix: record.get('prefix')
					},
					url: BlueSpice.buildRemoteString('InterWikiLinks', 'doDeleteInterWikiLink'),
					success: function(response, opts) {
						var obj = Ext.decode(response.responseText);
						BsInterWikiManager.panel.store.reload();
						if(obj.messages.length) {
							var tmp = '';
							for(i in obj.messages) {
								if(typeof(obj.messages[i]) != 'string') {
									continue;
								}
								tmp = tmp + obj.messages[i] + '<br />';
							}
							Ext.Msg.alert('Status', tmp);
						}
					},
					failure: function(response, opts) {
						// TODO MRG (04.07.11 10:41): No console logs, since they break IE
						//console.log('server-side failure with status code ' + response.status);
					},
					scope: this
				})
			}
		})
	}

}

Ext.onReady(function() {
	BsInterWikiManager.show();
}, window, { delay: 500 });