<?php
/**
 * NamespaceCss extension for BlueSpice
 *
 * Use different CSS-Styles in Namespaces by add page MediaWiki:'NAMESPACENAME'_css
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * This file is part of BlueSpice for MediaWiki
 * For further information visit http://www.blue-spice.org
 *
 * @author     Patric Wirth <wirth@hallowelt.biz>
 * @version    1.20.0
 * @version    $Id: NamespaceCss.class.php 9690 2013-06-12 10:13:02Z pwirth $
 * @package    Bluespice_Extensions
 * @subpackage NamespaceCss
 * @copyright  Copyright (C) 2011 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 * @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
 * @filesource
 */

// Last review MRG (07.11.11 18:22)

// TODO WP (09.11.2011): Wont work in namespace special!

class NamespaceCss extends BsExtensionMW {

	public function __construct() {
		wfProfileIn( 'BS::NamespaceCss_construct' );
		global $wgExtensionMessagesFiles;
		$wgExtensionMessagesFiles['NamespaceCss'] = dirname( __FILE__ ) . '/NamespaceCss.i18n.php';

		// Base settings
		$this->mExtensionFile = __FILE__;
		$this->mExtensionType = EXTTYPE::VARIABLE;
		$this->mInfo = array(
			EXTINFO::NAME        => 'NamespaceCss',
			EXTINFO::DESCRIPTION => 'Diff styles for namespaces in MW.',
			EXTINFO::AUTHOR      => 'Robert Vogel, Tobias Weichart, Patric Wirth',
			EXTINFO::VERSION     => '1.21.0 ($Rev: 9690 $)',
			EXTINFO::STATUS      => 'stable',
			EXTINFO::URL         => 'http://www.hallowelt.biz',
			EXTINFO::DEPS        => array( 'bluespice' => '1.21.0' )
		);

		$this->mExtensionKey = 'MW::NamespaceCss';
		BsConfig::registerVar( 'MW::NamespaceCss::Use_NSC', true, BsConfig::LEVEL_PUBLIC|BsConfig::TYPE_BOOL, 'bs-namespacecss-pref-Use_NSC', 'toggle' );
		wfProfileOut( 'BS::NamespaceCss_construct' );
	}

	public function initExt() {
		wfProfileIn( 'BS::'.__METHOD__ );
		if(BsConfig::get( 'MW::NamespaceCss::Use_NSC' )) {
			$this->setHook( 'OutputPageBeforeHTML' );
		}
		wfProfileOut( 'BS::'.__METHOD__ );
	}

	public function onOutputPageBeforeHTML( $oParserOutput, $sText ) {
		global $wgTitle;

		//TODO: use MWNamespace::getCanonicalNamespaces() instead (r3836)
		$wgContLang		= $this->mAdapter->get('ContLang');
		$aNamespaces	= $wgContLang->getNamespaces();
		$iCurrentNs		= $wgTitle->getNamespace();

		if( $wgTitle->isTalkPage() ) $iCurrentNs--;
		$oTitle = Title::newFromText( $aNamespaces[$iCurrentNs].'_css', NS_MEDIAWIKI );

		if( $oTitle->exists() ) {
			$this->registerStyleSheet( $oTitle->getLocalUrl( array( 'action' => 'raw', 'ctype' => 'text/css' ) ), false, 'MW::NamespaceCss::Use_NSC' );
			BsExtensionManager::setContext( 'MW::NamespaceCss::Use_NSC' );
		}
		return true;
	}
}