/**
 * PageTemplates extension
 *
 * Part of BlueSpice for MediaWiki
 *
 * @author     Markus Glaser <glaser@hallowelt.biz>
 * @version    $Id: PageTemplatesAdmin.js 7662 2012-12-10 14:34:46Z smuggli $
 * @package    Bluespice_Extensions
 * @subpackage PageTemplates
 * @copyright  Copyright (C) 2011 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 * @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
 * @filesource
 */

/**
 * Currently, there is a dependency to InsertLink in order to get Page Namespaces
 */

/**
 * Base class for all page template related methods and properties
 */
BsPageTemplatesAdmin = {
	/**
	 * Internationalisation
	 * @var object list of i18n key value pairs
	 */
	i18n: {
		headerLabel: 'Label',
		headerDescription: 'Description',
		headerTargetNamespace: 'Namespace',
		headerTemplate: 'Template',
		headerActions: 'Actions',
		tipEditDetails: 'Edit details',
		tipDeleteTemplate: 'Delete template',
		tipAddTemplate: 'Add template',
		btnOk: 'Ok',
		btnCancel: 'Cancel',
		titleError: 'Error',
		unknownError: 'An unknown error occurred.',
		titleAddTemplate: 'Add template',
		titleEditDetails: 'Edit details',
		labelLabel: 'Label',
		labelDescription: 'Description',
		labelTargetNamespace: 'Target namespace',
		labelTemplateNamespace: 'Template namespace',
		labelArticle: 'Template',
		titleDeleteTemplate: 'Delete template',
		confirmDeleteTemplate: 'Are you sure, you want delete this template?',
		showEntries: 'Displaying {0} - {1} of {2}'
	},
	/**
	 * Basic canvas to be rendered to
	 * @var Ext.grid.GridPanel
	 */
	panel: false,
	/**
	 * Store for template data, displayed in grid.
	 * @var Ext.data.JsonStore
	 */
	store: new Ext.data.JsonStore({
		autoDestroy: true,
		url: BlueSpice.buildRemoteString('PageTemplatesAdmin', 'getTemplates'),
		storeId: 'BsPageTemplatesAdminStore',
		root: 'templates',
		fields: [
			'id',
			'label',
			'desc',
			'targetns',
			'targetnsid',
			'template',
			'templatename',
			'templatens'
		],
		totalProperty: 'totalCount',
		remoteSort: true,
		sortInfo: {
			field: 'label',
			direction: 'ASC' // or 'DESC' (case sensitive for local sorting)
		}
	}),
	// TODO RBV (18.05.11 09:32): Dependency on InsertLink!
	/**
	 * Store for pages, needed for template editing
	 * @var Ext.data.JsonStore
	 */
	storePages: new Ext.data.JsonStore({
		url: BlueSpice.buildRemoteString('InsertLink', 'getPage', {ns:10}),
		root: 'items',
		fields: ['name', 'label', 'ns']
	}),
	/**
	 * Store for namespaces, neede for template editing, target namespace dropdown field
	 * @var Ext.data.JsonStore
	 */
	storeTargetNamespaces: new Ext.data.JsonStore({
		url: BlueSpice.buildRemoteString('PageTemplatesAdmin', 'getNamespaces', {showAll:1}),
		root: 'items',
		fields: ['name', 'id']
	}),
	/**
	 * Store for namespaces, neede for template editing, template namespace dropdown field
	 * @var Ext.data.JsonStore
	 */
	storeTemplateNamespaces: new Ext.data.JsonStore({
		url: BlueSpice.buildRemoteString('PageTemplatesAdmin', 'getNamespaces'),
		root: 'items',
		fields: ['name', 'id']
	}),
	/**
	 * Renders the page templates grid and fills it with data
	 */
	show: function() {
		//console.log(Ext.get('PageTemplatesAdminGrid'));
		if(!this.panel) {
			this.panel = new Ext.grid.GridPanel({
				renderTo: 'bs-pagetemplates-admingrid',
				id: 'ptaGrid',
				plugins: (Ext.isIE ? [] : ['fittoparent']),
				loadMask: true,  
				store: this.store,
				tbar: new Ext.Toolbar({
					style: {
						backgroundColor: '#FFFFFF',
						backgroundImage: 'none'
					},
					
					items: [{
						tooltip: this.i18n.tipAddTemplate,
						icon: stylepath+'/bluespice/bs-m_add.png',
						iconCls: 'btn44',
						handler: function(btn, ev) {
							this.showEditTemplate( null );
						},
						id: 'ptaAddTemplate',
						scope: this
					}, {
						xtype: 'tbseparator',
						height: 44,
						cls: 'sep44'
					}, {
						id: 'ptaEditTemplate',
						disabled: true,
						tooltip: this.i18n.tipEditDetails,
						icon: stylepath+'/bluespice/bs-um_config.png',
						iconCls: 'btn44',
						handler: function(btn, ev) {
							this.showEditTemplate(this.panel.getSelectionModel().getSelected());
						},
						scope: this
					}, {
						xtype: 'tbseparator',
						height: 44,
						cls: 'sep44'
					}, {
						id: 'ptaDeleteTemplate',
						disabled: true,
						tooltip: this.i18n.tipDeleteTemplate,
						icon: stylepath+'/bluespice/bs-m_delete.png',
						iconCls: 'btn44',
						handler: function(btn, ev) {
							if (this.panel.getSelectionModel().getSelected() ) {
								this.showDeleteTemplate(this.panel.getSelectionModel().getSelected());
							}
						},
						scope: this
					}
					]
				}),
				bbar: new Ext.PagingToolbar({
					displayMsg: this.i18n.showEntries,
					store: this.store,
					displayInfo: true,
					pageSize: 25,
					prependButtons: true
				}),
				colModel: new Ext.grid.ColumnModel({
					columns: [
					{
						id: 'templatename',
						header: this.i18n.headerLabel,
						sortable: true,
						width: 60,
						dataIndex: 'label'
					}, {
						header: this.i18n.headerDescription,
						sortable: true,
						dataIndex: 'desc'
					}, {
						header: this.i18n.headerTargetNamespace,
						sortable: true,
						width: 60,
						dataIndex: 'targetns'
					}, {
						header: this.i18n.headerTemplate,
						sortable: true,
						width: 60,
						dataIndex: 'template'
					}, {
						header: this.i18n.headerActions,
						xtype: 'actioncolumn',
						width: 40,
						cls: 'hideAction',
						items: [
						{
							icon: stylepath+'/bluespice/bs-um_config_tn.png',
							tooltip: this.i18n.tipEditDetails,
							handler: function(grid, rowIndex, colIndex) {
								this.showEditTemplate(this.store.getAt(rowIndex));
							},
							scope: this
						}, {
							icon: stylepath+'/bluespice/bs-m_delete_tn.png',
							tooltip: this.i18n.tipDeleteUser,
							handler: function(grid, rowIndex, colIndex) {
								this.showDeleteTemplate(this.store.getAt(rowIndex));
							},
							scope: this
						}
						],
						sortable: false
					}
					]
				}),
				viewConfig: {
					forceFit: true
				},
				sm: new Ext.grid.RowSelectionModel({
					singleSelect: true,
					listeners: {
						'selectionchange': function( sm ) {
							if (sm.hasSelection()) {
								Ext.getCmp('ptaEditTemplate').enable();
								Ext.getCmp('ptaDeleteTemplate').enable();
							} else {
								Ext.getCmp('ptaEditTemplate').disable();
								Ext.getCmp('ptaDeleteTemplate').disable();
							}
						}
					}
				}),
				border: false,
				columnLines: true,
				enableHdMenu: false,
				stripeRows: true,
				autoHeight: true
			});
			this.store.load();
		}
		this.panel.show();
		this.storePages.load();
		this.storeTargetNamespaces.load();
		this.storeTemplateNamespaces.load();
	},
	/**
	 * Reference to the Page Templates Window
	 * @var Window Ext.Window object
	 */
	window: false,

	/**
	 * Displays the edit template dialoge
	 * @param record Ext.data.Record an entry in the template store
	 */
	showEditTemplate: function(record) {

		this.window = new Ext.Window({
			title: this.i18n.titleEditDetails,
			modal: true,
			width: 350,
			items: [
			{
				xtype: 'form',
				id: 'panelFormEditTemplate',
				labelWidth: 120,
				padding: 10,
				border: false,
				defaults: {
					msgTarget: 'under'
				},
				items: [
				{
					xtype: 'textfield',
					fieldLabel: this.i18n.labelLabel,
					value: (record==null?'':record.get('label')),
					width: 180,
					name: 'label'//,
					//allowBlank: false
				}, {
					xtype: 'textarea',
					fieldLabel: this.i18n.labelDescription,
					value: (record==null?'':record.get('desc')),
					width: 180,
					name: 'desc'
				}, {
					xtype: 'combo',
					flex: 1,
					width: 180,
					store: this.storeTargetNamespaces,
					displayField:'name',
					valueField:'id',
					hiddenName: 'targetNs',
					typeAhead: true,
					mode: 'local',
					triggerAction: 'all',
					//emptyText:this.config.i18n.select_a_page,
					lastQuery: '',
					fieldLabel: this.i18n.labelTargetNamespace,
					name: 'targetNamespaceName',
					hiddenValue: (record==null?-99:record.get('targetnsid')),
					value: (record==null?-99:record.get('targetnsid'))
				}, {
					xtype: 'combo',
					flex: 1,
					width: 180,
					store: this.storeTemplateNamespaces,
					displayField:'name',
					valueField:'id',
					hiddenName: 'templateNs',
					typeAhead: true,
					mode: 'local',
					triggerAction: 'all',
					//emptyText:this.config.i18n.select_a_page,
					lastQuery: '',
					fieldLabel: this.i18n.labelTemplateNamespace,
					hiddenValue: (record==null?10:record.get('templatens')),
					value: (record==null?10:record.get('templatens')),
					name: 'templateNamespace',
					listeners: {
						'select': {
							fn:function(sel, value) {
								this.storePages.load({
									params:{
										ns: sel.getValue()
									}
								});
							},
							scope:this
						}
					}
				}, {
					xtype: 'combo',
					flex: 1,
					width: 180,
					store: this.storePages,
					displayField:'name',
					valueField:'label',
					//hiddenName: 'articleId',
					typeAhead: true,
					mode: 'local',
					triggerAction: 'all',
					//emptyText:this.config.i18n.select_a_page,
					lastQuery: '',
					fieldLabel: this.i18n.labelArticle,
					name: 'templateName',
					//hiddenValue: (record==null?'':record.get('template')),
					value: (record==null?'':record.get('templatename'))
				}, {
					xtype: 'hidden',
					name: 'oldId',
					value: (record==null?'':record.get('id'))
				}
			    ]
			}
			],
			buttons: [
			{
				text: this.i18n.btnOk,
				handler: function(){
					Ext.getCmp('panelFormEditTemplate').getForm().doAction('submit', {
						clientValidation: true,
						submitEmptyText: false,
						method: 'post',
						url: BlueSpice.buildRemoteString('PageTemplatesAdmin', 'doEditTemplate'),
						success: function(form, action) {
							this.store.reload();
							this.window.close();
							if(action.result.messages.length) {
								var tmp = '';
								for(i in action.result.messages) {
									if(typeof(action.result.messages[i]) != 'string') {
										continue;
									}
									tmp = tmp + action.result.messages[i] + '<br />';
								}
								Ext.Msg.alert('Status', tmp);
							}
							return true;
						},
						failure: function(form, action) {
							if(action.result.errors.length) {
								var tmp = '';
								for(i in action.result.errors) {
									if(typeof(action.result.errors[i]) != 'string') {
										continue;
									}
									tmp = tmp + action.result.errors[i] + '<br />';
								}
								Ext.Msg.alert('Status', tmp);
							}
						},
						scope: this
					});
				},
				scope: this
			}, {
				text: this.i18n.btnCancel,
				handler: function(){
					this.ownerCt.ownerCt.close();
				}
			}
			]
		});
		this.storePages.load();
		this.storeTargetNamespaces.load();
		this.storeTemplateNamespaces.load();
		this.window.show();
	},
	
	/**
	 * Displays the delete template dialoge
	 * @param record Ext.data.Record an entry in the template store
	 */
	showDeleteTemplate: function(record) {
		Ext.Msg.confirm(this.i18n.titleDeleteTemplate, this.i18n.confirmDeleteTemplate, function(btn) {
			if(btn == 'yes') {
				Ext.Ajax.request({
					method: 'post',
					params: {
						id: record.get('id')
					},
					url: BlueSpice.buildRemoteString('PageTemplatesAdmin', 'doDeleteTemplate'),
					success: function(response, opts) {
						var obj = Ext.decode(response.responseText);
						BsPageTemplatesAdmin.panel.store.reload();
						if(obj.messages.length) {
							var tmp = '';
							for(i in obj.messages) {
								if(typeof(obj.messages[i]) != 'string') {
									continue;
								}
								tmp = tmp + obj.messages[i] + '<br />';
							}
							Ext.Msg.alert('Status', tmp);
						}
					},
					failure: function(response, opts) {
					    // TODO RBV (18.05.11 09:46): Alert user?
					    //console.log('server-side failure with status code ' + response.status);
					},
					scope: this
				})
			}
		})
	}
}

setTimeout( "BsPageTemplatesAdmin.show()", 1 );