<?php

/**
 * Describes a review.
 *
 * Part of BlueSpice for MediaWiki
 *
 * @author     Markus Glaser <glaser@hallowelt.biz>
 * @version    $Id: ReviewProcess.class.php 9102 2013-04-08 10:32:26Z pwirth $
 * @package    BlueSpice_Extensions
 * @subpackage Review
 * @copyright  Copyright (C) 2011 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 * @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
 * @filesource
 */
/*
  DEFINE('RV_VOTE', 'vote');
  DEFINE('RV_SIGN', 'sign');
  DEFINE('RV_COMMENT', 'comment');
  DEFINE('RV_WORKFLOW', 'workflow');

  $rv_editable = array(RV_WORKFLOW, RV_COMMENT);
  $rv_sequential = array(RV_WORKFLOW, RV_SIGN);
  $rv_abortWhenDenied = array(RV_SIGN, RV_WORKFLOW);
 */

//$rv_access = array('user');
//Last Code Review RBV (30.0.2011)
// TODO RBV (30.06.11 15:10): Coding Conventions in all of the document
/**
 * Class that describes a review.
 * @package BlueSpice_Extensions
 * @subpackage Review
 */
class BsReviewProcess {
	/**
	 * The following (string) constants define review modes which are characterized below.
	 */

	const MODE_VOTE = "vote";
	const MODE_SIGN = "sign";
	const MODE_COMMENT = "comment";
	const MODE_WORKFLOW = "workflow";

	/**
	 * Defines which review modes are editable during a review process.
	 * @var Array of mode constants.
	 */
	protected static $aEditable = array(BsReviewProcess::MODE_WORKFLOW, BsReviewProcess::MODE_COMMENT);

	/**
	 * Defines for which review modes steps are to be processed in a sequential order.
	 * @var Array of mode constants.
	 */
	protected static $aSequential = array(BsReviewProcess::MODE_WORKFLOW, BsReviewProcess::MODE_SIGN);

	/**
	 * Defines which review modes shall fail when there is at least one denial.
	 * @var Array of mode constants.
	 */
	protected static $aAbortWhenDenied = array(BsReviewProcess::MODE_WORKFLOW, BsReviewProcess::MODE_SIGN);

	/**
	 * Page id of article the review is associated with.
	 * @var int Article ID.
	 */
	var $pid;

	/**
	 * Mode of a review which defines its behavior.
	 * @var string One of the mode constants.
	 */
	var $mode;
//	var $editable;
	/**
	 * Date when review starts.
	 * @var string yyyy-mm-dd
	 */
	// TODO MRG (13.06.11 01:00): Should be MWtimestamp.
	var $startdate;

	/**
	 * Date when review ends.
	 * @var string yyyy-mm-dd
	 */
	// TODO MRG (13.06.11 01:00): Should be MWtimestamp.
	var $enddate;

	/**
	 * Array of single steps the review has.
	 * @var array List of BsReviewProcessSteps.
	 */
	var $steps;

	/**
	 * User that initiated the review.
	 * @var int User id.
	 */
	var $owner;

	/**
	 * Save review as a template.
	 * @var bool. True if should be saved as template.
	 */
	var $tmpl_save = false;
	//var $tmpl_choice;
	//var $tmpl_name;

	protected $_aInjections = array();

	/**
	 * Constructor of BsReviewProcess
	 */
	function __construct() {
		$mode = "";
		//$editable = false;
		$startdate = '';
		$enddate = '';
		$steps = array();
		$owner = -1;

		$this->_aInjections = array();
		wfRunHooks('BsReviewProcess::construct', array(&$this->_aInjections));
	}

	/**
	 * Is the process still ongoing?
	 * @return bool True if ongoing.
	 */
	function isActive() {
		if (($this->isStarted()) && (!$this->isFinished()))
			return true;
		else
			return false;
	}

	/**
	 * Is the process already started?
	 * @return bool True if started.
	 */
	function isStarted() {
		$curdate = strtotime("now");
		$startdate = strtotime($this->startdate);
		if ($curdate >= $startdate)
			return true;
		else
			return false;
	}

	//return values:
	//	false: 		not started or still running
	//	'date':		finished due to a deadline
	// 	'status':	finished due to confirmation of all participants
	//  'denied':   at least one participant has denied the workflow
	/**
	 * Is the review finished?
	 * @return string false: not started or still running; 'date': finished due to a deadline; 'status': finished due to confirmation of all participants; 'denied': at least one participant has denied the workflow.
	 */
	function isFinished() {
		$curdate = strtotime("now");
		$enddate = strtotime($this->enddate);
		if ($curdate > $enddate)
			return 'date';
		foreach ($this->steps as $st) {
			if ($st->status == -1) {
				return false;
			}
			if ($this->isAbortWhenDenied()
				&&
				($st->status == 0)) {
				return 'denied';
			}
		}
		return 'status';
	}

	// TODO MRG (13.06.11 01:10): Unify return values with isFinished
	/**
	 * Returns the current status of the review.
	 * @param string $date Timestamp as produced by time().
	 * @return string '': ongoing; 'nothing': not everybody voted; 'denied': someone said no; 'status': passed.
	 */
	function getStatus($date) {
		$curdate = strtotime("now");  //TODO: variable not used => line 105 commented
		$enddate = strtotime($this->enddate); //TODO: same
		foreach ($this->steps as $st) {
			//if($st->status != 1 && $curdate > $enddate) return 'date';
			if ($st->status == -1) {
				if ($date > time()) {
					return '';
				}
				return 'nothing';
			}
			if ($st->status == 0) {
				return 'denied';
			}
		}
		return 'status';
	}

	// TODO MRG (13.06.11 01:12): JSON or Array
	/**
	 * Returns the current voting status of the review.
	 * @return string Format is "NUMBER_UNKNOWN;NUMBER_NO;NUMBER_YES"
	 */
	function currentStatus() {
		$unknown = 0;
		$yes = 0;
		$no = 0;
		foreach ($this->steps as $st) {
			if ($st->status == -1)
				$unknown++;
			if ($st->status == 0)
				$no++;
			if ($st->status == 1)
				$yes++;
		}
		return $unknown . ';' . $no . ';' . $yes;
	}

	/**
	 * Getter for startdate.
	 * @return string Localized date.
	 */
	function getStartdate() {
		return $this->formattedDate($this->startdate);
	}

	/**
	 * Getter for enddate.
	 * @return string Localized date.
	 */
	function getEnddate() {
		return $this->formattedDate($this->enddate);
	}

	function getMode() {
		return $this->mode;
	}

	/**
	 * Setter for owner.
	 * @param int $userId User ID of owner.
	 */
	function setOwner($userId) {
		$this->owner = $userId;
	}

	/**
	 * Getter for owner.
	 * @return int User ID of owner.
	 */
	function getOwner() {
		return $this->owner;
	}

	/**
	 * Getter for Page ID (pid)
	 * @return int Page ID.
	 */
	function getPid() {
		return $this->pid;
	}

	// TODO MRG (13.06.11 01:16): Use function in framework.
	/**
	 * Returns a localized date string.
	 * @param string $datestring Date in format yyyy-mm-dd
	 * @return string Localized date string in format dd.mm.yyyy
	 */
	function formattedDate($datestring) {
		$date = strtotime($datestring);
		return date("d.m.Y", $date);
	}

	/**
	 * Returns the step that is currently active for a given user.
	 * @param int $uid User ID.
	 * @return mixed false or BsReviewProcessStep
	 */
	function currentStep($uid) {
		foreach ($this->steps as $step) {
			$bResult = null;
			foreach ($this->_aInjections as $oInjection) {
				if ($oInjection->currentStepIsActive($step, $uid) && $bResult !== false) {
					$bResult = true;
				} else {
					$bResult = false;
				}
			}
			if (($step->user == $uid || (!is_null($bResult) && $bResult !== false))
				&&
				($step->status == -1))
				return $step;
		}
		return false;
	}

	/**
	 * Are there any steps for the current review? There are none if the review has not been saved yet.
	 * @return bool True if there are steps.
	 */
	function hasSteps() {
		if (is_array($this->steps) && count($this->steps) > 0) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Stores a review to database.
	 * @return bool Always true.
	 */
	function store($update = false) {
		if ($this->pid == 0)
			return false;
		$dbw = & wfGetDB(DB_MASTER);
		$data = array();
		global $wgDBtype;
		if ($wgDBtype == 'postgres' || $update) {
			// TODO: TL (18.08.2011, 09:30)
			// replace global
			// id is serial and auto increments
			// can't we use this for mysql too?
		} else {
			$data['rev_id'] = 0;
		}
		$data['rev_pid'] = $this->pid;
		//$data['editable']  = $this->editable?1:0;
		$data['rev_mode'] = $this->mode;
		$data['rev_startdate'] = $this->startdate;
		$data['rev_enddate'] = $this->enddate;
		$data['rev_owner'] = $this->owner;
		if (!$update) {
			$dbw->insert('bs_review', $data);
		} else {
			$dbw->update('bs_review', $data, array('rev_pid = ' . $update));
		}

		// Get Review-ID
		$res = $dbw->select('bs_review', 'rev_id', "rev_pid=" . $this->pid, '', array("ORDER BY" => "REV_ID DESC"));
		$row = $dbw->fetchRow($res);
		$dbw->freeResult($res);
		$review_id = $row['rev_id'];

		$tmp_users = array();
		$i = 0;
		foreach ($this->steps as $step) {
			$data = array();
			global $wgDBtype;
			if ($wgDBtype == 'postgres') {
				// TODO: TL (18.08.2011, 09:30)
				// replace global
				// id is serial and auto increments
				// can't we use this for mysql too?
			} else {
				$data['revs_id'] = 0;
			}
			$data['revs_review_id'] = $review_id;
			$data['revs_user_id'] = $tmp_users[] = $step->user;
			$data['revs_status'] = $step->status;
			$data['revs_sort_id'] = $i;
			$data['revs_comment'] = $step->comment;
			$dbw->insert('bs_review_steps', $data);
			$i++;
		}

		if ($this->tmpl_save) {
			$data = array();
			$data['revt_owner'] = $this->owner;
			$data['revt_user'] = join(',', $tmp_users);
			$data['revt_mode'] = $this->mode;
			$data['revt_name'] = addslashes($this->tmpl_name);
			$data['revt_public'] = $this->tmpl_choice + 0;
			// braucht neuen Index
			// ALTER TABLE `hw_review_templates` ADD UNIQUE `tmpl_idx` ( `name` , `owner` );
			$tbl = $dbw->tableName('bs_review_templates');
			$dbw->query("DELETE FROM $tbl
			 WHERE revt_owner = '{$data['revt_owner']}' AND
                             revt_mode  = '{$data['revt_mode']}' AND
                             revt_name  = '{$data['revt_name']}' AND
                             revt_public = '{$data['revt_public']}'");
			$dbw->query("INSERT INTO $tbl (revt_owner, `revt_user`, revt_mode, revt_name, revt_public)
			 VALUES ('{$data['revt_owner']}', '{$data['revt_user']}', '{$data['revt_mode']}', '{$data['revt_name']}', '{$data['revt_public']}')");
		}
		// TODO MRG (13.06.11 01:19): Answer with status of db operation
		return true;
	}

	/**
	 * This functions restarts the ended workflow.
	 */
	function restart() {
		$dbw = & wfGetDB(DB_MASTER);

		// Get Review-ID
		$res = $dbw->select('bs_review', 'rev_id', "rev_pid=" . $this->pid, '', array('ORDER BY' => 'REV_ID DESC'));
		$row = $dbw->fetchRow($res);
		$dbw->freeResult($res);
		$review_id = $row['rev_id'];

		$tbl = $dbw->tableName('bs_review');

		global $wgDBtype;
		// TODO: TL (18.08.2011, 09:30)
		// replace global
		if ($wgDBtype == 'postgres') {
			$dbw->query("UPDATE $tbl SET startdate=to_char(current_timestamp, 'YYYY-MM-DD HH24:MI:SS'), enddate=to_char(current_timestamp + interval '7 days', 'YYYY-MM-DD HH24:MI:SS') WHERE pid={$this->pid}");
		} else {
			$dbw->query("UPDATE $tbl SET rev_startdate=NOW(), rev_enddate=DATE_ADD(NOW(), INTERVAL 7 DAY) WHERE rev_pid={$this->pid}");
		}

		foreach ($this->_aInjections as $oInjection) {
			$oInjection->restartProcess($review_id);
		}
	}

	/**
	 * This functions resets a interupted workflow.
	 */
	function reset() {
		$dbw = & wfGetDB(DB_MASTER);

		// Get Review-ID and owner id
		$res = $dbw->select('bs_review', array('rev_id', 'rev_owner'), "rev_pid=" . $this->pid, '', array('ORDER BY' => 'rev_id DESC'));
		$row = $dbw->fetchRow($res);
		$dbw->freeResult($res);
		$review_id = $row['rev_id'];
		$owner_id = $row['rev_owner'];

		$tbl = $dbw->tableName('bs_review');

		global $wgDBtype;
		// TODO: TL (18.08.2011, 09:30)
		// replace global
		if ($wgDBtype == 'oracle') {
			$dbw->query("UPDATE $tbl SET rev_startdate=to_char(SYSDATE, 'YYYYMMDDHH24MISS'), rev_enddate=to_char(SYSDATE + interval '7 days', 'YYYYMMDDHH24MISS'), rev_mode='workflow' WHERE rev_pid={$this->pid}");
		} elseif ($wgDBtype == 'postgres') {
			$dbw->query("UPDATE $tbl SET rev_startdate=to_char(current_timestamp, 'YYYYMMDDHH24MISS'), rev_enddate=to_char(current_timestamp + interval '7 days', 'YYYYMMDDHH24MISS'), rev_mode='workflow' WHERE rev_pid={$this->pid}");
		} else {
			$dbw->query("UPDATE $tbl SET rev_startdate=NOW(), rev_enddate=DATE_ADD(NOW(), INTERVAL 7 DAY), rev_mode='workflow' WHERE rev_pid={$this->pid}");
		}

		$tbl = $dbw->tableName('bs_review_steps');
		$aUsersVoted = array();
		$iLastVotedId = 0;
		$iVoteAmount = 0;
		$res = $dbw->select('bs_review_steps', array('revs_user_id', 'revs_sort_id'), array('revs_status > -1', 'revs_review_id = ' . $review_id), '', array('ORDER BY revs_sort_id'));
		while ($row = $dbw->fetchRow($res)) {
			$aUsersVoted[] = $row;
			$iLastVotedId = $row['revs_sort_id'];
			$iVoteAmount++;
		}
		$iVoteAmount++;

		$dbw->query("UPDATE $tbl SET revs_sort_id=revs_sort_id+{$iVoteAmount} WHERE revs_review_id = {$review_id} AND revs_sort_id > {$iLastVotedId}");
		$dbw->query("UPDATE $tbl SET revs_status = -2 WHERE revs_review_id = {$review_id} AND revs_status = 1");
		$dbw->query("UPDATE $tbl SET revs_status = -3 WHERE revs_review_id = {$review_id} AND revs_status = 0");

		$data = array(
			'revs_review_id' => $review_id,
			'revs_user_id' => $owner_id,
			'revs_status' => -1,
			'revs_sort_id' => ++$iLastVotedId,
			'revs_comment' => ''
		);
		foreach ($this->_aInjections as $oInjection) {
			$oInjection->createStepDefault($data);
		}
		$dbw->insert('bs_review_steps', $data);

		$lastUserId = 0;
		foreach ($aUsersVoted as $aUser) {
			if ($aUser['revs_user_id'] == $lastUserId) {
				continue;
			}
			$lastUserId = $aUser['revs_user_id'];
			$data = array(
				'revs_review_id' => $review_id,
				'revs_user_id' => $aUser['revs_user_id'],
				'revs_status' => -1,
				'revs_sort_id' => ++$iLastVotedId,
				'revs_comment' => ''
			);
			foreach ($this->_aInjections as $oInjection) {
				$oInjection->createStepDefault($data);
			}
			$dbw->insert('bs_review_steps', $data);
		}
	}

	/**
	 * Load a review from database for a givn page id.
	 * @param int $pid Page ID the review should be loaded for.
	 * @return mixed BsReviewProcess if there is one, otherwise false.
	 */
	static function newFromPid($pid) {
		$oReviewProcess = false;
		$dbw = & wfGetDB(DB_MASTER);
		$res = $dbw->select('bs_review', '*', "rev_pid=" . $pid, '', array("ORDER BY" => "REV_ID DESC"));
		if ($row = $dbw->fetchRow($res)) {
			$oReviewProcess = new BsReviewProcess();
			$dbw->freeResult($res);

			//$hw_review->tag_txt = $tag;
			$oReviewProcess->pid = $pid;

			// starttag
			$oReviewProcess->mode = $row['rev_mode'];
			//$hw_review->editable       = $row['editable'];
			$oReviewProcess->startdate = $row['rev_startdate'];
			$oReviewProcess->enddate = $row['rev_enddate'];
			$oReviewProcess->owner = $row['rev_owner'];

			$rid = $row['rev_id'];

			$res = $dbw->select('bs_review_steps', '*', "revs_review_id=" . $rid, '', array("ORDER BY" => "revs_sort_id ASC"));
			while ($row = $dbw->fetchRow($res)) {
				$oReviewProcess->steps[] = BsReviewProcessStep::newFromRow($row);
			}

			$dbw->freeResult($res);
		}

		return $oReviewProcess;
	}

	/**
	 * Creates a new review object from Input parameters.
	 * @return BsReviewProcess The created review.
	 */
	static function newFromRequest() {
		$oReviewProcess = false;

		$oReviewProcess = new BsReviewProcess();

		//$hw_review->tag_txt = $tag;
		//$hw_review->pid = $_REQUEST['rv_pid'];
		// rv_pid is stored in table hw_review as a smallint(5) unsigned
		$oReviewProcess->pid = BsCore::getParam('rv_pid', 0, BsPARAM::REQUEST | BsPARAMTYPE::NUMERIC);

		// starttag
		//$hw_review->mode = $_REQUEST['rv_mode'];
		// rv_mode is stored in table hw_review as a varchar(20)
		$oReviewProcess->mode = BsCore::getParam('rv_mode', '', BsPARAM::REQUEST | BsPARAMTYPE::STRING);
		/*
		  if (in_array($hw_review->mode, $rv_editable)
		  $hw_review->editable = 1
		  else
		  $hw_review->editable = 0;
		 */
		//$hw_review->startdate = date("Y-m-d", strtotime($_REQUEST['rv_startdate']));
		$oReviewProcess->startdate = date("Y-m-d", strtotime(BsCore::getParam('rv_startdate', '', BsPARAM::REQUEST | BsPARAMTYPE::STRING)));
		//$hw_review->enddate = date("Y-m-d", strtotime($_REQUEST['rv_enddate']));
		$oReviewProcess->enddate = date("Y-m-d", strtotime(BsCore::getParam('rv_enddate', '', BsPARAM::REQUEST | BsPARAMTYPE::STRING)));

		//for ($i=0; $i<count($_REQUEST['rv_step_name']); $i++)
		//{
		//	$hw_review->steps[] = new HwReviewStep($_REQUEST['rv_step_name'][$i], $_REQUEST['rv_step_comment'][$i], $_REQUEST['rv_step_status'][$i]);
		//}
		$paramRvStepName = BsCore::getParam('rv_step_name', array(), BsPARAM::REQUEST | BsPARAMTYPE::ARRAY_STRING);

		$countParamRvStepName = count($paramRvStepName);
		if ($countParamRvStepName > 0) { // count($var) == 0 <==> Variable not set OR Array is empty
			$paramRvStepComment = BsCore::getParam('rv_step_comment', array(), BsPARAM::REQUEST | BsPARAMTYPE::ARRAY_STRING);
			$paramRvStepStatus = BsCore::getParam('rv_step_status', array(), BsPARAM::REQUEST | BsPARAMTYPE::ARRAY_NUMERIC | BsPARAMOPTION::DEFAULT_ON_ERROR);
		}

		for ($i = 0; $i < $countParamRvStepName; $i++) {
			$oReviewProcess->steps[] = BsReviewProcessStep::newFromData($paramRvStepName[$i], $paramRvStepComment[$i], $paramRvStepStatus[$i]);
		}

		return $oReviewProcess;
	}

	/**
	 * Creates a new review from JSON imput.
	 * @param string $sJSON JSON input the review should be created from.
	 * @param array $aErrors List of errors in case anything goes wrong.
	 * @return mixed BsReviewProcess or false if anything goes wrong.
	 */
	static function newFromJSON($sJSON = '', &$aErrors) {
		$aErrors = array();
		$oJsonReview = json_decode($sJSON);
		$oReviewProcess = new BsReviewProcess();
		// rv_pid is stored in table hw_review as a smallint(5) unsigned
		$oReviewProcess->pid = BsCore::sanitize($oJsonReview->pid, -1, BsPARAMTYPE::NUMERIC);
		$oReviewProcess->mode = BsCore::sanitize($oJsonReview->mode, '', BsPARAMTYPE::STRING);

		if (!$oJsonReview->startdate) {
			$aErrors[] = 'startdate-missing';
		};
		$oReviewProcess->startdate = date("YmdHis", strtotime(BsCore::sanitize($oJsonReview->startdate, '', BsPARAMTYPE::STRING)));
		if (!$oJsonReview->enddate) {
			$aErrors[] = 'enddate-missing';
		};
		$oReviewProcess->enddate = date("YmdHis", strtotime(BsCore::sanitize($oJsonReview->enddate, '', BsPARAMTYPE::STRING)));

		if ($oReviewProcess->startdate > $oReviewProcess->enddate) {
			$aErrors[] = 'startdate-after-enddate';
		};

		$paramRvSteps = $oJsonReview->steps;
		if (count($paramRvSteps) <= 0) {
			$aErrors[] = 'no-reviewers';
		};
		// TODO an MRG: wir dieser block noch benötigt oder ist er inzwischen deprecated ??
		// Fixed mit isset
		if (isset($oJsonReview->tmpl_save) && $oJsonReview->tmpl_save) {
			$oReviewProcess->tmpl_save = true;
			$oReviewProcess->tmpl_name = addslashes($oJsonReview->tmpl_name);
			$oReviewProcess->tmpl_choice = $oJsonReview->tmpl_choice;
		}

		foreach ($paramRvSteps as $oStep) {
			if ($oStep->status == '' || $oStep->status == 'unknown')
				$oStep->status = '-1';
			if ($oStep->status == 'yes')
				$oStep->status = '1';
			if ($oStep->status == 'no')
				$oStep->status = '0';
			$sComment = BsCore::sanitize($oStep->comment, '', BsPARAMTYPE::STRING);
			if (strlen($sComment) > 255) {
				$aErrors[] = 'comment-too-long';
			};
			if (!$oStep->userid || !is_numeric($oStep->userid)) {
				//TODO: make sure you get a valid id
				//PW: wont work with realname!
				$oStep->userid = User::idFromName($oStep->name);
				if (!$oStep->userid) {
					$aErrors[] = 'user-not-found';
				}
			} else {
				$oTmpUser = User::newFromId($oStep->userid);
				if (!$oTmpUser) {
					$aErrors[] = 'user-not-found';
				}
			}
			
			$oReviewProcess->steps[] = BsReviewProcessStep::newFromData(
					BsCore::sanitize($oStep->userid, '', BsPARAMTYPE::STRING),
					$sComment,
					BsCore::sanitize($oStep->status, '-1', BsPARAMTYPE::STRING)
			);
		}


		if (is_array($aErrors) && count($aErrors) > 0) {
			return false;
		} else {
			return $oReviewProcess;
		}
	}

	/**
	 * Returns a list of all users in the review process.
	 * @return array List of arrays: 'id'=> USERID, 'comment'=>COMMENT
	 */
	function getAllUsers() {
		$usersList = array();

		foreach ($this->steps as $step) {
			$usersList[] = array('id' => $step->user, 'comment' => $step->comment);
		}
		return $usersList;
	}

	/**
	 * Returns a list of users that are next in review process.
	 * @return array List of arrays: 'id'=> USERID, 'comment'=>COMMENT
	 */
	function getNextUsers() {
		$usersList = array();

		foreach ($this->steps as $step) {
			if ($step->status != -1)
				continue;
			if ($this->isSequential()) {
				$usersList[] = array('id' => $step->user, 'comment' => $step->comment);
				break;
			} else {
				$usersList[] = array('id' => $step->user, 'comment' => $step->comment);
			}
		}
		return $usersList;
	}

	/**
	 * Are the steps to be executed in sequential order?
	 * @return bool True if yes.
	 */
	function isSequential() {
		if (in_array($this->mode, self::$aSequential))
			return true;
		else
			return false;
	}

	/**
	 * Is the review editable?
	 * @return bool True if yes.
	 */
	function isEditable() {
		if (in_array($this->mode, self::$aEditable))
			return true;
		else
			return false;
	}

	/**
	 * Should the review be stopped when at least on person denies?
	 * @return bool True if yes.
	 */
	function isAbortWhenDenied() {
		if (in_array($this->mode, self::$aAbortWhenDenied))
			return true;
		else
			return false;
	}

	static function userHasWaitingReviews($oUser) {
		$iUserId = $oUser->getId();
		$dbw = & wfGetDB(DB_MASTER);

		$sTblReview = $dbw->tableName('bs_review');
		$sTblReviewSteps = $dbw->tableName('bs_review_steps');

		$aTables = array(
			'bs_review',
			'bs_review_steps'
		);
		$aConditions = array(
			$sTblReviewSteps . '.revs_review_id = ' . $sTblReview . '.rev_id',
			$sTblReview . '.rev_owner' => $oUser->getId()
		);
		$aOptions = array(
			'ORDER BY' => '' . $sTblReview . '.rev_id, ' . $sTblReviewSteps . '.revs_sort_id'
		);

		$aReviews = array();
		$iReviewsWaiting = 0;
		$res = $dbw->select( $aTables, array('rev_id', 'revs_status'), $aConditions, __METHOD__, $aOptions );

		while ($row = $dbw->fetchRow($res)) {
			if (!isset($aReviews[$row['rev_id']])) {
				$aReviews[$row['rev_id']] = true;
			}
			if ($row['revs_status'] == -1) {
				$aReviews[$row['rev_id']] = false;
			} else if ($aReviews[$row['rev_id']] != false) {
				$aReviews[$row['rev_id']] = true;
			}
		}

		foreach ($aReviews as $bFinished) {
			if ($bFinished) {
				$iReviewsWaiting++;
			}
		}

		return $iReviewsWaiting;
	}

	/**
	 * Returns a list of pages that are being reviewed by a given user.
	 * @param int $uid ID if given user.
	 * @return array List of pages that are being reviewed.
	 */
	static function listReviews($uid, $bIgnoreStatus = true) {
		if (!$uid) {
			return array();
		}
		global $wgDBtype;

		$dbw = & wfGetDB(DB_MASTER);

		$tables = array();
		$tables[] = 'bs_review';
		$tables[] = 'bs_review_steps';

		$tbl_rev = $dbw->tableName('bs_review');
		$tbl_step = $dbw->tableName('bs_review_steps');

		//PW: SYSDATE needed for Oracle - current_timestamp needed for postgres
		//$sStartdate = $wgDBtype == 'oracle' ? "to_char(SYSDATE, 'YYYYMMDDHH24MISS')" : $dbw->timestamp(time());
		switch ($wgDBtype) {
			case 'oracle': {
					$sStartdate = " AND rev_startdate <= to_char(SYSDATE, 'YYYYMMDDHH24MISS')";
					break;
				}
			case 'postgres': {
					$sStartdate = " AND to_date(rev_startdate,'YYYYMMDDHH24MISS') <= to_date(to_char(CURRENT_DATE,'YYYYMMDD'),'YYYYMMDD') ";
					break;
				}
			default: {
					$sStartdate = " AND rev_startdate <= " . $dbw->timestamp(time());
				}
		}
		$conds = array();
		$conds[] = $tbl_step . '.revs_review_id = ' . $tbl_rev . '.rev_id';   // join tables

		$conds[] = '((' . $tbl_step . '.revs_user_id = ' . $uid . ' AND ' . $tbl_step . '.revs_delegate_to = 0) OR ' . $tbl_step . '.revs_delegate_to = ' . $uid . ')' .
			$sStartdate .
			' AND ' . $tbl_step . '.revs_status=-1)' .
			' OR (' . $tbl_rev . '.rev_owner = ' . $uid . '';

		//PW (14.06.2012): all fields need to be listet on GROUP BY for Oracle
		//                  (wont work on "*")
		//$aOptions = array('GROUP BY' => $tbl_rev.'.rev_id');
		//$sFields = $tbl_rev . '.*';
		$aRevColumns = array(
			'rev_id',
			'rev_pid',
			'rev_editable',
			'rev_mode',
			'rev_startdate',
			'rev_enddate',
			'rev_owner'
		);

		$aRevsColumns = array(
			'revs_id',
			'revs_review_id',
			'revs_user_id',
			'revs_status',
			'revs_sort_id',
			'revs_comment',
			'revs_delegate_to',
			'revs_timestamp'
		);


		$aOptions = array('GROUP BY' => join(',', $aRevColumns) . ',' . join(',', $aRevsColumns));
		$sFields = join(',', $aRevColumns) . ',' . join(',', $aRevsColumns);

		$res = $dbw->select($tables, $sFields, $conds, __METHOD__, $aOptions);
		$pages = array();

		if ($res) {
			while ($row = $dbw->fetchRow($res)) {
				if (in_array($row['rev_mode'], self::$aSequential)) {
					if ($wgDBtype == 'oracle') {
						$seq_res = $dbw->query('SELECT * FROM (
													SELECT revs_id, revs_user_id, revs_delegate_to, ROW_NUMBER() OVER (ORDER BY revs_sort_id) lmt
														FROM ' . $dbw->tableName('bs_review_steps') . '
														WHERE revs_review_id=' . $row['rev_id'] . '
														AND revs_status=-1
													)
													WHERE lmt BETWEEN 0 AND 1 '
						);
					} else {
						$seq_res = $dbw->select('bs_review_steps', 'revs_id, revs_user_id, revs_delegate_to', array('revs_review_id=' . $row['rev_id'],
							'revs_status=-1'), '', array("ORDER BY" => "revs_sort_id",
							"LIMIT" => "1")
						);
					}
					if ($dbw->numRows($seq_res) > 0) {
						$seq_row = $dbw->fetchRow($seq_res);
						if ($seq_row['revs_user_id'] == $uid || $seq_row['revs_delegate_to'] == $uid)
							$pages[] = $row['rev_pid'];
					}
					else if (!$bIgnoreStatus && $row['rev_owner'] == $uid) {
						$pages[] = $row['rev_pid'];
					}
				}
				else
					$pages[] = $row['rev_pid'];
			}
		}
		return array_unique($pages);
	}

	/**
	 * Removes all reviews for a given page.
	 * @param int $pid Page ID for which all reviews should be removed.
	 */
	static function removeReviews($pid) {
		$dbw = & wfGetDB(DB_MASTER);
		$res = $dbw->select('bs_review', 'rev_id', array("rev_pid" => $pid));
		while ($row = $dbw->fetchRow($res)) {
			$dbw->delete('bs_review_steps', array('revs_review_id' => $row['rev_id']));
			//$dbw->delete('review', array('id'=>$row['id']));#
			$dbw->delete('bs_review', array('rev_pid' => $pid));
		}
	}

	static function removeReviewSteps($pid) {
		$dbw = & wfGetDB(DB_MASTER);
		$res = $dbw->select('bs_review', 'rev_id', array("rev_pid" => $pid));
		while ($row = $dbw->fetchRow($res)) {
			$dbw->delete('bs_review_steps', array('revs_review_id' => $row['rev_id']));
		}
	}

}
