<?php
/**
 * Renders the Shoutbox frame.
 *
 * Part of BlueSpice for MediaWiki
 *
 * @author     Markus Glaser <glaser@hallowelt.biz>
 * @version    $Id: view.ShoutBox.php 7870 2012-12-20 14:26:41Z pwirth $
 * @package    BlueSpice_Extensions
 * @subpackage ShoutBox
 * @copyright  Copyright (C) 2011 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 * @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
 * @filesource
 */

//Last Code review RBV (30.06.2011)

/**
 * This view renders the Shoutbox frame.
 * @package    BlueSpice_Extensions
 * @subpackage ShoutBox 
 */
class ViewShoutBox extends ViewBaseElement {

	/**
	 * Constructor
	 */
	public function __construct() {
		parent::__construct();
	}

	/**
	 * This method actually generates the output
	 * @return string HTML output
	 */
	public function execute( $params = false ) {
		$sOut = '';

		if ($this->getOption( 'showmessageform' ) ) {
			$sOut .= $this->renderMessageForm();
		}
		$sOut = $this->wrapAll( $sOut );

		return $sOut;
	}

	/**
	 * Renders the form that is used to enter new shouts
	 * @return string HTML of the form
	 */
	protected function renderMessageForm() {
		$aOut = array();
		$aOut[] = '<form id="bs-sb-form" class="clearfix">';
		$aOut[] = '  <textarea id="bs-sb-message" maxlength="'.BsConfig::get( 'MW::ShoutBox::MaxMessageLength' ).'">' . wfMsg( 'bs-shoutbox-message' ) . '</textarea>';
		$aOut[] = '  <br />';
		$aOut[] = '  <img id="bs-sb-loading" src="' . BsCore::getInstance( 'MW' )->getAdapter()->get( 'StylePath' ) . '/bluespice/bs-ajax-loader-bar-blue.gif" alt="Loading..."/>';
		$aOut[] = '  <input id="bs-sb-send" type="submit" value="' . wfMsg( 'bs-shoutbox-shout' ) . '" />';
		$aOut[] = '</form>';

		return implode( "\n" , $aOut );
	}


	/**
	 * Renders the basic shoutbox layer
	 * @param string $innerText HTML that is to be put inside the basic shoutbox layer, i.e. the output box and shouts.
	 * @return string HTML for output
	 */
	protected function wrapAll( $innerText ) {
		$aOut = array();
		$aOut[] = '<div class="bs-sb">';
		$aOut[] = '  <fieldset>';
		$aOut[] = '    <legend>' . wfMsg( 'bs-shoutbox-title' ) . '</legend>';
		$aOut[] = $innerText;
		//$aOut[] = '    <div id="bs-sb-loading" style="display:none;"><img src="' . BsConfig::get('MW::ScriptPath') . '/bluespice-mw/ext/ShoutBox/css/images/loading.gif" alt="'.wfMsg('loading').'" /></div>';
		$aOut[] = '    <div id="bs-sb-content" style="display:none;"></div>';
		$aOut[] = '  </fieldset>';
		$aOut[] = '</div>';

		return implode( "\n" , $aOut );
	}

}
