<?php
/**
* This file is part of blue spice for MediaWiki.
*
* Use MediaWiki:TopBarMenu to customize the TopMenuBar
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
* 
* This file is part of BlueSpice for MediaWiki
* For further information visit http://www.blue-spice.org
*
* @author     Patric Wirth <wirth@hallowelt.biz>
* @version    1.20.0
* @version    $Id: TopMenuBarCustomizer.class.php 9690 2013-06-12 10:13:02Z pwirth $
* @package    Bluespice_Extensions
* @subpackage TopMenuBarCustomizer
* @copyright  Copyright (C) 2011 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
* @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
* @filesource
*/

/**
 * v1.20.0
 * - MediaWiki I18N
 */

class TopMenuBarCustomizer extends BsExtensionMW {

	public function __construct() {
		global $wgExtensionMessagesFiles;
		$wgExtensionMessagesFiles['TopMenuBarCustomizer'] = dirname( __FILE__ ) . '/TopMenuBarCustomizer.i18n.php';
		// Base settings
		$this->mExtensionFile = __FILE__;
		$this->mExtensionType = EXTTYPE::VARIABLE;
		$this->mInfo = array(
			EXTINFO::NAME        => 'TopMenuBarCustomizer',
			EXTINFO::DESCRIPTION => 'Customize the Top Menu Links.',
			EXTINFO::AUTHOR      => 'Patric Wirth',
			EXTINFO::VERSION     => '1.21.0 ($Rev: 9690 $)',
			EXTINFO::STATUS      => 'stable',
			EXTINFO::URL         => 'http://www.hallowelt.biz',
			EXTINFO::DEPS        => array(
											'bluespice' => '1.21.0'
										)
		);

		$this->mExtensionKey = 'TopMenuBarCustomizer';
	}

	public function initExt() {
		//TODO: Add some error massages on article save (more than 5 entrys etc.)
		//TODO: Insert existing menu entries when MediaWiki:TopBarMenu is going to be first created
		$this->setHook('BlueSpiceSkin:ApplicationList','onBlueSpiceSkinApplicationList');
	}

	public function onBlueSpiceSkinApplicationList( &$aApplications, &$sCurrentApplicationContext, $wgUser){
		$oTopBarMenuTitle = Title::newFromText( "MediaWiki:TopBarMenu" );
		if( !$oTopBarMenuTitle->exists() ) return true;
		
		$oArticle = new Article ( $oTopBarMenuTitle, 0);
		$newAppList = $oArticle->fetchContent( 0 );

		// force unset Applications by create an empty page
		if( $newAppList === "" ) {
			$aKeys = array_keys( $aApplications );
			foreach($aKeys as $key) {
				unset( $aApplications[$key] );
			}
			return true;
		}

		$sParseError = "";
		$aNewApplications = array();
		$newAppList = explode( "\n", trim( $newAppList ) );
		foreach( $newAppList as $newApp ) {
			if( strpos( $newApp, "*") === false ) {
				continue;
			}
			$newApp = substr( $newApp, strpos($newApp, "*")+1 );
			
			if( count($aNewApplications) >= 5 || !empty($sParseError) ) {
				break;
			}

			if( !empty( $newApp ) ) {
				$newAppParts = explode( '|', trim ( $newApp ) );
				$newApp = array(
					'name' => $newAppParts[0],
					'displaytitle' => $newAppParts[0],
					'url' => $newAppParts[0],
				);
				if( empty($newAppParts[0]) ) continue;

				if( !empty( $newAppParts[2] ) ) {
					$aParsedUrl = wfParseUrl( $newAppParts[2] );
					if( $aParsedUrl !== false ) {
						if(preg_match('# |\\*#',$aParsedUrl['host'])) {
							$sParseError = $newApp;
						}
						if( $aParsedUrl['scheme'] == 'http' || $aParsedUrl['scheme'] == 'https' ) {
							$sQuery = !empty( $aParsedUrl['query'] ) ? '?'.$aParsedUrl['query'] : '';
							$newApp['url'] = $aParsedUrl['scheme'].$aParsedUrl['delimiter'].$aParsedUrl['host'].$aParsedUrl['path'].$sQuery;
						} 
					} else {
						$oTitle = Title::newFromText( trim($newAppParts[2]) );
						if( is_null($oTitle) ) {
							$sParseError = $newApp;
						} else {
							$newApp['url'] = $oTitle->getLinkUrl();
						}
					}
				}

				if( !empty( $newAppParts[1] ) ) {
					$newApp['displaytitle'] = $newAppParts[1];
				} else {
					$newApp['displaytitle'] = wfMessage( 'bs-topmenubarcustomizer-no-display-title' )->plain();
				}

				//Unset old menu entries with the same id
				foreach($aApplications as $key => $aOldApp) {
					if( $aOldApp['name'] == $newApp['name'] ) {
						if( empty($newAppParts[2]) ) {
							//no new url given - use old url
							$newApp['url'] = $aOldApp['url'];
						}
						if( empty($newAppParts[1]) ) {
							//no new display title - use old displaytitle
							$newApp['displaytitle'] = $aOldApp['displaytitle'];
						}
						break;
					}
				}
				$aNewApplications[] = $newApp;
			}
		}
		
		//Replace all old entrys
		$aApplications = $aNewApplications;

		//Replace all entrys with error line
		if( !empty($sParseError) ) {
			if( empty($newAppList[0]) ) {
				unset($newAppList[0]);
			}

			$aApplications = array(
				array(
					'name' => 'parseerror',
					'displaytitle' => wfMessage( 'bs-topmenubarcustomizer-parseerror', $newAppList )->plain(),
					'url' => $oTopBarMenuTitle->getFullURL(),
				),
			);

		}

		return true;
	}
}