<?php
/**
 * UniversalExport PDF Module extension for BlueSpice
 *
 * Enables MediaWiki to export pages into PDF format.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * This file is part of BlueSpice for MediaWiki
 * For further information visit http://www.blue-spice.org
 *
 * @author     Robert Vogel <vogel@hallowelt.biz>
 * @version    1.20.0
 * @version    $Id: UEModulePDF.class.php 9690 2013-06-12 10:13:02Z pwirth $
 * @package    BlueSpice_Extensions
 * @subpackage UEModulePDF
 * @copyright  Copyright (C) 2011 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 * @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
 * @filesource
 */

/* Changelog
 * v1.20.0
 * - Added feature "Suppress namespace"
 * v1.1.0
 * - Support for attachments
 * - Rebuild a lot of code
 * - Added template support
 * - Reduced use of deprecated BsEventDispatcher
 * v1.0.0
 * - Initial release
 */

/**
 * Base class for UniversalExport PDF Module extension
 * @package BlueSpice_Extensions
 * @subpackage UEModulePDF
 */
class UEModulePDF extends BsExtensionMW {

	/**
	 * Contructor of the UEModulePDF class
	 */
	public function __construct() {
		wfProfileIn( 'BS::'.__METHOD__ );
		global $wgExtensionMessagesFiles;
		$wgExtensionMessagesFiles['UEModulePDF'] = dirname( __FILE__ ) . '/UEModulePDF.i18n.php';

		// Base settings
		$this->mExtensionFile = __FILE__;
		$this->mExtensionType = EXTTYPE::SPECIALPAGE;

		$this->mInfo = array(
			EXTINFO::NAME        => 'UEModulePDF',
			EXTINFO::DESCRIPTION => 'Enables MediaWiki to export pages into PDF format.',
			EXTINFO::AUTHOR      => 'Robert Vogel',
			EXTINFO::VERSION     => '1.21.0 ($Rev: 9690 $)',
			EXTINFO::STATUS      => 'stable',
			EXTINFO::URL         => 'http://www.hallowelt.biz',
			EXTINFO::DEPS        => array(
				'bluespice'       => '1.21.0',
				'UniversalExport' => '1.20.0'
			)
		);
		$this->mExtensionKey = 'MW::UEModulePDF';
		wfProfileOut( 'BS::'.__METHOD__ );
	}

	/**
	 * Initialization of UEModulePDF extension
	 */
	protected function initExt() {
		wfProfileIn( 'BS::'.__METHOD__ );
		//Variables
		BsConfig::registerVar( 'MW::UEModulePDF::PdfServiceURL',   'http://localhost:8080/BShtml2PDF', BsConfig::LEVEL_PUBLIC|BsConfig::TYPE_STRING, 'bs-uemodulepdf-pref-PdfServiceURL' );
		BsConfig::registerVar( 'MW::UEModulePDF::PDFenNS',         array( 0 ),  BsConfig::LEVEL_PUBLIC|BsConfig::TYPE_ARRAY_STRING|BsConfig::USE_PLUGIN_FOR_PREFS, 'bs-uemodulepdf-pref-PDFenNS', 'multiselectex' );
		BsConfig::registerVar( 'MW::UEModulePDF::DefaultTemplate', 'BlueSpice', BsConfig::LEVEL_PUBLIC|BsConfig::TYPE_STRING|BsConfig::USE_PLUGIN_FOR_PREFS, 'bs-uemodulepdf-pref-DefaultTemplate' );
		BsConfig::registerVar( 'MW::UEModulePDF::SuppressNS',      false,       BsConfig::LEVEL_PUBLIC|BsConfig::TYPE_BOOL, 'bs-uemodulepdf-pref-SuppressNS', 'toggle' );
		BsConfig::registerVar( 'MW::UEModulePDF::Backend',        'BsPDFServlet', BsConfig::LEVEL_PUBLIC|BsConfig::TYPE_STRING|BsConfig::USE_PLUGIN_FOR_PREFS, 'bs-uemodulepdf-pref-Backend' );
		BsConfig::registerVar( 'MW::UEModulePDF::TemplatePath',   'bluespice-core/data/PDFTemplates', BsConfig::LEVEL_PUBLIC|BsConfig::TYPE_STRING, 'bs-uemodulepdf-pref-TemplatePath' );

		//Hooks
		$this->setHook('BSUniversalExportGetWidget');
		$this->setHook('BSUniversalExportSpecialPageExecute');
		$this->setHook('LoadExtensionSchemaUpdates');

		//Classes
		$sExtensionLibDir = dirname(__FILE__) . DS . 'lib';
		BsCore::registerClass( 'BsPDFPageProvider',     $sExtensionLibDir );
		BsCore::registerClass( 'BsPDFTemplateProvider', $sExtensionLibDir );
		BsCore::registerClass( 'BsPDFWebService',       $sExtensionLibDir );
		BsCore::registerClass( 'BsPDFServlet',          $sExtensionLibDir );
		BsCore::registerClass( 'BsExportModulePDF',     $sExtensionLibDir );

		wfProfileOut( 'BS::'.__METHOD__ );
	}

	/**
	 * Sets parameters for more complex options in preferences
	 * @param string $sAdapterName Name of the adapter, e.g. MW
	 * @param BsConfig $oVariable Instance of variable
	 * @return array Preferences options
	 */
	public function runPreferencePlugin( $sAdapterName, $oVariable ) {
		$aPrefs = array();
		switch( $oVariable->getName() ) {
			case 'PDFenNS': // TODO RBV (24.05.12 15:16): Move to Bookshelf
			case 'PDFwithATTenNS':
				$aPrefs = array(
					'type'    => 'multiselectex',
					'options' => BsAdapterMW::getNamespacesForSelectOptions( array( -2, NS_MEDIA, NS_MEDIAWIKI, NS_MEDIAWIKI_TALK ) )
				);

				break;
			case 'DefaultTemplate':
				$aParams = array( 'template-path' => BsConfig::get('MW::UEModulePDF::TemplatePath') );
				$aPrefs = array(
					'type'    => 'select',
					'options' => BsPDFTemplateProvider::getTemplatesForSelectOptions( $aParams )
				);
				break;
			case 'Backend':
				$aPrefs = array(
					'type'    => 'select',
					'options' => array(
						'BShtml2PDF.war' => 'BsPDFServlet',
						'bn2pdf.war'     => 'BsPDFWebService',
					)
				);
				break;
			default:
				break;
		}
		return $aPrefs;
	}
	
	/**
	 * Sets up requires directories
	 * @param DatabaseUpdater $updater Provided by MediaWikis update.php
	 * @return boolean Always true to keep the hook running
	 */
	public function onLoadExtensionSchemaUpdates( $updater ) {
		//TODO: Create abstraction in Core/Adapter
		$sTmpDir = BSDATADIR.DS.'UEModulePDF';
		if( !file_exists( $sTmpDir ) ) {
			echo 'Directory "'.$sTmpDir.'" not found. Creating.'."\n";
			mkdir( $sTmpDir );
		}
		else {
			echo 'Directory "'.$sTmpDir.'" found.'."\n";
		}
		
		return true;
	}

	/**
	 *
	 * @param SpecialUniversalExport $oSpecialPage
	 * @param string $sParam
	 * @param array $aModules
	 * @return true 
	 */
	public function onBSUniversalExportSpecialPageExecute( $oSpecialPage, $sParam, &$aModules ) {
		$aModules['pdf'] = new BsExportModulePDF();
		return true;
	}

	/**
	 * Event-Handler method for the 'BSUniversalExportCreateWidget' event.
	 * Registers the PDF Module with the UniversalExport Extension.
	 * @param BsEvent $oEvent
	 * @param array $aModules
	 * @return array
	 * @deprecated in 1.1.1
	 */
	public function onUniversalExportSpecialPageExecute( $oCurrentTitle, $oSpecialPage, $aCurrentQueryParams, $aModules) {
		$aModules['pdf'] = new BsExportModulePDF();
		return $aModules;
	}

	/**
	 * Hook-Handler method for the 'BSUniversalExportGetWidget' event.
	 * @param UniversalExport $oUniversalExport
	 * @param array $aModules
	 * @param Title $oSpecialPage
	 * @param Title $oCurrentTitle
	 * @param array $aCurrentQueryParams
	 * @return boolean Always true to keep hook running
	 */
	public function onBSUniversalExportGetWidget( $oUniversalExport, &$aModules, $oSpecialPage, $oCurrentTitle, $aCurrentQueryParams ) {
		$aCurrentQueryParams['ue[module]'] = 'pdf';
		$aLinks = array();
		if( in_array( $oCurrentTitle->getNamespace(), BsConfig::get('MW::UEModulePDF::PDFenNS') ) ) {
			$aLinks['pdf-single-no-attachments'] = array(
					'URL'     => htmlspecialchars( $oSpecialPage->getLinkUrl( $aCurrentQueryParams ) ),
					'TITLE'   => wfMsg( 'bs-uemodulepdf-widgetlink-single-no-attachments-title' ),
					'CLASSES' => 'bs-uemodulepdf-single',
					'TEXT'    => wfMsg( 'bs-uemodulepdf-widgetlink-single-no-attachments-text' ),
				);
		}

		wfRunHooks( 'BSUEModulePDFBeforeCreateWidget', array( $this, $oSpecialPage, &$aLinks, $aCurrentQueryParams ) );

		$oPdfView = new ViewBaseElement();
		$oPdfView->setAutoWrap( '<ul>###CONTENT###</ul>' );
		$oPdfView->setTemplate( '<li><a href="{URL}" rel="nofollow" title="{TITLE}" class="{CLASSES}">{TEXT}</a></li>' );#

		foreach( $aLinks as $sKey => $aData ) {
			$oPdfView->addData( $aData );
		}

		$aModules[] = $oPdfView;
		return true;
	}
}
