<?php
/**
 * BsUniversalExportHelper.
 *
 * Part of BlueSpice for MediaWiki
 *
 * @author     Robert Vogel <vogel@hallowelt.biz>
 * @version    $Id: UniversalExportHelper.class.php 7044 2012-10-29 13:21:04Z rvogel $
 * @package    BlueSpice_Extensions
 * @subpackage UniversalExport
 * @copyright  Copyright (C) 2011 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 * @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
 * @filesource
 */

/**
 * UniversalExport BsUniversalExportHelper class.
 * @package BlueSpice_Extensions
 * @subpackage UniversalExport
 */
class BsUniversalExportHelper {
	/**
	 * Extracts the parameters from the querystring and merges it wir the
	 * default and overrige settings of the UniversalExport Extension.
	 * @param array $aParams
	 */
	public static function getParamsFromQueryString( &$aParams ) {
		$oWebRequest = BsCore::getInstance( 'MW' )->getAdapter()->get( 'Request' );
		$aParamsOverrides = BsConfig::get( 'MW::UniversalExport::ParamsOverrides' );
		$aParams = array_merge( $aParams, $oWebRequest->getArray( 'ue', array() ) );
		$aParams = array_merge( $aParams, $aParamsOverrides );
		$aParams['oldid']  = $oWebRequest->getVal( 'oldid', 0 );
		$sDirection = $oWebRequest->getVal( 'direction', '' );
		if( !empty ( $sDirection ) ){
			$aParams['direction'] = $sDirection;
		}
	}

	public static function checkPermissionForTitle( $oTitle, &$aParams ) {
		$oCurrentUser = BsCore::getInstance( 'MW' )->getAdapter()->get( 'User' );
		
		$bErrorOccured = false;
		foreach( $aParams as $sValue ) {
			if ( $oTitle->getNamespace() == NS_SPECIAL ) {
				switch( $sValue ) {
					case 'recursive':
						if( !$oCurrentUser->isAllowed( 'universalexport-export-recursive' ) ) $bErrorOccured = true;
						break;
					case 'with-attachments':
						if( !$oCurrentUser->isAllowed( 'universalexport-export-with-attachments' ) ) $bErrorOccured = true;
						break;
					case 'unfiltered':
						if( !$oCurrentUser->isAllowed( 'universalexport-export-unfiltered' ) ) $bErrorOccured = true;
						break;
				}
			}
			else{
				switch( $sValue ) {
					case 'recursive':
						if( !$oTitle->userCan( 'universalexport-export-recursive' ) ) $bErrorOccured = true;
						break;
					case 'with-attachments':
						if( !$oTitle->userCan( 'universalexport-export-with-attachments' ) ) $bErrorOccured = true;
						break;
					case 'unfiltered':
						if( !$oTitle->userCan( 'universalexport-export-unfiltered' ) ) $bErrorOccured = true;
						break;
				}
			}
		}


		if( $bErrorOccured ) throw new Exception ( 'error-no-permission' );
	}

	public static function getCategoriesForTitle( $oTitle ) {
		/* Title::getParentCategories() returns an array like this:
		 * array (
		 *  'Category:Foo' => 'My Article',
		 *  'Category:Bar' => 'My Article',
		 *  'Category:Baz' => 'My Article',
		 * )
		 */
		$aCategories         = $oTitle->getParentCategories();
		$aSimpleCategoryList = array();
		if( !empty( $aCategories ) ) {
			foreach( $aCategories as $sCategoryPageName => $sCurrentTitle ) {
				$aCategoryPageNameParts = explode( ':', $sCategoryPageName );
				$aSimpleCategoryList[]  = $aCategoryPageNameParts[1];
			}
		}
		return $aSimpleCategoryList;
	}

	/**
	 * Finds suitable headlines in $oPageDOM and creates returns a <bookmarks /> element with links to them
	 * @param DOMDocument $oPageDOM
	 * @return DOMElement 
	 */
	public static function getBookmarkElementForPageDOM( $oPageDOM ) {	
		$oBookmarksDOM = new DOMDocument();
		//$aHeadingWeigths = array_flip( array( 'h6', 'h5', 'h4', 'h3', 'h2', 'h1' ) );
		//h6 has the lowest weigth with 0, h1 the highest with 5. For later use...
		
		//HINT: http://calibre-ebook.com/user_manual/xpath.html
		$oBodyContentXPath = new DOMXPath( $oPageDOM );
		$oHeadingElements  = $oBodyContentXPath->query( "//*[contains(@class, 'firstHeading') or contains(@class, 'mw-headline')]" );

		//By convention the first <h1> in the PageDOM is the title of the page
		$oPageTitleBookmarkElement    = $oBookmarksDOM->createElement( 'bookmark' );
		$oPageTitleHeadingElement     = $oHeadingElements->item( 0 );
		$sPageTitleHeadingTextContent = trim( $oPageTitleHeadingElement->textContent );

		//By convention previousSibling is an Anchor-Tag (see BsPageContentProvider)
		$sPageTitleHeadingJumpmark = self::findPreviousDOMElementSibling( $oPageTitleHeadingElement, 'a' )->getAttribute( 'name' );
		$oPageTitleBookmarkElement->setAttribute( 'name', $sPageTitleHeadingTextContent );
		$oPageTitleBookmarkElement->setAttribute( 'href', '#'.$sPageTitleHeadingJumpmark );

		//Adapt MediaWiki TOC #1
		//TODO RBV (01.02.11 15:01): EVENT!
		$oTocTableElement        = $oBodyContentXPath->query( "//*[@id='toc']" );
		$oTableOfContentsAnchors = array();
		if ( $oTocTableElement->length > 0 ) { //Is a TOC available?
			// HINT: http://de.selfhtml.org/xml/darstellung/xpathsyntax.htm#position_bedingungen - recursive descent operator = getElementsByTag
			$oTableOfContentsAnchors = $oBodyContentXPath->query( "//*[@id='toc']//a" );
			$oTocTableElement->item( 0 )->setAttribute( 'id', 'toc-'.$sPageTitleHeadingJumpmark ); //make id unique
			$oTocTitleElement = $oBodyContentXPath->query( "//*[@id='toctitle']" )->item(0);
			$oTocTitleElement->setAttribute( 'id', 'toctitle-'.$sPageTitleHeadingJumpmark ); //make id unique;
			$oTocTitleElement->setAttribute( 'class', 'toctitle' );
		}

		for ( $i = 1; $i < $oHeadingElements->length; $i++ ) {
			$oHeadingElement     = $oHeadingElements->item( $i );
			$sHeadingTextContent = trim( $oHeadingElement->textContent );
			//In $sPageTitleHeadingJumpmark there is the PageTitle AND the RevisionId incorporated
			$sHeadingJumpmark    = 'bs-ue-jumpmark-'.md5( $sPageTitleHeadingJumpmark.$sHeadingTextContent );

			$oBookmarkElement = $oBookmarksDOM->createElement( 'bookmark' );
			$oBookmarkElement->setAttribute( 'name', $sHeadingTextContent );
			$oBookmarkElement->setAttribute( 'href', '#'.$sHeadingJumpmark );

			$oPageTitleBookmarkElement->appendChild( $oBookmarkElement );

			$oHeadingElementAnchor = self::findPreviousDOMElementSibling( $oHeadingElement, 'a' );
			if( $oHeadingElementAnchor !== null ) {
				
				$sOrigialNameValue = $oHeadingElementAnchor->getAttribute( 'name' );
				$oHeadingElementAnchor->setAttribute( 'name', $sHeadingJumpmark );

				//Adapt MediaWiki TOC #2
				// TODO RBV (01.02.11 14:58): Make this better
				foreach( $oTableOfContentsAnchors as $oTOCAnchorElement ) {
					if( $oTOCAnchorElement->getAttribute( 'href' ) == '#'.$sOrigialNameValue ) {
						//error_log( 'Replacing href "#'.$sOrigialNameValue.'" with "'.$sHeadingJumpmark.'"' );
						$oTOCAnchorElement->setAttribute( 'href', '#'.$sHeadingJumpmark );
					}
				}

			}else {
				//Inject a new anchor for the PDF bookmarks
				$oNewAnchorTag = $oPageDOM->createElement( 'a' );
				$oNewAnchorTag->setAttribute( 'name' , $sHeadingJumpmark );
				$oHeadingElement->insertBefore( $oNewAnchorTag );
			}

			
		}

		return $oPageTitleBookmarkElement;
	}

	//Seems not to work...
	//HINT: http://www.php.net/manual/en/domdocument.validate.php#99818
	public static function ensureGetElementByIdAccessibility( DOMNode &$oNode ) {
		if( $oNode->hasChildNodes() ) {
			foreach( $oNode->childNodes as $oChildNode ) {
				if( $oChildNode->hasAttributes() ) {
					$sId = $oChildNode->getAttribute( 'id' );
					if( $sId ) {
						$oChildNode->setAttribute( 'id', $sId );
					}
				}
				self::ensureGetElementByIdAccessibility( $oChildNode );
			}
		}
	}

	public static function findPreviousDOMElementSibling( DOMNode &$oDOMNode, $sWantedNodeName = '' ) {
		$oDOMNodesPrevSibling = $oDOMNode->previousSibling;

		if( $oDOMNodesPrevSibling !== null ) {
			if( $oDOMNodesPrevSibling->nodeType == XML_ELEMENT_NODE ) {
				if( !empty( $sWantedNodeName ) ) {
					if( $oDOMNodesPrevSibling->nodeName == $sWantedNodeName ) {
						return $oDOMNodesPrevSibling;
					}
				}
				else {
					return $oDOMNodesPrevSibling;
				}
			}
			return self::findPreviousDOMElementSibling( $oDOMNodesPrevSibling, $sWantedNodeName );
		}
		
		return null;
	}
}