<?php

/**
 * blue spice for MediaWiki
 * Extension: UserPreferences
 * Description: Renders the blue spice tab in preferences
 * Authors: Sebastian Ulbricht
 *
 * Copyright (C) 2010 Hallo Welt! � Medienwerkstatt GmbH, All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * http://www.gnu.org/copyleft/gpl.html
 *
 * For further information visit http://www.blue-spice.org
 *
 * Version information
 * $LastChangedDate: 2013-06-12 12:13:02 +0200 (Mi, 12. Jun 2013) $
 * $LastChangedBy: pwirth $
 * $Rev: 9690 $
 * $Id: UserPreferences.class.php 9690 2013-06-12 10:13:02Z pwirth $
 */
/* Changelog
 * v1.20.0
 * - MediaWiki I18N
 * v1.0.0
 * - raised to stable
 * v0.3.0b
 * - Code Review (2010-10-01)
 * - Refactored Code for compliance to new coding conventions
 * v0.2.0b
 * - Refactored / beautified code
 * - Using new database table scheme from WhosOnline Extension
 */

// Last review MRG (13.10.10 23:23)


class UserPreferences extends BsExtensionMW {

	public function __construct() {
		wfProfileIn( 'BS::' . __METHOD__ );
		global $wgExtensionMessagesFiles;
		$wgExtensionMessagesFiles['UserPreferences'] = dirname( __FILE__ ) . '/UserPreferences.i18n.php';

		// Base settings
		$this->mExtensionPath = __FILE__;
		$this->mExtensionType = EXTTYPE::VARIABLE;
		$this->mInfo = array(
			EXTINFO::NAME => 'UserPreferences',
			EXTINFO::DESCRIPTION => 'Renders the BlueSpice tab in preferences.',
			EXTINFO::AUTHOR => 'Sebastian Ulbricht, Stephan Muggli',
			EXTINFO::VERSION => '1.21.0 ($Rev: 9690 $)',
			EXTINFO::STATUS => 'stable',
			EXTINFO::URL => 'http://www.hallowelt.biz',
			EXTINFO::DEPS => array( 'bluespice' => '1.21.0' )
		);
		$this->mExtensionKey = 'MW::UserPreferences';
		wfProfileOut( 'BS::' . __METHOD__ );
	}

	protected function initExt() {
		wfProfileIn( 'BS::' . __METHOD__ );
		//Styles
		$this->registerStyleSheet( BsConfig::get( 'MW::ScriptPath' ) . '/bluespice-mw/ext/UserPreferences/styles/bs-userpreferences.css', true, 'MW::UserPreferences::Form' );

		//Hooks
		$this->setHook( 'GetPreferences' );
		$this->setHook( 'UserLoadOptions' );
		$this->setHook( 'UserSaveOptions' );
		wfProfileOut( 'BS::' . __METHOD__ );
	}

	/**
	 * Hook handler for GetPreferences
	 * @param User $user User whose preferences are being modified.
	 * @param Array &$preferences Preferences description array, to be fed to an HTMLForm object
	 * @return true always true to keep hook alive
	 */
	public function onGetPreferences( $user, &$preferences ) {
		BsConfig::loadUserSettings( $user );
		$bOrigDeliver = BsConfig::deliverUsersSettings( true );
		BsAdapterMW::loadHtmlFormClass();
		$aRegisteredVariables = BsConfig::getRegisteredVars();
		$aSortedVariables = array();

		foreach ( $aRegisteredVariables as $oVariable ) {
			$iOptions = $oVariable->getOptions();

			if ( !( $iOptions & ( BsConfig::LEVEL_USER ) ) )
				continue;

			$sAdapterName = strtoupper( $oVariable->getAdapter() );
			$sExtensionName = $oVariable->getExtension();

			if ( empty( $sExtensionName ) ){
				$sExtensionName = "BASE";
				$sExtensionTranslation = wfMsg( 'CORE' );
			}
			else {
				$sExtensionNameLower = strtolower( $sExtensionName );
				$sExtensionTranslation = wfMsg( 'prefs-' . $sExtensionNameLower );
			}
			$aSortedVariables[ $sAdapterName ][$sExtensionTranslation][ $sExtensionName][ ] = $oVariable;
			ksort($aSortedVariables[$sAdapterName]);
		}

		foreach ( $aSortedVariables as $sAdapterName => $aExtensions ) {
			if ( !count( $aExtensions ) )
				continue;
			$sBaseSection = 'bluespice/' . $sAdapterName;
			foreach ($aExtensions as $val){
				if ( !count( $val ) )
					continue;
				foreach ( $val as $sExtensionName => $aSettings ) {
					// if continue, then $oAdapterSetView is not added to output
					if ( !count( $aSettings ) )
						continue;
					$sSection = $sBaseSection . '/' . $sExtensionName;

					foreach ( $aSettings as $oVariable ) {
						$field = array(
							'type' => $oVariable->getFieldMapping(),
							'label-message' => $oVariable->getI18nName(), // a system message
							'section' => $sSection,
							'default' => $oVariable->getValue()
						);
						if ( $oVariable->getFieldMapping() == 'multiselectplusadd' ) {
							$field[ 'options' ] = $oVariable->getValue();
							$field[ 'title' ] = 'toc-' . $oVariable->getName() . '-title';
							$field[ 'message' ] = 'toc-' . $oVariable->getName() . '-message';
						}
						if ( $oVariable->getOptions() & BsConfig::USE_PLUGIN_FOR_PREFS ) {
							$tmp = NULL;
							if ( $sExtensionName == 'BASE' ) {
								$tmp = $this->mAdapter->runPreferencePlugin( 'MW', $oVariable );
							} else {
								$oExtension = BsExtensionManager::getExtension( $sExtensionName );
								$tmp = $oExtension->runPreferencePlugin( 'MW', $oVariable );
							}
							$field = array_merge( $field, $tmp );
						}

						$preferences[ $oVariable->generateFieldId() ] = $field;
					}
				}
			}
		}
		BsConfig::deliverUsersSettings( $bOrigDeliver );

		return true;
	}

	/**
	 * Hook handler for UserLoadOptions
	 * @param User $user User whose options are being modified.
	 * @param Array &$options Options array
	 * @return true always true to keep hook alive
	 */
	public function onUserLoadOptions( $user, &$options ) {
		$aRegisteredVariables = BsConfig::getRegisteredVars();
		$aSortedVariables = array( );
		foreach ( $aRegisteredVariables as $oVariable ) {
			$iOptions = $oVariable->getOptions();

			if ( !( $iOptions & ( BsConfig::LEVEL_USER ) ) )
				continue;

			$sAdapterName = strtoupper( $oVariable->getAdapter() );
			$sExtensionName = $oVariable->getExtension();
			if ( empty( $sExtensionName ) )
				$sExtensionName = 'BASE';
			$aSortedVariables[ $sAdapterName ][ $sExtensionName ][ ] = $oVariable;
		}

		foreach ( $aSortedVariables as $sAdapterName => $aExtensions ) {
			if ( !count( $aExtensions ) )
				continue;

			foreach ( $aExtensions as $sExtensionName => $aSettings ) {
				if ( !count( $aSettings ) )
					continue;

				foreach ( $aSettings as $oVariable ) {
					$id = $oVariable->getKey();
					if ( isset( $options[ $id ] ) ) {
						$options[ $id ] = unserialize( $options[ $id ] );
						BsConfig::set( $id, $options[ $id ] );
					}
				}
			}
		}

		return true;
	}

	/**
	 * Hook handler for UserSaveOptions
	 * @param User $user User whose options are being modified.
	 * @param Array &$options Options array
	 * @return true always true to keep hook alive
	 */
	public function onUserSaveOptions( $user, &$options ) {
		BsConfig::loadSettings();

		$aRegisteredVariables = BsConfig::getRegisteredVars();
		$aSortedVariables = array();

		foreach ( $aRegisteredVariables as $oVariable ) {
			$iOptions = $oVariable->getOptions();

			if ( !( $iOptions & ( BsConfig::LEVEL_USER ) ) )
				continue;

			$sAdapterName = strtoupper( $oVariable->getAdapter() );
			$sExtensionName = $oVariable->getExtension();
			if ( empty( $sExtensionName ) )
				$sExtensionName = 'BASE';
			$aSortedVariables[ $sAdapterName ][ $sExtensionName ][ ] = $oVariable;
		}

		foreach ( $aSortedVariables as $sAdapterName => $aExtensions ) {
			if ( !count( $aExtensions ) )
				continue;
			$sBaseSection = 'bluespice/' . $sAdapterName;

			foreach ( $aExtensions as $sExtensionName => $aSettings ) {
				// if continue, then $oAdapterSetView is not added to output
				if ( !count( $aSettings ) )
					continue;
				$sSection = $sBaseSection . '/' . $sExtensionName;

				BsConfig::deliverUsersSettings( false );
				foreach ( $aSettings as $oVariable ) {

					//Avoid "undefined index" notices and weird NULL values in settings
					$value = $oVariable->getValue();

					if( isset( $options[ $oVariable->generateFieldId() ] ) ) { //Set but no bool
						$value = $options[ $oVariable->generateFieldId() ];
					}
					if( isset( $options[ $oVariable->getKey() ] ) ) { //Set but no bool
						$value = $options[ $oVariable->getKey() ];
					}

					$options[ $oVariable->getKey() ] = serialize( $value );
					unset( $options[ $oVariable->generateFieldId() ] );
				}
				BsConfig::deliverUsersSettings( true );
			}
		}

		return true;
	}

}