/**
 * WantedArticle extension
 *
 * Part of BlueSpice for MediaWiki
 *
 * @author     Markus Glaser <glaser@hallowelt.biz>
 * @author     Robert Vogel <vogel@hallowelt.biz>
 * @version    $Id: WantedArticle.js 7127 2012-11-06 13:43:53Z rvogel $
 * @package    Bluespice_Extensions
 * @subpackage WantedArticle
 * @copyright  Copyright (C) 2011 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 * @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
 * @filesource
 */

/**
 * Base class for all WantedArticle related methods and properties
 */
BsWantedArticle = {
	config: {
		urlBase: wgServer + wgScriptPath,
		urlAddWantedArticle: BlueSpice.buildRemoteString( 'WantedArticle', 'ajaxAddWantedArticle' ),
		textFieldDefaultValue: '',
		i18n: {
			info_dialog_title                     : 'Information',
			info_nothing_entered                  : 'Please provide a valid title.',
			info_title_contains_invalid_characters: 'The following characters are not valid for use in an article title: ' + '<br />'
		}
	},

	oForm:          null,
	oCreateButton:  null,
	oSuggestButton: null,
	oTextField:     null,

	sTextFieldDefaultValue: '',
	sTextFieldTmp:          '',

	init: function() {
		//ExtendedSearch
		$( '#bs-extendedsearch-suggest' ).live( 'click', function() { //Has to be "live" because #bs-extendedsearch-suggest may be changed via AJAX
			//TODO: $.live() is deprecated since v1.7. Replace with $.on() as soon as we drop MW 1.17 support.
			return BsWantedArticle.sendSuggestion( $( this ).attr( 'href' ).substr( 1 ) );
		});
		
		this.oForm          = $( '#bs-wantedarticle-form' );
		this.oTextField     = $( '#bs-wantedarticle-composite-textfield' );
		this.oCreateButton  = $( '#bs-wantedarticle-createbutton' );
		this.oSuggestButton = $( '#bs-wantedarticle-suggestbutton' );

		this.sTextFieldDefaultValue = this.oTextField.val();

		this.config.textFieldDefaultValue = this.sTextFieldDefaultValue;

		this.oTextField.focus( function() {
			if( $(this).val() == BsWantedArticle.sTextFieldDefaultValue ) $(this).val( '' );
			$(this).removeClass( 'bs-unfocused-textfield' );
		}).blur( function() {
			if( $(this).val() == '' ) $(this).val( BsWantedArticle.sTextFieldDefaultValue );
			$(this).addClass( 'bs-unfocused-textfield' );
		});

		this.oCreateButton.click( function() {
			return BsWantedArticle.navigateToTarget(
			BsWantedArticle.oTextField.val()
			);
		});

		this.oSuggestButton.click( function() {
			return BsWantedArticle.sendSuggestion(
			BsWantedArticle.oTextField.val()
			);
		});

		this.oTextField.keypress( function(e) {
			if(e.keyCode == 13) { //Enter
				e.preventDefault();
				BsWantedArticle.oCreateButton.click();
			}
		});

		this.oForm.submit( function() {
			var sTitle = BsWantedArticle.oTextField.val();
			if( bsWantedArticleShowCreate == true ){
				return BsWantedArticle.navigateToTarget( sTitle );
			}
			else{
				return BsWantedArticle.sendSuggestion( sTitle );
			}
		});
	},

	checkArticleTitle: function( sArticleTitle ) { // TODO RBV (06.10.10 09:42): Should be in common framework
		if ( sArticleTitle == '' || sArticleTitle == BsWantedArticle.config.textFieldDefaultValue ) {
			BlueSpice.alert( BsWantedArticle.config.i18n.info_nothing_entered )
			/* 
			// Please leave for further reference
			Ext.Msg.show({
				title  : this.config.i18n.info_dialog_title,
				msg    : this.config.i18n.info_nothing_entered,
				buttons: Ext.Msg.OK,
				icon   : Ext.MessageBox.INFO
			});
			*/
			return false;
		}

		var aFoundChars = new Array();
		for ( var i=0; i < bsForbiddenCharsInArticleTitle .length; i++ ) {
			if ( sArticleTitle.indexOf( bsForbiddenCharsInArticleTitle [i] ) != -1 ) {
				aFoundChars.push( '"' + bsForbiddenCharsInArticleTitle [i] + '"' );
			}
		}
		if( aFoundChars.length > 0 ) {
			BlueSpice.alert( BsWantedArticle.config.i18n.info_title_contains_invalid_characters + aFoundChars.join( ', ' ) );
			/*
			// Please leave for further reference
			Ext.Msg.show({
				title  : this.config.i18n.info_dialog_title,
				msg    : this.config.i18n.info_title_contains_invalid_characters + aFoundChars.join(', '),
				buttons: Ext.Msg.OK,
				icon   : Ext.MessageBox.INFO
			});
			*/
			return false;
		}
		return true;
	},

	navigateToTarget: function( sArticleTitle ) {
		if ( this.checkArticleTitle( sArticleTitle ) != true )  return false;

		this.sTextFieldTmp = sArticleTitle;
		this.oForm.trigger( 'bs-wantedarticle-navigateToTarget' );

		sArticleTitle = this.sTextFieldTmp.replace( ' ', '_' );
		var sUrl = this.config.urlBase + '/index.php?title=' + encodeURIComponent( sArticleTitle );
		document.location.href = sUrl;

		return false;
	},

	sendSuggestion: function( sArticleTitle ) {
		if ( this.checkArticleTitle( sArticleTitle ) != true ) return false;

		this.sTextFieldTmp = sArticleTitle;
		this.oForm.trigger( 'bs-wantedarticle-sendSuggestion' );

		$.getJSON( 
			this.config.urlAddWantedArticle,
			{ 'suggestion' : this.sTextFieldTmp },
			function( oData, oTextStatus ) {
				BlueSpice.alert( oData[1] );
				/* 
				// Please leave for further reference
				Ext.Msg.show({
					title  : BsWantedArticle.config.i18n.info_dialog_title,
					msg    : oData[1],
					buttons: Ext.Msg.OK,
					icon   : Ext.MessageBox.INFO
				});*/
				if( oData[0] == 'SUC' ) {
					BsWantedArticle.oTextField.val( 
					BsWantedArticle.config.textFieldDefaultValue
					);
				}
			}
		);

		return false;
	}
}

$(document).ready(function(){
	BsWantedArticle.init();
});

// Register with ExtendedSearch Autocomplete
$(document).bind('BSExtendedSearchAutocompleteItemSelect', function( event, selectEvent, ui, status ){
	if ( ui.item.attr !== 'bs-extendedsearch-suggest' ) return;

	BsWantedArticle.sendSuggestion( ui.item.value );
	status.skipFurtherProcessing = true;
});