<?php
/**
 * Renders the wanted article form in the left column.
 *
 * Part of BlueSpice for MediaWiki
 *
 * @author     Robert Vogel <vogel@hallowelt.biz>
 * @version    $Id: view.WantedArticleForm.php 7742 2012-12-17 12:10:36Z mglaser $
 * @package    BlueSpice_Extensions
 * @subpackage WantedArticle
 * @copyright  Copyright (C) 2011 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 * @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
 * @filesource
 */

/**
 * This view renders the wanted article form in the left column.
 * @package    BlueSpice_Extensions
 * @subpackage WantedArticle
 */
class ViewWantedArticleForm extends ViewBaseElement {
	static $iTagFormCounter = 0;
	
	protected $bShowCreateArticle  = false;
	protected $bShowSuggestArticle = false;
	protected $bDisplayHeading = false;
	protected $bDisplayLabels  = false;
	protected $sFormVariant    = 'single-textfield'; //Possible values 'single-textfield', 'seperate-textfields'

	public function execute( $params = false ) {
		$sControls = '';
		switch( $this->sFormVariant ) {
			case 'single-textfield': $sControls = $this->renderSingleTextField();
				break;
			case 'seperate-textfields': $sControls = $this->renderSeperateTextFields();
				break;
			case 'tag-form' : 
				$sControls = $this->renderTagForm();
				self::$iTagFormCounter++;
				return $this->renderDummyForm( $sControls );
			default:
				break;
		}

		return $this->renderForm( $sControls );
	}

	/**
	 *
	 * @return string
	 */
	private function renderSingleTextField() {
		$aOut = array();

		$aOut[] = '<input id="bs-wantedarticle-composite-textfield"';
		$aOut[] = '       name="bs-wantedarticle-composite-textfield"';
		$aOut[] = '       type="text"';
		$aOut[] = '       class="bs-unfocused-textfield"';
		$aOut[] = '       value="'.wfMsg( 'bs-wantedarticle-single-textfield-defaulttext' ).'" />';
		$aOut[] = '<button id="bs-wantedarticle-suggestbutton" class="bs-linkbutton">';
		$aOut[] = wfMsg( 'bs-wantedarticle-single-textfield-suggestbutton-text' );
		$aOut[] = '</button>';
		if( $this->bShowCreateArticle === true ) {
			$aOut[] = '<button id="bs-wantedarticle-createbutton" class="bs-linkbutton">';
			$aOut[] = wfMsg( 'bs-wantedarticle-single-textfield-createbutton-text' );
			$aOut[] = '</button>';
		}

		return implode( "\n", $aOut );
	}
	
		/**
	 *
	 * @return string
	 */
	private function renderTagForm() {
		$aOut = array();

		$aOut[] = '<input id="bs-wantedarticle-composite-textfield-'.self::$iTagFormCounter.'"';
		$aOut[] = '       name="bs-wantedarticle-composite-textfield-'.self::$iTagFormCounter.'"';
		$aOut[] = '       type="text"';
		$aOut[] = '       class="bs-unfocused-textfield"';
		$aOut[] = '       value="'.wfMsg( 'bs-wantedarticle-single-textfield-defaulttext' ).'" />';
		$aOut[] = '<button id="bs-wantedarticle-suggestbutton-'.self::$iTagFormCounter.'" class="bs-linkbutton"';
		$aOut[] = '        onclick = "return BsWantedArticle.sendSuggestion( $( this ).prev().attr( \'value\' ) );"';
		$aOut[] = '>';
		$aOut[] = wfMsg( 'bs-wantedarticle-single-textfield-suggestbutton-text' ).' -formtag';
		$aOut[] = '</button>';

		return implode( "\n", $aOut );
	}
	

	/**
	 *
	 */
	private function renderSeperateTextFields() {
		throw new BsException( 'Not implemented' ); // TODO RBV (06.10.10 11:06): implement
	}

	/**
	 *
	 * @param string $sControls
	 * @return string
	 */
	private function renderForm( $sControls ) {
		$aOut = array();

		$aOut[] = '<div id="bs-wantedarticle-container" class="">';
		$aOut[] = ' <form action="#" id="bs-wantedarticle-form" >';
		$aOut[] = '  <div>';
		$aOut[] = $sControls;
		$aOut[] = '  </div>';
		$aOut[] = ' </form>';
		$aOut[] = '</div>'; // #bs-wantedarticle-container

		return implode( "\n", $aOut );
	}
	
	/**
	 *
	 * @param string $sControls
	 * @return string
	 */
	private function renderDummyForm( $sControls ) {
		$aOut = array();

		$aOut[] = '<div id="bs-wantedarticle-container-'.self::$iTagFormCounter.'" class="">';
		$aOut[] = ' <form action="#" id="bs-wantedarticle-form-'.self::$iTagFormCounter.'" >';
		$aOut[] = '  <div>';
		$aOut[] = $sControls;
		$aOut[] = '  </div>';
		$aOut[] = ' </form>';
		$aOut[] = '</div>'; // #bs-wantedarticle-container

		return implode( "\n", $aOut );
	}

	/**
	 *
	 * @param <type> $bShowCreateArticle
	 * @return ViewWantedArticleForm
	 */
	public function setShowCreateArticle( $bShowCreateArticle ) {
		$this->bShowCreateArticle = $bShowCreateArticle;
		return $this;
	}

	/**
	 *
	 * @param <type> $bShowSuggestArticle
	 * @return ViewWantedArticleForm
	 */
	public function setShowSuggestArticle( $bShowSuggestArticle ) {
		$this->bShowSuggestArticle = $bShowSuggestArticle;
		return $this;
	}

	/**
	 *
	 * @param <type> $bDisplayHeading
	 * @return ViewWantedArticleForm
	 */
	public function setDisplayHeading( $bDisplayHeading ) {
		$this->bDisplayHeading = $bDisplayHeading;
		return $this;
	}

	/**
	 *
	 * @param <type> $bDisplayLables
	 * @return ViewWantedArticleForm
	 */
	public function setDisplayLabels( $bDisplayLables ) {
		$this->bDisplayLabels = $bDisplayLables;
		return $this;
	}

	/**
	 * Influences the rendering of the form element
	 * @param string $sFormVariant Possible values 'single-textfield', 'seperate-textfields', 'tag-form'
	 * @return ViewWantedArticleForm
	 */
	public function setFormVariant( $sFormVariant ) {
		$this->sFormVariant = $sFormVariant;
		return $this;
	}
}
