<?php
/**
 * Renders the <bs:wantedarticles /> tag.
 *
 * Part of BlueSpice for MediaWiki
 *
 * @author     Robert Vogel <vogel@hallowelt.biz>
 * @version    $Id: view.WantedArticleTag.php 7595 2012-12-04 12:25:52Z rvogel $
 * @package    BlueSpice_Extensions
 * @subpackage WantedArticle
 * @copyright  Copyright (C) 2011 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 * @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
 * @filesource
 */

/**
 * This view renders the <bs:wantedarticles /> tag.
 * @package    BlueSpice_Extensions
 * @subpackage WantedArticle
 */
class ViewWantedArticleTag extends ViewBaseElement {

	protected $mTitle       = '';
	protected $mListEntries = array();
	protected $mCount       = 5;
	protected $mType        = 'list';

	public function execute( $params = false ) {
		$aOut         = array();
		$aVisibleList = array();
		$aHiddenList  = array();
		
		foreach( $this->mListEntries as $oTitle ){
			$sLink = sprintf( '<a class="internal %s" href="%s" title="%s">%s</a>',
				$oTitle->exists() == false ? 'new' : '',
				$oTitle->getLocalURL(),
				$oTitle->getPrefixedText(),
				$oTitle->getPrefixedText()
			);
			if( count( $aVisibleList ) < $this->mCount ) {
				$aVisibleList[] = $sLink;
			}
			else {
				$aHiddenList[]  = $sLink;
			}
		}
		
		if( !empty( $this->mTitle ) ) {
			$aOut[] = '<h3>'.$this->mTitle.'</h3>';
		}

		if( $this->mType == 'queue' ) {
			$aOut[] = '<span class="bs-wantedarticle-suggestion">';
			$aOut[] = implode( '</span>, <span class="bs-wantedarticle-suggestion">', $aVisibleList );
			$aOut[] = '</span>';
		}
		else {
			$aOut[] = '<ul>';
			$aOut[] = '  <li class="bs-wantedarticle-suggestion">';
			$aOut[] = implode( '</li><li class="bs-wantedarticle-suggestion">', $aVisibleList );
			$aOut[] = '  </li>';
			$aOut[] = '</ul>';
		}
		if( !empty ( $aHiddenList ) ) {
			if( $this->mType == 'queue' ) {
				$aOut[] = sprintf( ', <a href="#" onclick="$(this).hide();$(this).next().show(); return false;" title="%s">%s</a>',
						wfMsg( 'bs-wantedarticle-tag-more-linktext' ),
						wfMsg( 'bs-wantedarticle-tag-more-linktext' )
						);
				$aOut[] = '<span style="display:none">';
				$aOut[] = '<span class="bs-wantedarticle-suggestion">';
				$aOut[] = implode( '</span>, <span class="bs-wantedarticle-suggestion">', $aHiddenList );
				$aOut[] = '</span>';
				$aOut[] = '</span>';
			}
			else {
				$aOut[] = sprintf( '<a href="#" onclick="$(this).parent().hide();$(this).parent().next().show(); return false;" title="%s">%s</a>',
						wfMsg( 'bs-wantedarticle-tag-more-linktext' ),
						wfMsg( 'bs-wantedarticle-tag-more-linktext' )
						);
				$aOut[] = '<ul style="display:none">';
				$aOut[] = '  <li class="bs-wantedarticle-suggestion">';
				$aOut[] = implode( '</li><li class="bs-wantedarticle-suggestion">', $aHiddenList );
				$aOut[] = '  </li>';
				$aOut[] = '</ul>';
			}
		}
		return implode( "\n", $aOut);
	}

	public function setTitle( $sTitle ) {
		$this->mTitle = $sTitle;
		return $this;
	}

	public function setList( $aList ) {
		$this->mListEntries = $aList;
		return $this;
	}

	public function setCount( $iCount ) {
		$this->mCount = $iCount;
		return $this;
	}

	public function setType( $sType ) {
		$this->mType = $sType;
		return $this;
	}
}