<?php
/**
 * Renders the WhoIsOnline tag.
 *
 * Part of BlueSpice for MediaWiki
 *
 * @author     Markus Glaser <glaser@hallowelt.biz>
 * @version    $Id: view.WhoIsOnlineTag.php 7550 2012-12-03 12:04:49Z swidmann $
 * @package    BlueSpice_Extensions
 * @subpackage WhoIsOnline
 * @copyright  Copyright (C) 2011 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 * @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
 * @filesource
 */

/**
 * This view renders the WhoIsOnline tag.
 * @package    BlueSpice_Extensions
 * @subpackage WhoIsOnline
 */
class ViewWhoIsOnlineTag extends ViewBaseElement {
	
	/**
	 * Counter increments with every instance. Used to separate several instances on one page.
	 * @var int Current number of instance.
	 */
	private static $iCount = 0;
	/**
	 * ID of the tag area.
	 * @var string Any distinct name.
	 */
	protected $sTargetId   = '';

	/**
	 * Constructor
	 */
	public function __construct() {
		parent::__construct();
		$this->setAutoElement(''); // TODO RBV (16.11.10 16:42): Workaround for bug "Parser/Outputhandler breaks markup"
		$this->sTargetId = 'bs-wo-link-'.self::$iCount++;
	}

	/**
	 * Getter for sTargetId.
	 * @return string The target id.
	 */
	public function getTargetId() {
		return $this->sTargetId;
	}

	/**
	 * This method actually generates the output
	 * @param array $params List of parameters
	 * @return string HTML output
	 */
	public function execute( $params = false ) {
		return sprintf( 
			'<a class="bs-tooltip-link" href="#" id="%s">%s</a><div class="bs-tooltip"><ul class="bs-who-heading"><li>'.wfMsg('bs-whoisonline-widget-title').'</li></ul><div id="%s"></div></div>',
			$this->getTargetId(),
			$this->getOption( 'title' ),
			$this->getTargetId().'-target'
		);
	}
}
