<?php
/**
 * Vector - Branch of MonoBook which has many usability improvements and
 * somewhat cleaner code.
 *
 * @todo document
 * @file
 * @ingroup Skins
 */

if( !defined( 'MEDIAWIKI' ) ) {
	die( -1 );
}

/**
 * SkinTemplate class for Vector skin
 * @ingroup Skins
 */
class SkinVector extends SkinTemplate {

	/* Functions */
	var $skinname = 'vector', $stylename = 'vector',
		$template = 'VectorTemplate', $useHeadElement = true;

	/**
	 * Initializes output page and sets up skin-specific parameters
	 * @param $out OutputPage object to initialize
	 */
	public function initPage( OutputPage $out ) {
		global $wgLocalStylePath, $wgRequest, $wgStylePath;

		parent::initPage( $out );
		// Append CSS which includes IE only behavior fixes for hover support -
		// this is better than including this in a CSS fille since it doesn't
		// wait for the CSS file to load before fetching the HTC file.
		$min = $wgRequest->getFuzzyBool( 'debug' ) ? '' : '.min';
		$out->addHeadItem( 'csshover',
			'<!--[if lt IE 7]><style type="text/css">body{behavior:url("' .
				htmlspecialchars( $wgLocalStylePath ) .
				"/{$this->stylename}/csshover{$min}.htc\")}</style><![endif]-->"
		);

		BsExtensionManager::loadAllScriptFiles();
		BsExtensionManager::loadAllStyleSheets();
		$out->addHeadItem('BsScripts', BsStyleManager::getOutput());
		$out->addHeadItem('BsStyle', '<link rel="stylesheet" type="text/css" href="'.htmlspecialchars( $wgStylePath ) .'/'.$this->stylename.'/main.css" media="screen, projection" />');
	}

	/**
	 * Load skin and user CSS files in the correct order
	 * fixes bug 22916
	 * @param $out OutputPage object
	 */
	function setupSkinUserCss( OutputPage $out ){
		parent::setupSkinUserCss( $out );
		$out->addModuleStyles( 'skins.vector' );
	}

	/**
	 * Builds a structured array of links used for tabs and menus
	 * @return array
	 * @private
	 */
	function buildNavigationUrls() {
		global $wgContLang, $wgLang, $wgOut, $wgUser, $wgRequest, $wgArticle;
		global $wgDisableLangConversion, $wgVectorUseIconWatch;

		wfProfileIn( __METHOD__ );

		$links = array(
			'namespaces' => array(),
			'views' => array(),
			'actions' => array(),
			'variants' => array()
		);

		// Detects parameters
		$action = $wgRequest->getVal( 'action', 'view' );
		$section = $wgRequest->getVal( 'section' );

		$userCanRead = $this->mTitle->userCanRead();

		// Checks if page is some kind of content
		if( $this->iscontent ) {
			// Gets page objects for the related namespaces
			$subjectPage = $this->mTitle->getSubjectPage();
			$talkPage = $this->mTitle->getTalkPage();

			// Determines if this is a talk page
			$isTalk = $this->mTitle->isTalkPage();

			// Generates XML IDs from namespace names
			$subjectId = $this->mTitle->getNamespaceKey( '' );

			if ( $subjectId == 'main' ) {
				$talkId = 'talk';
			} else {
				$talkId = "{$subjectId}_talk";
			}

			// Adds namespace links
			$links['namespaces'][$subjectId] = $this->tabAction(
				$subjectPage, 'nstab-' . $subjectId, !$isTalk, '', $userCanRead
			);
			$links['namespaces'][$subjectId]['context'] = 'subject';
			$links['namespaces'][$talkId] = $this->tabAction(
				$talkPage, 'talk', $isTalk, '', $userCanRead
			);
			$links['namespaces'][$talkId]['context'] = 'talk';

			// Adds view view link
			if ( $this->mTitle->exists() && $userCanRead ) {
				$links['views']['view'] = $this->tabAction(
					$isTalk ? $talkPage : $subjectPage,
						'vector-view-view', ( $action == 'view' || $action == 'purge' ), '', true
				);
			}

			wfProfileIn( __METHOD__ . '-edit' );

			// Checks if user can...
			if (
				// read and edit the current page
				$userCanRead && $this->mTitle->quickUserCan( 'edit' ) &&
				(
					// if it exists
					$this->mTitle->exists() ||
					// or they can create one here
					$this->mTitle->quickUserCan( 'create' )
				)
			) {
				// Builds CSS class for talk page links
				$isTalkClass = $isTalk ? ' istalk' : '';

				// Determines if we're in edit mode
				$selected = (
					( $action == 'edit' || $action == 'submit' ) &&
					( $section != 'new' )
				);
				$links['views']['edit'] = array(
					'class' => ( $selected ? 'selected' : '' ) . $isTalkClass,
					'text' => $this->mTitle->exists()
						? wfMsg( 'vector-view-edit' )
						: wfMsg( 'vector-view-create' ),
					'href' =>
						$this->mTitle->getLocalURL( $this->editUrlOptions() )
				);
				// Checks if this is a current rev of talk page and we should show a new
				// section link
				if ( ( $isTalk && $wgArticle && $wgArticle->isCurrent() ) || ( $wgOut->showNewSectionLink() ) ) {
					// Checks if we should ever show a new section link
					if ( !$wgOut->forceHideNewSectionLink() ) {
						// Adds new section link
						//$links['actions']['addsection']
						$links['views']['addsection'] = array(
							'class' => 'collapsible ' . ( $section == 'new' ? 'selected' : false ),
							'text' => wfMsg( 'vector-action-addsection' ),
							'href' => $this->mTitle->getLocalURL(
								'action=edit&section=new'
							)
						);
					}
				}
			// Checks if the page has some kind of viewable content
			} elseif ( $this->mTitle->hasSourceText() && $userCanRead ) {
				// Adds view source view link
				$links['views']['viewsource'] = array(
					'class' => ( $action == 'edit' ) ? 'selected' : false,
					'text' => wfMsg( 'vector-view-viewsource' ),
					'href' =>
						$this->mTitle->getLocalURL( $this->editUrlOptions() )
				);
			}
			wfProfileOut( __METHOD__ . '-edit' );

			wfProfileIn( __METHOD__ . '-live' );

			// Checks if the page exists
			if ( $this->mTitle->exists() && $userCanRead ) {
				// Adds history view link
				$links['views']['history'] = array(
					'class' => 'collapsible ' . ( ( $action == 'history' ) ? 'selected' : false ),
					'text' => wfMsg( 'vector-view-history' ),
					'href' => $this->mTitle->getLocalURL( 'action=history' ),
					'rel' => 'archives',
				);

				if( $wgUser->isAllowed( 'delete' ) ) {
					$links['actions']['delete'] = array(
						'class' => ( $action == 'delete' ) ? 'selected' : false,
						'text' => wfMsg( 'vector-action-delete' ),
						'href' => $this->mTitle->getLocalURL( 'action=delete' )
					);
				}
				if ( $this->mTitle->quickUserCan( 'move' ) ) {
					$moveTitle = SpecialPage::getTitleFor(
						'Movepage', $this->thispage
					);
					$links['actions']['move'] = array(
						'class' => $this->mTitle->isSpecial( 'Movepage' ) ?
										'selected' : false,
						'text' => wfMsg( 'vector-action-move' ),
						'href' => $moveTitle->getLocalURL()
					);
				}

				if (
					$this->mTitle->getNamespace() !== NS_MEDIAWIKI &&
					$wgUser->isAllowed( 'protect' )
				) {
					if ( !$this->mTitle->isProtected() ) {
						$links['actions']['protect'] = array(
							'class' => ( $action == 'protect' ) ?
											'selected' : false,
							'text' => wfMsg( 'vector-action-protect' ),
							'href' =>
								$this->mTitle->getLocalURL( 'action=protect' )
						);

					} else {
						$links['actions']['unprotect'] = array(
							'class' => ( $action == 'unprotect' ) ?
											'selected' : false,
							'text' => wfMsg( 'vector-action-unprotect' ),
							'href' =>
								$this->mTitle->getLocalURL( 'action=unprotect' )
						);
					}
				}
			} else {
				// article doesn't exist or is deleted
				if (
					$wgUser->isAllowed( 'deletedhistory' ) &&
					$wgUser->isAllowed( 'undelete' )
				) {
					$n = $this->mTitle->isDeleted();
					if( $n ) {
						$undelTitle = SpecialPage::getTitleFor( 'Undelete' );
						$links['actions']['undelete'] = array(
							'class' => false,
							'text' => wfMsgExt(
								'vector-action-undelete',
								array( 'parsemag' ),
								$wgLang->formatNum( $n )
							),
							'href' => $undelTitle->getLocalURL(
								'target=' . urlencode( $this->thispage )
							)
						);
					}
				}

				if (
					$this->mTitle->getNamespace() !== NS_MEDIAWIKI &&
					$wgUser->isAllowed( 'protect' )
				) {
					if ( !$this->mTitle->getRestrictions( 'create' ) ) {
						$links['actions']['protect'] = array(
							'class' => ( $action == 'protect' ) ?
											'selected' : false,
							'text' => wfMsg( 'vector-action-protect' ),
							'href' =>
								$this->mTitle->getLocalURL( 'action=protect' )
						);

					} else {
						$links['actions']['unprotect'] = array(
							'class' => ( $action == 'unprotect' ) ?
											'selected' : false,
							'text' => wfMsg( 'vector-action-unprotect' ),
							'href' =>
								$this->mTitle->getLocalURL( 'action=unprotect' )
						);
					}
				}
			}
			wfProfileOut( __METHOD__ . '-live' );
			/**
			 * The following actions use messages which, if made particular to
			 * the Vector skin, would break the Ajax code which makes this
			 * action happen entirely inline. Skin::makeGlobalVariablesScript
			 * defines a set of messages in a javascript object - and these
			 * messages are assumed to be global for all skins. Without making
			 * a change to that procedure these messages will have to remain as
			 * the global versions.
			 */
			// Checks if the user is logged in
			if ( $this->loggedin ) {
				if ( $wgVectorUseIconWatch ) {
					$class = 'icon';
					$place = 'views';
				} else {
					$class = '';
					$place = 'actions';
				}
				$mode = $this->mTitle->userIsWatching() ? 'unwatch' : 'watch';
				$links[$place][$mode] = array(
					'class' => $class . ( ( $action == 'watch' || $action == 'unwatch' ) ? ' selected' : false ),
					'text' => wfMsg( $mode ), // uses 'watch' or 'unwatch' message
					'href' => $this->mTitle->getLocalURL( 'action=' . $mode )
				);
			}
			// This is instead of SkinTemplateTabs - which uses a flat array
			wfRunHooks( 'SkinTemplateNavigation', array( &$this, &$links ) );
		
		// If it's not content, it's got to be a special page
		} else {
			$links['namespaces']['special'] = array(
				'class' => 'selected',
				'text' => wfMsg( 'nstab-special' ),
				'href' => $wgRequest->getRequestURL()
			);
			// Equiv to SkinTemplateBuildContentActionUrlsAfterSpecialPage
			wfRunHooks( 'SkinTemplateNavigation::SpecialPage', array( &$this, &$links ) );
		}

		// Gets list of language variants
		$variants = $wgContLang->getVariants();
		// Checks that language conversion is enabled and variants exist
		if( !$wgDisableLangConversion && count( $variants ) > 1 ) {
			// Gets preferred variant
			$preferred = $wgContLang->getPreferredVariant();
			// Loops over each variant
			foreach( $variants as $code ) {
				// Gets variant name from language code
				$varname = $wgContLang->getVariantname( $code );
				// Checks if the variant is marked as disabled
				if( $varname == 'disable' ) {
					// Skips this variant
					continue;
				}
				// Appends variant link
				$links['variants'][] = array(
					'class' => ( $code == $preferred ) ? 'selected' : false,
					'text' => $varname,
					'href' => $this->mTitle->getLocalURL( '', $code )
				);
			}
		}
		// Equiv to SkinTemplateContentActions
		wfRunHooks( 'SkinTemplateNavigation::Universal', array( &$this,  &$links ) );

		wfProfileOut( __METHOD__ );

		return $links;
	}
}

/**
 * QuickTemplate class for Vector skin
 * @ingroup Skins
 */
class VectorTemplate extends QuickTemplate {

	/* Members */

	/**
	 * @var Cached skin object
	 */
	var $skin;

	/* Functions */
	protected function printViews ( $aViews ) {
		foreach ( $aViews as $oView ) {
			if ( $oView !== null && $oView instanceof ViewBaseElement ) {
				echo $oView->execute();
			} else {
				wfDebugLog( 'BS::Skin', 'BlueSpiceTemplate::printViews: Invalid view.' );
			}
		}
	}

	protected function printBeforeArticleHeadline () {
		global $wgUser, $wgTitle;
		$aViews = array( );
		wfRunHooks( 'BSBlueSpiceSkinBeforeArticleHeadline', array( &$aViews, $wgUser, $wgTitle ) );
		if ( !empty( $aViews ) ) {
			echo '<div id="bs-beforearticleheadline">';
			$this->printViews( $aViews );
			echo '</div>';
		}
	}

	protected function printBeforeArticleContent () {
		global $wgUser, $wgTitle;
		$aViews = array();
		wfRunHooks( 'BSBlueSpiceSkinBeforeArticleContent', array( &$aViews, $wgUser, $wgTitle ) );
		if ( !empty( $aViews ) ) {
			echo '<div id="bs-beforearticlecontent">';
			$this->printViews( $aViews );
			echo '</div>';
		}
		if ( isset($this->data[ 'dataBeforeContent' ]) ) {
			echo '<div id="bs-dataaftercontent">';
			echo $this->html( 'dataBeforeContent' );
			echo '</div>';
		}
	}

	protected function printAdminSidebar () {
		global $wgUser;
		$aViews = array();
		wfRunHooks( 'BSBlueSpiceSkinAdminSidebar', array( &$aViews, $wgUser, $this ) ); // TODO RBV (29.10.10 08:49): For future use

		$oWikiAdminSpecialPageTitle = SpecialPage::getTitleFor( 'SpecialWikiAdmin' );

		$aOut = array( );
		$aOut[ ] = '<div id="p-adminbar" class="bs-nav-links">';
		$aOut[ ] = '  <ul>';

		// CR RBV (27.06.11 14:46): Use hook or event
		if ( class_exists( 'WikiAdmin' ) ) {
			$aRegisteredModules = WikiAdmin::getRegisteredModules();

			foreach ( $aRegisteredModules as $sModuleKey => $aModulParams ) {
				$skeyLower = strtolower( $sModuleKey );
				$sModulLabel = wfMsg( 'bs-' . $skeyLower . '-label' );
				$sUrl = $oWikiAdminSpecialPageTitle->getLocalURL( 'mode=' . $sModuleKey );
				$aPointsAdmin [ $sModulLabel ] = $sUrl;
			}
			ksort( $aPointsAdmin );
			foreach ( $aPointsAdmin as $sModuleLabel => $sUrl ) {
				$aOut[ ] = '    <li><a href="' . $sUrl . '" title="' . $sModuleLabel . '">' . $sModuleLabel . '</a></li>';
			}
		}

		$aOut[ ] = '  </ul>';
		$aOut[ ] = '</div>';

		echo implode( "\n", $aOut );
	}

	protected function printFocusSidebar () {
		global $wgUser;
		$aViews = array();
		wfRunHooks( 'BSBlueSpiceSkinFocusSidebar', array( &$aViews, $wgUser, $this ) );

		$this->printViews( $aViews );
	}

	protected function printNavigationTop () {
		$this->printViews( $this->aNavigationTopViews );
	}

	protected function printAfterArticleContent () {
		global $wgUser, $wgTitle;
		$aViews = array( );
		wfRunHooks( 'BSBlueSpiceSkinAfterArticleContent', array( &$aViews, $wgUser, $wgTitle ) );

		if ( !empty( $aViews ) ) {
			echo '<div id="bs-afterarticlecontent">';
			$this->printViews( $aViews );
			echo '</div>';
		}

		if ( $this->data[ 'dataAfterContent' ] ) {
			echo '<div id="bs-dataaftercontent">';
			echo $this->html( 'dataAfterContent' );
			echo '</div>';
		}
	}

	protected function printSearchBox () {
		$aSearchBoxKeyValues = array();
		wfRunHooks( 'FormDefaults', array( $this, &$aSearchBoxKeyValues ) );

		$aOut = array( );
		$aOut[ ] = '<div id="bs-searchbar">';
		$aOut[ ] = '  <form class="bs-search-form" action="' . $aSearchBoxKeyValues[ 'SearchDestination' ] . '" onsubmit="if (document.getElementById(\'bs-search-input\').value == \'' . $aSearchBoxKeyValues[ 'SearchTextFieldDefaultText' ] . '\') return false;" method="' . $aSearchBoxKeyValues[ 'method' ] . '">';
		if ( isset( $aSearchBoxKeyValues[ 'HiddenFields' ] ) && is_array( $aSearchBoxKeyValues[ 'HiddenFields' ] ) ) {
			foreach ( $aSearchBoxKeyValues[ 'HiddenFields' ] as $key => $value )
				$aOut[ ] = "    <input type=\"hidden\" name=\"$key\" value=\"$value\" />";
		}
		// the first submit button is requested when enter is hit.
		// A hidden default button is defined to catch this event and trigger the action set to default
		// IE 8 compatibility; Input type submit with style=display:none is not submitted by IE8;
		if( substr_count( $_SERVER['HTTP_USER_AGENT'], 'MSIE' ) === 1 ) {
			$aOut[ ] = '    <input type="hidden" name="' . $aSearchBoxKeyValues[ 'DefaultKeyValuePair' ][ 0 ] . '"  value="' . $aSearchBoxKeyValues['DefaultKeyValuePair'][ 1 ] . '" id="bs-search-button-hidden" />';
		}
		else {
			$aOut[ ] = '    <button type="submit" name="' . $aSearchBoxKeyValues[ 'DefaultKeyValuePair' ][ 0 ] . '"  value="' . $aSearchBoxKeyValues['DefaultKeyValuePair'][ 1 ] . '" id="bs-search-button-hidden" style="display: none;"></button>';
		}
		$aOut[ ] = '    <button type="submit" name="' . $aSearchBoxKeyValues[ 'FulltextKeyValuePair' ][ 0 ] . '" value="' . $aSearchBoxKeyValues[ 'FulltextKeyValuePair' ][ 1 ] . '" id="bs-search-fulltext" title="' . $aSearchBoxKeyValues[ 'SubmitButtonFulltext' ] . '" ><span class="bs-extendedsearch-iebuttonworkaround">bs-extendedsearch-scope-text</span></button>';
		$aOut[ ] = '    <button type="submit" name="' . $aSearchBoxKeyValues[ 'TitleKeyValuePair' ][ 0 ] . '" value="' . $aSearchBoxKeyValues[ 'TitleKeyValuePair' ][ 1 ] . '" id="bs-search-button" title="' . $aSearchBoxKeyValues[ 'SubmitButtonTitle' ] . '" ><span class="bs-extendedsearch-iebuttonworkaround">bs-extendedsearch-scope-title</span></button>';
		$aOut[ ] = '    <div id="bs-search-right"></div>';
		$aOut[ ] = '    <input name="' . $aSearchBoxKeyValues[ 'SearchTextFieldName' ] . '" type="text" title="' . $aSearchBoxKeyValues[ 'SearchTextFieldTitle' ] . '" id="bs-search-input" value="' . $aSearchBoxKeyValues[ 'SearchTextFieldDefaultText' ] . '" class="bs-unfocused-textfield bs-autocomplete-field" accesskey="f" />';
		$aOut[ ] = ( isset( $aSearchBoxKeyValues[ 'IdBsSearchLeft' ] ) ) ? $aSearchBoxKeyValues[ 'IdBsSearchLeft' ] : '    <div id="bs-search-left"></div>';
		$aOut[ ] = '  </form>';
		$aOut[ ] = '</div>';

		$out = implode( "\n", $aOut );
		echo $out;
	}

	protected function printContentActionsList () {
		global $wgStylePath;
		wfRunHooks( 'SkinTemplateContentActions', array( &$this->data[ 'content_actions' ] ) );
		wfRunHooks( 'SkinTemplateTabs', array( $this->skin, &$this->data[ 'content_actions' ] ) ); // TODO RBV (12.12.11 12:46): Check for Cross-Version compat. Is there still a $this->skin in MW 1.18 ($this->getSkin()?)?
		$aActionsNotInMoreMenu = array( 'talk', 'edit', 'viewsource', 'history', 'addsection' );
		// Hook to add "not in more menu" and reorder content_actions
		wfRunHooks( 'BlueSpiceSkin:ReorderActionTabs', array( &$this->data[ 'content_actions' ], &$aActionsNotInMoreMenu ) );
		$aOut = array( );
		$aMoreMenuOut = array( );

		foreach ( $this->data[ 'content_actions' ] as $sKey => $aActionTab ) {
			$bIsNamespaceActionTab = strstr( $sKey, 'nstab' ) ? true : false;
			$sCssClass = isset( $aActionTab[ 'class' ] ) ? ' class="' . htmlspecialchars( $aActionTab[ 'class' ] ) . '"' : '';
			if ( in_array( $sKey, $aActionsNotInMoreMenu ) || $bIsNamespaceActionTab ) {
				$aOut[ ] = '<li id="ca-' . Sanitizer::escapeId( $sKey ) . '"' . $sCssClass . '>';
				$aOut[ ] = '  <a href="' . htmlspecialchars( $aActionTab[ 'href' ] ) . '" ' . $aActionTab[ 'attributes' ] . '>';
				$aOut[ ] = htmlspecialchars( $aActionTab[ 'text' ] );
				// Should be secure enough
				$aOut[ ] = $aActionTab[ 'textasrawhtml' ];
				if ( $sKey == 'talk' )
					$aOut[ ] = $this->getDiscussionAmount();
				$aOut[ ] = '  </a>';
				$aOut[ ] = '</li>';
			}
			else {
				$aMoreMenuOut[ ] = '<li id="ca-' . Sanitizer::escapeId( $sKey ) . '"' . $sCssClass . '>';
				$aMoreMenuOut[ ] = '  <a href="' . htmlspecialchars( $aActionTab[ 'href' ] ) . '" ' . $aActionTab[ 'attributes' ] . '>';
				$aMoreMenuOut[ ] = htmlspecialchars( $aActionTab[ 'text' ] );
				$aMoreMenuOut[ ] = '  </a>';
				$aMoreMenuOut[ ] = '</li>';
			}
		}
		if ( !empty( $aMoreMenuOut ) ) {
			$aOut[ ] = '<li id="ca-more">';
			$aOut[ ] = '  <img id="ca-more-arrow" src="' . $this->data['stylepath'] . '/' . $this->data['stylename'] . '/bs-moremenu-less.png" alt="more menu" />';
			$aOut[ ] = '  <ul id="ca-more-menu">';
			$aOut[ ] = implode( "\n", $aMoreMenuOut );
			$aOut[ ] = '  </ul>';
			$aOut[ ] = '</li>';
		}

		echo implode( "\n", $aOut );
	}

	/**
	 * Outputs the entire contents of the XHTML page
	 */
	 protected function printNavigationSidebar () {
		$aOut = array( );
		// TODO RBV (20.10.10 13:58): For future use
		//wfRunHooks( 'SkinTemplateNavigation', array( &$this, &$links ) );

		foreach ( $this->data[ 'sidebar' ] as $bar => $cont ) {
			$sTitle = wfEmptyMsg( $bar, wfMsg( $bar ) ) ? $bar : wfMsg( $bar );

			if ( $cont ) {
				$aOut[ ] = '<div id="p-' . Sanitizer::escapeId( $bar ) . '" class="bs-nav-links">';
				$aOut[ ] = '  <h5>' . $sTitle . '</h5>';
				$aOut[ ] = '  <ul>';
				foreach ( $cont as $key => $val ) {
					$cssClass = !isset( $val[ 'active' ] ) ? ' class="active"' : '';
					$aOut[ ] = '<li id="' . Sanitizer::escapeId( $val[ 'id' ] ) . '"' . $cssClass . '>';
					$aOut[ ] = '  <a href="' . htmlspecialchars( $val[ 'href' ] ) . '">' . htmlspecialchars( $val[ 'text' ] ) . '</a>';
					$aOut[ ] = '</li>';
				}
				$aOut[ ] = '  </ul>';
				$aOut[ ] = '</div>';
			}
		}

		if ( $this->data[ 'language_urls' ] ) {
			$aOut[ ] = '<div title="' . wfMsg( 'otherlanguages' ) . '" id="p-lang" class="bs-widget portal">';
			$aOut[ ] = '  <div class="bs-widget-head">';
			$aOut[ ] = '    <h5 class="bs-widget-title" ' . $this->html( 'userlangattributes' ) . '>' . wfMsg( 'otherlanguages' ) . '</h5>';
			$aOut[ ] = '  </div>';
			$aOut[ ] = '  <div class="bs-widget-body bs-nav-links">';
			$aOut[ ] = '    <ul>';
			foreach ( $this->data[ 'language_urls' ] as $langlink ) {
				$aOut[ ] = '      <li class="' . htmlspecialchars( $langlink[ 'class' ] ) . '">';
				$aOut[ ] = '        <a href="' . htmlspecialchars( $langlink[ 'href' ] ) . '">' . $langlink[ 'text' ] . '</a>';
				$aOut[ ] = '      </li>';
			}
			$aOut[ ] = '    </ul>';
			$aOut[ ] = '  </div>';
			$aOut[ ] = '</div>';
		}

		echo implode( "\n", $aOut );
	}

	protected function printUserBar () {
		global $wgArticle, $wgTitle, $wgUser, $wgScriptPath, $wgStylePath;

		$aOut = array( );

		/* $aBeforeUserBarViews = array();
		  wfRunHooks('BlueSpiceSkin:BeforeUserBar', array( &$aBeforeUserBarViews, $wgUser, $this ));
		  foreach ( $aBeforeUserBarViews as $oUserBarView ) {
		  $aOut[] = $oUserBarView->execute();
		  }
		 */
		if ( $wgUser->isLoggedIn() ) {
			$sButtonUserImage = 'bs-moremenu-less.png';
			$sLoginSwitchTooltip = wfMsg( 'bs-userbar_loginswitch_logout', 'Logout' );

			$aOut[ ] = '<div id="bs-button-logout">';
			$aOut[ ] = '  <a href="' . SpecialPage::getTitleFor('UserLogout')->getLinkUrl() . '" title="' . $sLoginSwitchTooltip . '">';
			$aOut[ ] = '    <img src="' . $this->data['stylepath'] . '/' . $this->data['stylename'] . '/bs-icon-loginswitch-on.png" alt="' . $sLoginSwitchTooltip . '" />';
			$aOut[ ] = '  </a>';
			$aOut[ ] = '</div>';

			$aUserBarBeforeLogoutViews = array( );
			/*BsEventDispatcher::getInstance( 'Skin' )->notify(
				new BsEvent( $this, 'BlueSpiceSkin:UserBarBeforeLogout', array( 'views' => &$aUserBarBeforeLogoutViews, 'user' => $wgUser ) )
			);*/
			wfRunHooks( 'BSBlueSpiceSkinUserBarBeforeLogout', array( &$aUserBarBeforeLogoutViews, $wgUser, $this ) );
			foreach ( $aUserBarBeforeLogoutViews as $oUserBarView ) {
				$aOut[ ] = $oUserBarView->execute();
			}

			$sUserDisplayName = BsAdapterMW::getUserDisplayName( $wgUser );
			$aOut[ ] = '<div id="bs-button-user">';
			$aOut[ ] = '  <img src="' . $this->data['stylepath'] . '/' . $this->data['stylename'] . '/' . $sButtonUserImage . '" alt="' . $sUserDisplayName . '" />';
			$aOut[ ] = '  <ul id="bs-personal-menu">';

			$aPersonalUrlsFilter = array( 'userpage', 'logout', 'anonlogin' );
			foreach ( $this->data[ 'personal_urls' ] as $sKey => $aItem ) {
				if ( in_array( $sKey, $aPersonalUrlsFilter ) )
					continue;
				$sCssClass = $aItem[ 'active' ] ? ' class="active"' : '';
				$aOut[ ] = '<li id="pt-' . Sanitizer::escapeId( $sKey ) . '"' . $sCssClass . '>';
				$sCssClass = !empty( $aItem[ 'class' ] ) ? ' class="' . htmlspecialchars( $aItem[ 'class' ] ) . '"' : '';
				$aOut[ ] = '  <a href="' . htmlspecialchars( $aItem[ 'href' ] ) . '"' . $sCssClass . '>';
				$aOut[ ] = htmlspecialchars( $aItem[ 'text' ] );
				$aOut[ ] = '  </a>';
				$aOut[ ] = '</li>';
			}
			$aOut[ ] = '  </ul>';
			$aOut[ ] = '</div>';

			$sUsernameCSSClass = '';

			if ( $wgTitle->equals( $wgUser->getUserPage() ) ) {
				$sUsernameCSSClass = ' class="active"';
			}

			$aOut[ ] = '<div id="bs-skin-username"' . $sUsernameCSSClass . '>';
			$aOut[ ] = $wgUser->getSkin()->link( $wgUser->getUserPage(), $sUserDisplayName );
			$aOut[ ] = '</div>';
		} else {
			$sButtonUserImage = 'bs-icon-user-transp-50.png';
			$sLoginSwitchTooltip = wfMsg( 'bs-userbar_loginswitch_login', 'Login' );
			$aOut[ ] = '<div id="bs-button-logout">';
			$aOut[ ] = '  <a href="' . SpecialPage::getTitleFor('UserLogin')->escapeLocalURL( array( 'returnto' => $wgTitle->getPrefixedText() ) ) . '" title="' . $sLoginSwitchTooltip . '">';
			$aOut[ ] = '    <img src="' . $this->data['stylepath'] . '/' . $this->data['stylename'] . '/bs-icon-loginswitch-off.png" alt="' . $sLoginSwitchTooltip . '" />';
			$aOut[ ] = '  </a>';
			$aOut[ ] = '</div>';
		}

		echo implode( "\n", $aOut );
	}

		public function onBSWidgetBarGetDefaultWidgets ( &$aViews, $oUser, $oTitle ) {
		if( !isset( $this->data[ 'sidebar' ][ 'TOOLBOX' ] ) ) {
			$aViews[ ] = $this->getToolBoxWidget();
		}
		return true;
	}

	public function onBSWidgetListHelperInitKeyWords ( &$aKeywords, $oTitle ) {
		$aKeywords[ 'TOOLBOX' ] = array( $this, 'getToolBoxWidget' );
		return true;
	}

	public function execute() {
		global $wgRequest, $wgLang, $wgHooks;

		$this->aNavigationTopViews = array();
		wfRunHooks( 'BlueSpiceSkin:NavigationTop', array( &$this->aNavigationTopViews, $this ) );
		$wgHooks[ 'BSWidgetBarGetDefaultWidgets' ][ ] = array( &$this, 'onBSWidgetBarGetDefaultWidgets' );
		$wgHooks[ 'BSWidgetListHelperInitKeyWords' ][ ] = array( &$this, 'onBSWidgetListHelperInitKeyWords' );

		$this->skin = $this->data['skin'];
		$action = $wgRequest->getText( 'action' );

		// Build additional attributes for navigation urls
		$nav = $this->skin->buildNavigationUrls();
		foreach ( $nav as $section => $links ) {
			foreach ( $links as $key => $link ) {
				$xmlID = $key;
				if ( isset( $link['context'] ) && $link['context'] == 'subject' ) {
					$xmlID = 'ca-nstab-' . $xmlID;
				} else if ( isset( $link['context'] ) && $link['context'] == 'talk' ) {
					$xmlID = 'ca-talk';
				} else {
					$xmlID = 'ca-' . $xmlID;
				}
				$nav[$section][$key]['attributes'] =
					' id="' . Sanitizer::escapeId( $xmlID ) . '"';
				if ( $nav[$section][$key]['class'] ) {
					$nav[$section][$key]['attributes'] .=
						' class="' . htmlspecialchars( $link['class'] ) . '"';
					unset( $nav[$section][$key]['class'] );
				}
				// We don't want to give the watch tab an accesskey if the page
				// is being edited, because that conflicts with the accesskey on
				// the watch checkbox.  We also don't want to give the edit tab
				// an accesskey, because that's fairly superfluous and conflicts
				// with an accesskey (Ctrl-E) often used for editing in Safari.
				if (
					in_array( $action, array( 'edit', 'submit' ) ) &&
					in_array( $key, array( 'edit', 'watch', 'unwatch' ) )
				) {
					$nav[$section][$key]['key'] =
						$this->skin->tooltip( $xmlID );
				} else {
					$nav[$section][$key]['key'] =
						$this->skin->tooltipAndAccesskey( $xmlID );
				}
			}
		}
		$this->data['namespace_urls'] = $nav['namespaces'];
		$this->data['view_urls'] = $nav['views'];
		$this->data['action_urls'] = $nav['actions'];
		$this->data['variant_urls'] = $nav['variants'];
		// Build additional attributes for personal_urls
		foreach ( $this->data['personal_urls'] as $key => $item) {
			$this->data['personal_urls'][$key]['attributes'] =
				' id="' . Sanitizer::escapeId( "pt-$key" ) . '"';
			if ( isset( $item['active'] ) && $item['active'] ) {
				$this->data['personal_urls'][$key]['attributes'] .=
					' class="active"';
			}
			$this->data['personal_urls'][$key]['key'] =
				$this->skin->tooltipAndAccesskey('pt-'.$key);
		}

		// Generate additional footer links
		$footerlinks = $this->data["footerlinks"];
		
		// Reduce footer links down to only those which are being used
		$validFooterLinks = array();
		foreach( $footerlinks as $category => $links ) {
			$validFooterLinks[$category] = array();
			foreach( $links as $link ) {
				if( isset( $this->data[$link] ) && $this->data[$link] ) {
					$validFooterLinks[$category][] = $link;
				}
			}
		}
		
		// Generate additional footer icons
		$footericons = $this->data["footericons"];
		// Unset any icons which don't have an image
		foreach ( $footericons as $footerIconsKey => &$footerIconsBlock ) {
			foreach ( $footerIconsBlock as $footerIconKey => $footerIcon ) {
				if ( !is_string($footerIcon) && !isset($footerIcon["src"]) ) {
					unset($footerIconsBlock[$footerIconKey]);
				}
			}
		}
		// Redo removal of any empty blocks
		foreach ( $footericons as $footerIconsKey => &$footerIconsBlock ) {
			if ( count($footerIconsBlock) <= 0 ) {
				unset($footericons[$footerIconsKey]);
			}
		}
		
		// Reverse horizontally rendered navigation elements
		if ( $wgLang->isRTL() ) {
			$this->data['view_urls'] =
				array_reverse( $this->data['view_urls'] );
			$this->data['namespace_urls'] =
				array_reverse( $this->data['namespace_urls'] );
			$this->data['personal_urls'] =
				array_reverse( $this->data['personal_urls'] );
		}
		
		// Output HTML Page
		$this->html( 'headelement' );
		
?>
		<div id="mw-page-base" class="noprint"></div>
		<div id="mw-head-base" class="noprint"></div>
		<!-- content -->
		<div id="content"<?php $this->html('specialpageattributes') ?>>
			<a id="top"></a>
			<div id="mw-js-message" style="display:none;"<?php $this->html('userlangattributes') ?>></div>
			<?php if ( $this->data['sitenotice'] ): ?>
			<!-- sitenotice -->
			<div id="siteNotice"><?php $this->html( 'sitenotice' ) ?></div>
			<!-- /sitenotice -->
			<?php endif; ?>
			<!-- firstHeading -->
			<?php $this->printBeforeArticleHeadline(); ?>
			<h1 id="firstHeading" class="firstHeading"><?php $this->html( 'title' ) ?></h1>
			<?php $this->printBeforeArticleContent(); ?>
			<!-- /firstHeading -->
			<!-- bodyContent -->
			<div id="bodyContent">
				<!-- tagline -->
				<div id="siteSub"><?php $this->msg( 'tagline' ) ?></div>
				<!-- /tagline -->
				<!-- subtitle -->
				<div id="contentSub"<?php $this->html('userlangattributes') ?>><?php $this->html( 'subtitle' ) ?></div>
				<!-- /subtitle -->
				<?php if ( $this->data['undelete'] ): ?>
				<!-- undelete -->
				<div id="contentSub2"><?php $this->html( 'undelete' ) ?></div>
				<!-- /undelete -->
				<?php endif; ?>
				<?php if($this->data['newtalk'] ): ?>
				<!-- newtalk -->
				<div class="usermessage"><?php $this->html( 'newtalk' )  ?></div>
				<!-- /newtalk -->
				<?php endif; ?>
				<?php if ( $this->data['showjumplinks'] ): ?>
				<!-- jumpto -->
				<div id="jump-to-nav">
					<?php $this->msg( 'jumpto' ) ?> <a href="#mw-head"><?php $this->msg( 'jumptonavigation' ) ?></a>,
					<a href="#p-search"><?php $this->msg( 'jumptosearch' ) ?></a>
				</div>
				<!-- /jumpto -->
				<?php endif; ?>
				<!-- bodytext -->
				<?php $this->html( 'bodytext' ) ?>
				<!-- /bodytext -->
				<?php if ( $this->data['catlinks'] ): ?>
				<!-- catlinks -->
				<?php //$this->html( 'catlinks' ); ?>
				<!-- /catlinks -->
				<?php endif; ?>
				<?php if ( $this->data['dataAfterContent'] ): ?>
				<!-- dataAfterContent -->
				<?php $this->html( 'dataAfterContent' ); ?>
				<!-- /dataAfterContent -->
				<?php endif; ?>
				<div class="visualClear"></div>
			</div>
			<?php $this->printAfterArticleContent(); ?>
			<!-- /bodyContent -->
		</div>
		<!-- /content -->
		<!-- header -->
		<div id="mw-head" class="noprint">
			<?php $this->renderNavigation( 'PERSONAL' ); ?>
			<div id="left-navigation">
				<?php $this->renderNavigation( array( 'NAMESPACES', 'VARIANTS' ) ); ?>
			</div>
			<div id="right-navigation">
				<?php $this->renderNavigation( array( 'VIEWS', 'ACTIONS' ) ); $this->printSearchBox(); ?>
			</div>
		</div>
		<!-- /header -->
		<!-- panel -->
			<div id="mw-panel" class="noprint">
				<!-- logo -->
					<div id="p-logo"><a style="background-image: url(<?php echo BsConfig::get( 'MW::LogoPath' ) ?>);" href="<?php echo htmlspecialchars( $this->data['nav_urls']['mainpage']['href'] ) ?>" <?php echo $this->skin->tooltipAndAccesskey( 'p-logo' ) ?>></a></div>
				<!-- /logo -->
				<?php $this->printNavigationTop(); ?>
				<div id="bs-nav-sections"> <?php // TODO RBV (02.11.10 11:36): encapsulate creation of left navigation. Maybe views?   ?>
								<ul id ="bs-nav-tabs">
									<li>
										<a href="#bs-nav-section-navigation"><?php echo wfMsg( 'bs-tab_navigation', 'Navigation' ); ?></a>
									</li>
									<?php global $wgUser; if ( $wgUser->isLoggedIn() ) { ?>
										<li>
											<a href="#bs-nav-section-focus"><?php echo wfMsg( 'bs-tab_focus', 'Focus' ); ?></a>
										</li>
										<?php if ( $wgUser->isAllowed( 'wikiadmin' ) || $wgUser->isAllowed( 'useradmin' ) || $wgUser->isAllowed( 'editadmin' ) ) { ?>
											<li>
												<a href="#bs-nav-section-admin"><?php echo wfMsg( 'bs-tab_admin', 'Admin' ); ?></a>
											</li>
										<?php }
									} ?>
								</ul>
								<div id="bs-nav-section-navigation">
									<!-- Navigation-Code -->
									<?php $this->printNavigationSidebar(); ?>
									<?php //(10.05.2012) PW: Use Mediawiki TOOLBOX here
										if(isset($this->data['sidebar']['TOOLBOX'])) $this->renderPortals( array($this->data['sidebar']['TOOLBOX']) ); ?>
								</div>
								<?php if ( $wgUser->isLoggedIn() ) { ?>
									<div id="bs-nav-section-focus">
										<!-- Focus-Code -->
										<?php $this->printFocusSidebar(); ?>
									</div>
									<?php if ( $wgUser->isAllowed( 'wikiadmin' ) || $wgUser->isAllowed( 'useradmin' ) || $wgUser->isAllowed( 'editadmin' ) ) { ?>
										<div id="bs-nav-section-admin">
											<!-- Admin-Code -->
											<?php $this->printAdminSidebar(); ?>
										</div>
									<?php }
								} ?>
							</div>
			</div>
		<!-- /panel -->
		<!-- footer -->
		<div id="footer"<?php $this->html('userlangattributes') ?>>
			<?php foreach( $validFooterLinks as $category => $links ): ?>
				<?php if ( count( $links ) > 0 ): ?>
				<ul id="footer-<?php echo $category ?>">
					<?php foreach( $links as $link ): ?>
						<?php if( isset( $this->data[$link] ) && $this->data[$link] ): ?>
						<li id="footer-<?php echo $category ?>-<?php echo $link ?>"><?php $this->html( $link ) ?></li>
						<?php endif; ?>
					<?php endforeach; ?>
				</ul>
				<?php endif; ?>
			<?php endforeach; ?>
<?php			if ( count( $footericons ) > 0 ): ?>
				<ul id="footer-icons" class="noprint">
<?php			foreach ( $footericons as $blockName => $footerIcons ): ?>
					<li id="footer-<?php echo htmlspecialchars($blockName); ?>ico">
<?php				foreach ( $footerIcons as $icon ): ?>
						<?php echo $this->skin->makeFooterIcon( $icon ); ?>

<?php				endforeach; ?>
					</li>
<?php			endforeach; ?>
				<li class="footer-poweredbyico"><a href="http://www.blue-spice.org" target="_blank"><img src="bluespice-skin/helpdesk/bs-poweredby_bluespice_88x31.png" alt="BlueSpice for MediaWiki" /></a></li>
				</ul>
			<?php endif; ?>
			<div style="clear:both"></div>
		</div>
		<!-- /footer -->
		<?php $this->html( 'bottomscripts' ); /* JS call to runBodyOnloadHook */ ?>
		<!-- fixalpha -->
		<?php echo BsScriptManager::getOutput(); ?>
		<script type="<?php $this->text( 'jsmimetype' ) ?>" src="<?php $this->text( 'stylepath' ) ?>/<?php $this->text( 'stylename' ) ?>/main.js"></script>
		<!-- /fixalpha -->
		<?php $this->html( 'reporttime' ) ?>
		<?php if ( $this->data['debug'] ): ?>
		<!-- Debug output: <?php $this->text( 'debug' ); ?> -->
		<?php endif; ?>
	</body>
</html>
<?php
	}

	/**
	 * Render a series of portals
	 */
	private function renderPortals( $portals ) {
		// Force the rendering of the following portals
		if ( !isset( $portals['SEARCH'] ) ) $portals['SEARCH'] = true;
		if ( !isset( $portals['TOOLBOX'] ) ) $portals['TOOLBOX'] = true;
		if ( !isset( $portals['LANGUAGES'] ) ) $portals['LANGUAGES'] = true;
		// Render portals
		foreach ( $portals as $name => $content ) {
			echo "\n<!-- {$name} -->\n";
			switch( $name ) {
				case 'SEARCH':
					break;
				case 'TOOLBOX':
?>
<div class="bs-nav-links" id="p-tb">
	<h5<?php $this->html('userlangattributes') ?>><?php $this->msg( 'toolbox' ) ?></h5>
	<div class="body">
		<ul>
		<?php if( $this->data['notspecialpage'] ): ?>
			<li id="t-whatlinkshere"><a href="<?php echo htmlspecialchars( $this->data['nav_urls']['whatlinkshere']['href'] ) ?>"<?php echo $this->skin->tooltipAndAccesskey( 't-whatlinkshere' ) ?>><?php $this->msg( 'whatlinkshere' ) ?></a></li>
			<?php if( $this->data['nav_urls']['recentchangeslinked'] ): ?>
			<li id="t-recentchangeslinked"><a href="<?php echo htmlspecialchars( $this->data['nav_urls']['recentchangeslinked']['href'] ) ?>"<?php echo $this->skin->tooltipAndAccesskey( 't-recentchangeslinked' ) ?>><?php $this->msg( 'recentchangeslinked-toolbox' ) ?></a></li>
			<?php endif; ?>
		<?php endif; ?>
		<?php if( isset( $this->data['nav_urls']['trackbacklink'] ) ): ?>
		<li id="t-trackbacklink"><a href="<?php echo htmlspecialchars( $this->data['nav_urls']['trackbacklink']['href'] ) ?>"<?php echo $this->skin->tooltipAndAccesskey( 't-trackbacklink' ) ?>><?php $this->msg( 'trackbacklink' ) ?></a></li>
		<?php endif; ?>
		<?php if( $this->data['feeds']): ?>
		<li id="feedlinks">
			<?php foreach( $this->data['feeds'] as $key => $feed ): ?>
			<a id="<?php echo Sanitizer::escapeId( "feed-$key" ) ?>" href="<?php echo htmlspecialchars( $feed['href'] ) ?>" rel="alternate" type="application/<?php echo $key ?>+xml" class="feedlink"<?php echo $this->skin->tooltipAndAccesskey( 'feed-' . $key ) ?>><?php echo htmlspecialchars( $feed['text'] ) ?></a>
			<?php endforeach; ?>
		</li>
		<?php endif; ?>
		<?php foreach( array( 'contributions', 'log', 'blockip', 'emailuser', 'upload', 'specialpages' ) as $special ): ?>
			<?php if( $this->data['nav_urls'][$special]): ?>
			<li id="t-<?php echo $special ?>"><a href="<?php echo htmlspecialchars( $this->data['nav_urls'][$special]['href'] ) ?>"<?php echo $this->skin->tooltipAndAccesskey( 't-' . $special ) ?>><?php $this->msg( $special ) ?></a></li>
			<?php endif; ?>
		<?php endforeach; ?>
		<?php if( !empty( $this->data['nav_urls']['print']['href'] ) ): ?>
		<li id="t-print"><a href="<?php echo htmlspecialchars( $this->data['nav_urls']['print']['href'] ) ?>" rel="alternate"<?php echo $this->skin->tooltipAndAccesskey( 't-print' ) ?>><?php $this->msg( 'printableversion' ) ?></a></li>
		<?php endif; ?>
		<?php if (  !empty(  $this->data['nav_urls']['permalink']['href'] ) ): ?>
		<li id="t-permalink"><a href="<?php echo htmlspecialchars( $this->data['nav_urls']['permalink']['href'] ) ?>"<?php echo $this->skin->tooltipAndAccesskey( 't-permalink' ) ?>><?php $this->msg( 'permalink' ) ?></a></li>
		<?php elseif ( $this->data['nav_urls']['permalink']['href'] === '' ): ?>
		<li id="t-ispermalink"<?php echo $this->skin->tooltip( 't-ispermalink' ) ?>><?php $this->msg( 'permalink' ) ?></li>
		<?php endif; ?>
		<?php wfRunHooks( 'SkinTemplateToolboxEnd', array( &$this ) ); ?>
		</ul>
	</div>
</div>
<?php
					break;
				case 'LANGUAGES':
					if ( $this->data['language_urls'] ) {
?>
<div class="portal" id="p-lang">
	<h5<?php $this->html('userlangattributes') ?>><?php $this->msg( 'otherlanguages' ) ?></h5>
	<div class="body">
		<ul>
		<?php foreach ( $this->data['language_urls'] as $langlink ): ?>
			<li class="<?php echo htmlspecialchars(  $langlink['class'] ) ?>"><a href="<?php echo htmlspecialchars( $langlink['href'] ) ?>" title="<?php echo htmlspecialchars( $langlink['title'] ) ?>"><?php echo $langlink['text'] ?></a></li>
		<?php endforeach; ?>
		</ul>
	</div>
</div>
<?php
					}
					break;
				default:
?>
<div class="portal" id='<?php echo Sanitizer::escapeId( "p-$name" ) ?>'<?php echo $this->skin->tooltip( 'p-' . $name ) ?>>
	<h5<?php $this->html('userlangattributes') ?>><?php $out = wfMsg( $name ); if ( wfEmptyMsg( $name, $out ) ) echo htmlspecialchars( $name ); else echo htmlspecialchars( $out ); ?></h5>
	<div class="body">
		<?php if ( is_array( $content ) ): ?>
		<ul>
		<?php foreach( $content as $val ): ?>
			<li id="<?php echo Sanitizer::escapeId( $val['id'] ) ?>"<?php if ( $val['active'] ): ?> class="active" <?php endif; ?>><a href="<?php echo htmlspecialchars( $val['href'] ) ?>"<?php echo $this->skin->tooltipAndAccesskey( $val['id'] ) ?>><?php echo htmlspecialchars( $val['text'] ) ?></a></li>
		<?php endforeach; ?>
		</ul>
		<?php else: ?>
		<?php echo $content; /* Allow raw HTML block to be defined by extensions */ ?>
		<?php endif; ?>
	</div>
</div>
<?php
				break;
			}
			echo "\n<!-- /{$name} -->\n";
		}
	}

	/**
	 * Render one or more navigations elements by name, automatically reveresed
	 * when UI is in RTL mode
	 */
	private function renderNavigation( $elements ) {
		global $wgVectorUseSimpleSearch, $wgVectorShowVariantName, $wgUser;

		// If only one element was given, wrap it in an array, allowing more
		// flexible arguments
		if ( !is_array( $elements ) ) {
			$elements = array( $elements );
		// If there's a series of elements, reverse them when in RTL mode
		} else if ( wfUILang()->isRTL() ) {
			$elements = array_reverse( $elements );
		}
		// Render elements
		foreach ( $elements as $name => $element ) {
			echo "\n<!-- {$name} -->\n";
			switch ( $element ) {
				case 'NAMESPACES':
?>
<div id="p-namespaces" class="vectorTabs<?php if ( count( $this->data['namespace_urls'] ) == 0 ) echo ' emptyPortlet'; ?>">
	<h5><?php $this->msg('namespaces') ?></h5>
	<ul<?php $this->html('userlangattributes') ?>>
		<?php foreach ($this->data['namespace_urls'] as $link ): ?>
			<li <?php echo $link['attributes'] ?>><span><a href="<?php echo htmlspecialchars( $link['href'] ) ?>" <?php echo $link['key'] ?>><?php echo htmlspecialchars( $link['text'] ) ?></a></span></li>
		<?php endforeach; ?>
	</ul>
</div>
<?php
				break;
				case 'VARIANTS':
?>
<div id="p-variants" class="vectorMenu<?php if ( count( $this->data['variant_urls'] ) == 0 ) echo ' emptyPortlet'; ?>">
	<?php if ( $wgVectorShowVariantName ): ?>
		<h4>
		<?php foreach ( $this->data['variant_urls'] as $link ): ?>
			<?php if ( stripos( $link['attributes'], 'selected' ) !== false ): ?>
				<?php echo htmlspecialchars( $link['text'] ) ?>
			<?php endif; ?>
		<?php endforeach; ?>
		</h4>
	<?php endif; ?>
	<h5><span><?php $this->msg('variants') ?></span><a href="#"></a></h5>
	<div class="menu">
		<ul<?php $this->html('userlangattributes') ?>>
			<?php foreach ( $this->data['variant_urls'] as $link ): ?>
				<li<?php echo $link['attributes'] ?>><a href="<?php echo htmlspecialchars( $link['href'] ) ?>" <?php echo $link['key'] ?>><?php echo htmlspecialchars( $link['text'] ) ?></a></li>
			<?php endforeach; ?>
		</ul>
	</div>
</div>
<?php
				break;
				case 'VIEWS':
?>
<div id="p-views" class="vectorTabs<?php if ( count( $this->data['view_urls'] ) == 0 ) echo ' emptyPortlet'; ?>">
	<h5><?php $this->msg('views') ?></h5>
	<ul<?php $this->html('userlangattributes') ?>>
		<?php foreach ( $this->data['view_urls'] as $link ): ?>
			<li<?php echo $link['attributes'] ?>><span><a href="<?php echo htmlspecialchars( $link['href'] ) ?>" <?php echo $link['key'] ?>><?php echo (array_key_exists('img',$link) ?  '<img src="'.$link['img'].'" alt="'.$link['text'].'" />' : htmlspecialchars( $link['text'] ) ) ?></a></span></li>
		<?php endforeach; ?>
	</ul>
</div>
<?php
				break;
				case 'ACTIONS':
?>
<div id="p-cactions" class="vectorMenu<?php if ( count( $this->data['action_urls'] ) == 0 ) echo ' emptyPortlet'; ?>">
	<h5><span><?php $this->msg('actions') ?></span><a href="#"></a></h5>
	<div class="menu">
		<ul<?php $this->html('userlangattributes') ?>>
			<?php foreach ($this->data['action_urls'] as $link ): ?>
				<li<?php echo $link['attributes'] ?>><a href="<?php echo htmlspecialchars( $link['href'] ) ?>" <?php echo $link['key'] ?>><?php echo htmlspecialchars( $link['text'] ) ?></a></li>
			<?php endforeach; ?>
		</ul>
	</div>
</div>
<?php
				break;
				case 'PERSONAL':
?>
<div id="p-personal" class="<?php if ( count( $this->data['personal_urls'] ) == 0 ) echo ' emptyPortlet'; ?>">
	<?php $this->printUserBar(); ?>
</div>
<?php
				break;
			}
			echo "\n<!-- /{$name} -->\n";
		}
	}
}
